import cython
import sys
import os

from typing import TypeVar

from shared_atomic.atomic_decryption_ import ffi
from shared_atomic.atomic_decryption_ import lib as lib_decryption


def sha1(message: bytes) -> str:
    """calculate sha1(internally used)
    :return: calculated sha1
    """
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")
    if lib_decryption.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib_decryption.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join(
            [i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")

if sha1(b'===randomstring===') != '===randomstringsha1===':
    raise ValueError("Didn't pass checksum!")

if sys.platform in ("linux", "darwin", "win32"):
    import shared_atomic.shared_atomic_
    from shared_atomic.shared_atomic_ import lib
    from shared_atomic.shared_atomic_ import ffi

    with open(shared_atomic.shared_atomic_.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomiclibsha1===':
            raise ValueError("Didn't pass checksum2!")

    import shared_atomic.atomic_object
    with open(shared_atomic.atomic_object.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomic_object_libsha1===':
            raise ValueError("Didn't pass checksum3!")

    from shared_atomic.atomic_object import atomic_object
    from shared_atomic.atomic_object import bool_load
    from shared_atomic.atomic_object import bool_get_and_set
    from shared_atomic.atomic_object import bool_compare_and_set_value

    from shared_atomic import mmap_flags

else:
    raise NotImplementedError("Unsupported platform!")

if sys.platform == "win32":
    import platform
    if platform.win32_ver()[0] +'.' + platform.win32_ver()[2] >= '10.SP0':
        from shared_atomic.shared_atomic_win10_ import lib as lib10
    with open(shared_atomic.shared_atomic_win10_.__file__, 'rb') as f:
        shared_atomic_win10_bytes = f.read()
        if sha1(shared_atomic_win10_bytes) != '===atomicwin10libsha1===':
            raise ValueError("Didn't pass checksum2!")
    import shared_atomic.atomic_object
    from shared_atomic.atomic_object import *
    with open(shared_atomic.atomic_object.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomic_object_libsha1===':
            raise ValueError("Didn't pass checksum3!")
    from shared_atomic import mmap_flags

self_atomic_bool = TypeVar("self_atomic_bool", bound="atomic_bool")
self_atomic_float = TypeVar("self_atomic_float", bound="atomic_float")

class atomic_bool(atomic_object):
    """bool type provide atomic operations, it takes 1 byte

    :attributes
        reference: readonly pointer to the bool\n
        value: read/write value of the bool\n
        mode: readonly 's' for single process, 'm' for multiprocessing\n
    """

    mode = cython.declare(str, visibility='readonly')
    windows_unix_compatibility = cython.declare(cython.bint, visibility='readonly')

    def __init__(self, value: cython.bint, mode: str = 'singleprocessing',
                 windows_unix_compatibility: bool = False):
        '''
        constructor of atomic_bool

        :param value: initial value of atomic_bool
        :param mode: 's' or 'singleprocessing' for single process, 'm' or 'multiprocessing' for multiprocessing
        :param windows_unix_compatibility: whether the source code should be compatible with windows x64
        '''
        super().__init__()
        if mode in ('s', 'singleprocessing'):
            self.mode = 's'
            if sys.platform in ('darwin','linux'):
                self.size = 1
                self.reference = ffi.new("_Bool *")
                self.set(value)
            else:
                self.size = 4
                self.reference = ffi.new("long *")
                self.windows_unix_compatibility = False
                self.set(value)

        if mode in ('m', 'multiprocessing'):
            self.mode = 'm'
            if sys.platform in ('darwin', 'linux'):
                self.size = 1
                void_pointer = lib.mmap(ffi.NULL, 1, 3, mmap_flags, -1, 0)
                self.reference = ffi.cast("_Bool *", void_pointer)
                self.set(value)
            elif sys.platform == 'win32':
                self.size = 4
                multiprocessing_win32_init(self, windows_unix_compatibility, 4)
                self.set(value)

    if sys.platform in ('darwin', 'linux'):
        def __del__(self):
            '''
                destructor
            '''
            if self.mode == 'm':
                lib.munmap(self.reference, 1)
    else:
        def __del__(self):
            '''
                destructor
            '''
            multiprocessing_win32_del(self)

    if sys.platform in ('darwin','linux'):
        def get(self) -> cython.bint:
            '''
                getter

            :return: bool value
            '''
            return lib.bool_load(self.reference)

        def set(self, value: cython.bint):
            '''
                setter

            :param value: value to set
            :return: None
            '''
            lib.bool_get_and_set(self.reference, value)

        def bool_store(self, n: self_atomic_bool):
            """Store value atomically

            :param n: another atomic_bool from which value to set
            :return: None
            """
            lib.bool_store(self.reference, n.reference)

        def bool_shift(self, n: self_atomic_bool, r: self_atomic_bool):
            """value exchange between 3 atomic_bools in 2 groups atomically, store n in v after store v in r

            :param v: atomic_bool v
            :param n: atomic_bool n
            :param r: atomic_bool r
            :return: None
            """
            lib.bool_shift(self.reference, n.reference, r.reference)

        def bool_get_and_set(self, n: cython.bint) -> cython.bint:
            """get and set atomically

            :param n: value to set
            :return: original value
            """
            return lib.bool_get_and_set(self.reference, n)

        def bool_compare_and_set(self, e: self_atomic_bool, n: cython.bint) -> cython.bint:
            """Compare and set atomically. This compares the contents of self
            with the contents of e. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            the operation is a read and the current contents of self are written into
            e.

            :param e: pointer of e
            :param n: value to be set
            :return: whether the contents of self and contents of e is the same
            """
            return lib.bool_compare_and_set(self.reference, e.reference, n)

        def bool_compare_and_set_value(self, e: cython.bint, n: cython.bint) -> cython.bint:
            """Compare and set atomically. This compares the contents of self
            with the contents of e. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal, no operation will be taken.

            :param e: The exchange value
            :param n: The value to compare to
            :return: initial value
            """
            #expected = ffi.new("_Bool *", n)
            return lib.bool_compare_and_set_value(self.reference, e, n)
            #return expected[0]

    else:
        def get(self) -> cython.bint:
            '''
                getter

            :return: value
            '''
            return bool_load(self.reference,
                            windows_unix_compatibility=self.windows_unix_compatibility
                            )

        def set(self, value: cython.bint):
            '''
                setter

            :param value: value to set
            :return: None
            '''
            bool_get_and_set(self.reference, value,
                            windows_unix_compatibility=self.windows_unix_compatibility
                            )

        def bool_get_and_set(self, n: cython.bint) -> cython.bint:
            """get and set atomically

            :param n: value to set
            :return: original value
            """
            return bool(bool_get_and_set(self.reference, n,
                            windows_unix_compatibility=self.windows_unix_compatibility
                            ))

        def bool_compare_and_set_value(self, e: cython.bint, n: cython.bint) -> cython.bint:
            """Compare and set atomically. This compares the contents of self
            with the contents of e. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal, no operation will be taken.

            :param e: The exchange value
            :param n: The value to compare to
            :return: initial value
            """
            return bool(bool_compare_and_set_value(self.reference, e, n,
                                              windows_unix_compatibility=self.windows_unix_compatibility

                                              ))

    value = property(fget=get, fset=set, doc="get and set for the bool number.")


if sys.platform in ('linux', 'darwin'):
    class atomic_float(atomic_object):
            """float type provide atomic operations, it takes 8 bytes

            :attributes
                reference: readonly pointer to the float\n
                value: read/write value of the float\n
                mode: readonly 's' for single process, 'm' for multiprocessing, on windows platform, only singleprocessing is supported\n
            """
            mode = cython.declare(str, visibility='readonly')

            def __init__(self, value: cython.double, mode: str = 'singleprocessing'):
                '''
                constructor of atomic_float

                :param value: initial value of atomic_float
                :param mode: 's' or 'singleprocessing' for single process, 'm' or 'multiprocessing' for multiprocessing
                '''
                super().__init__()
                self.size = 8
                if mode in ('m', 'multiprocessing'):
                    self.mode = 'm'
                    void_pointer = lib.mmap(ffi.NULL, 8, 3, mmap_flags, -1, 0)
                    self.reference = ffi.cast("double *", void_pointer)
                elif mode in ('s', 'singleprocessing'):
                    self.mode = 's'
                    self.reference = ffi.new("double *")
                self.set(value)

            def __del__(self):
                '''
                    destructor
                '''
                if self.mode == 'm':
                    lib.munmap(self.reference, 8)

            def get(self) -> cython.double:
                '''
                    getter

                :return: value
                '''
                pointer = ffi.new("double *", 0.0)
                lib.double_store(pointer, self.reference)
                value = ffi.unpack(pointer, 1)[0]
                return value

            def set(self, value: cython.double):
                '''
                    setter

                :param value: value to set
                :return: None
                '''
                pointer = ffi.new("double *", value)
                lib.double_store(self.reference, pointer)

            def float_store(self, n: self_atomic_float):
                """Store value atomically

                 :param n: another atomic_float from which value is from
                 :return: None
                 """
                lib.double_store(self.reference, n.reference)

            value = property(fget=get, fset=set, doc="get and set for the float number.")

