import sys
import os
import cython
from typing import TypeVar

from shared_atomic.atomic_decryption_ import ffi
from shared_atomic.atomic_decryption_ import lib as lib_decryption


def sha1(message: bytes) -> str:
    """calculate sha1(internally used)
    :return: calculated sha1
    """
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")
    if lib_decryption.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib_decryption.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join(
            [i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")


if sha1(b'===randomstring===') != '===randomstringsha1===':
    raise ValueError("Didn't pass checksum1!")

if sys.platform in ("linux", "darwin", "win32"):
    import shared_atomic.shared_atomic_
    from shared_atomic.shared_atomic_ import lib
    from shared_atomic.shared_atomic_ import ffi

    with open(shared_atomic.shared_atomic_.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomiclibsha1===':
            raise ValueError("Didn't pass checksum2!")

    import shared_atomic.atomic_object
    with open(shared_atomic.atomic_object.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomic_object_libsha1===':
            raise ValueError("Didn't pass checksum3!")

    from shared_atomic.atomic_object import *
    from shared_atomic import mmap_flags

else:
    raise NotImplementedError("Unsupported platform!")

if sys.platform == "win32":
    import platform

    if platform.win32_ver()[0] + '.' + platform.win32_ver()[2] >= '10.SP0':
        from shared_atomic.shared_atomic_win10_ import lib as lib10
    with open(shared_atomic.shared_atomic_win10_.__file__, 'rb') as f:
        shared_atomic_win10_bytes = f.read()
        if sha1(shared_atomic_win10_bytes) != '===atomicwin10libsha1===':
            raise ValueError("Didn't pass checksum!")



self_atomic_bytearray = TypeVar("self_atomic_bytearray", bound="atomic_bytearray")


class atomic_bytearray(atomic_object):
    """bytearray provide atomic operations, the bytearray should be no longer than 8 bytes
        :attributes
            encoding: readonly string character set
            size: readonly size of the array
            initial_length: readonly initial input length
            value: read/write value of the bytearray
            int_value: readonly int value
            reference: readonly pointer to the backend array
            mode: readonly 's' for single process, 'm' for multiprocessing, on windows platform, only singleprocessing is supported

    """

    encoding = cython.declare(str, visibility='readonly')
    size = cython.declare(cython.char, visibility='readonly')
    initial_length = cython.declare(cython.char, visibility='readonly')
    mode = cython.declare(str,visibility='readonly')

    def __init__(self, initial: bytes,
                 mode: str = 'singleprocessing',
                 length: int = None,
                 paddingdirection: str = 'right',
                 paddingbytes: bytes = b'\0',
                 trimming_direction: str = 'right',
                 windows_unix_compatibility: bool = False):
        r"""
        constructor to initialize the bytearray,
        the bytearray should be no longer than 8 bytes

        :param initial: initial value of the bytearray, if the initial value is longer than 8 bytes, please specify the trimming target length, or else it would fail.
        :param mode: the mode in which the bytearray will be shared. 'singleprocessing' or 's' for single process, 'multiprocessing' or 'm' for multiprocessing, on windows platform, only singleprocessing is supported, setting it to 'm' or 'multiprocessing' will be ignored.
        :param length: the expected length after padding/trimming for the input value, if not specified, no padding or trimming performed, use original value.
        :param paddingdirection: right, or left side the padding bytes would be added if not specified, pad to the right side, use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
        :param paddingbytes: bytes to pad to the original bytes, by default b'\\0' can be multiple bytes like b'ab', will be padded to the original bytes in circulation until the expected length is reached.
        :param trimming_direction: if initial bytes are longer, on which side the bytes will be trimmed. By default, on the right side, use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
        :param windows_unix_compatibility: whether the source code should be compatible with windows x64

        """
        super().__init__()
        input_length: cython.char = len(initial)
        self.size: cython.char
        self.initial_length: cython.char

        if length is not None:
            if length > 8:
                raise ValueError("length is longer than expected, "
                                 "should be less than 9 bytes!")

        elif input_length > 8 and length is None:
            raise ValueError("input bytearray is longer than expected, "
                             "should be less than 9 bytes!")

        if length is not None:
            if input_length > length:
                if trimming_direction in ('r', 'right'):
                    data = initial[:length]
                elif trimming_direction in ('l', 'left'):
                    data = initial[input_length - length:]

            elif input_length < length:
                devision, remaining = divmod(length - input_length, len(paddingbytes))
                padding = paddingbytes * devision + paddingbytes[:remaining]
                if paddingdirection in ('l', 'left'):
                    data = padding + initial
                elif paddingdirection in ('r', 'right'):
                    data = initial + padding
            else:
                data = initial
        else:
            data = initial

        self.initial_length = len(data)
        if sys.platform in ('darwin', 'linux'):
            if self.initial_length == 1:
                self.size = 1
                self._type = "unsigned char"
                self._array_load = lib.ubyte_load
                self._array_store = lib.ubyte_store
                self._array_get_and_set = lib.ubyte_get_and_set
                self._array_shift = lib.ubyte_shift
                self._array_compare_and_set = lib.ubyte_compare_and_set
                self._array_compare_and_set_value = lib.ubyte_compare_and_set_value
                self._array_add_and_fetch = lib.ubyte_add_and_fetch
                self._array_sub_and_fetch = lib.ubyte_sub_and_fetch
                self._array_and_and_fetch = lib.ubyte_and_and_fetch
                self._array_or_and_fetch = lib.ubyte_or_and_fetch
                self._array_xor_and_fetch = lib.ubyte_xor_and_fetch
                self._array_nand_and_fetch = lib.ubyte_nand_and_fetch
                self._array_fetch_and_add = lib.ubyte_fetch_and_add
                self._array_fetch_and_sub = lib.ubyte_fetch_and_sub
                self._array_fetch_and_and = lib.ubyte_fetch_and_and
                self._array_fetch_and_or = lib.ubyte_fetch_and_or
                self._array_fetch_and_xor = lib.ubyte_fetch_and_xor
                self._array_fetch_and_nand = lib.ubyte_fetch_and_nand

            elif self.initial_length == 2:
                self.size = 2
                self._type = "unsigned short"
                self._array_load = lib.ushort_load
                self._array_store = lib.ushort_store
                self._array_get_and_set = lib.ushort_get_and_set
                self._array_shift = lib.ushort_shift
                self._array_compare_and_set = lib.ushort_compare_and_set
                self._array_compare_and_set_value = lib.ushort_compare_and_set_value
                self._array_add_and_fetch = lib.ushort_add_and_fetch
                self._array_sub_and_fetch = lib.ushort_sub_and_fetch
                self._array_and_and_fetch = lib.ushort_and_and_fetch
                self._array_or_and_fetch = lib.ushort_or_and_fetch
                self._array_xor_and_fetch = lib.ushort_xor_and_fetch
                self._array_nand_and_fetch = lib.ushort_nand_and_fetch
                self._array_fetch_and_add = lib.ushort_fetch_and_add
                self._array_fetch_and_sub = lib.ushort_fetch_and_sub
                self._array_fetch_and_and = lib.ushort_fetch_and_and
                self._array_fetch_and_or = lib.ushort_fetch_and_or
                self._array_fetch_and_xor = lib.ushort_fetch_and_xor
                self._array_fetch_and_nand = lib.ushort_fetch_and_nand

            elif self.initial_length <= 4:
                self.size = 4
                self._type = "unsigned int"
                self._array_load = lib.uint_load
                self._array_store = lib.uint_store
                self._array_get_and_set = lib.uint_get_and_set
                self._array_shift = lib.uint_shift
                self._array_compare_and_set = lib.uint_compare_and_set
                self._array_compare_and_set_value = lib.uint_compare_and_set_value
                self._array_add_and_fetch = lib.uint_add_and_fetch
                self._array_sub_and_fetch = lib.uint_sub_and_fetch
                self._array_and_and_fetch = lib.uint_and_and_fetch
                self._array_or_and_fetch = lib.uint_or_and_fetch
                self._array_xor_and_fetch = lib.uint_xor_and_fetch
                self._array_nand_and_fetch = lib.uint_nand_and_fetch
                self._array_fetch_and_add = lib.uint_fetch_and_add
                self._array_fetch_and_sub = lib.uint_fetch_and_sub
                self._array_fetch_and_and = lib.uint_fetch_and_and
                self._array_fetch_and_or = lib.uint_fetch_and_or
                self._array_fetch_and_xor = lib.uint_fetch_and_xor
                self._array_fetch_and_nand = lib.uint_fetch_and_nand

            elif self.initial_length <= 8:
                self.size = 8
                self._type = "unsigned long long"
                self._array_load = lib.ulonglong_load
                self._array_store = lib.ulonglong_store
                self._array_get_and_set = lib.ulonglong_get_and_set
                self._array_shift = lib.ulonglong_shift
                self._array_compare_and_set = lib.ulonglong_compare_and_set
                self._array_compare_and_set_value = lib.ulonglong_compare_and_set_value
                self._array_add_and_fetch = lib.ulonglong_add_and_fetch
                self._array_sub_and_fetch = lib.ulonglong_sub_and_fetch
                self._array_and_and_fetch = lib.ulonglong_and_and_fetch
                self._array_or_and_fetch = lib.ulonglong_or_and_fetch
                self._array_xor_and_fetch = lib.ulonglong_xor_and_fetch
                self._array_nand_and_fetch = lib.ulonglong_nand_and_fetch
                self._array_fetch_and_add = lib.ulonglong_fetch_and_add
                self._array_fetch_and_sub = lib.ulonglong_fetch_and_sub
                self._array_fetch_and_and = lib.ulonglong_fetch_and_and
                self._array_fetch_and_or = lib.ulonglong_fetch_and_or
                self._array_fetch_and_xor = lib.ulonglong_fetch_and_xor
                self._array_fetch_and_nand = lib.ulonglong_fetch_and_nand
            else:
                raise ValueError("Input bytearray is longer than 8 bytes!")

        elif sys.platform == 'win32':
            if self.initial_length == 1:
                self.size = 1
                self._type = "char"
                self._array_load = byte_load
                self._array_get_and_set = byte_get_and_set
                self._array_compare_and_set_value = byte_compare_and_set_value
                self._array_add_and_fetch = byte_add_and_fetch
                self._array_sub_and_fetch = byte_sub_and_fetch
                self._array_fetch_and_add = byte_fetch_and_add
                self._array_fetch_and_sub = byte_fetch_and_sub
                self._array_fetch_and_and = byte_fetch_and_and
                self._array_fetch_and_or = byte_fetch_and_or
                self._array_fetch_and_xor = byte_fetch_and_xor
            elif self.initial_length == 2:
                self.size = 2
                self._type = "short"
                self._array_load = short_load
                self._array_get_and_set = short_get_and_set
                self._array_compare_and_set_value = short_compare_and_set_value
                self._array_add_and_fetch = short_add_and_fetch
                self._array_sub_and_fetch = short_sub_and_fetch
                self._array_fetch_and_add = short_fetch_and_add
                self._array_fetch_and_sub = short_fetch_and_sub
                self._array_fetch_and_and = short_fetch_and_and
                self._array_fetch_and_or = short_fetch_and_or
                self._array_fetch_and_xor = short_fetch_and_xor
            elif self.initial_length <= 4:
                self.size = 4
                self._type = "long"
                self._array_load = long_load
                self._array_get_and_set = long_get_and_set
                self._array_compare_and_set_value = long_compare_and_set_value
                self._array_add_and_fetch = long_add_and_fetch
                self._array_sub_and_fetch = long_sub_and_fetch
                self._array_fetch_and_add = long_fetch_and_add
                self._array_fetch_and_sub = long_fetch_and_sub
                self._array_fetch_and_and = long_fetch_and_and
                self._array_fetch_and_or = long_fetch_and_or
                self._array_fetch_and_xor = long_fetch_and_xor
                self._array_bittest_and_set = long_bittest_and_set
                self._array_bittest_and_reset = long_bittest_and_reset
            elif self.initial_length <= 8:
                self.size = 8
                self._type = "ssize_t"
                self._array_load = int_load
                self._array_get_and_set = int_get_and_set
                self._array_compare_and_set_value = int_compare_and_set_value
                self._array_add_and_fetch = int_add_and_fetch
                self._array_sub_and_fetch = int_sub_and_fetch
                self._array_fetch_and_add = int_fetch_and_add
                self._array_fetch_and_sub = int_fetch_and_sub
                self._array_fetch_and_and = int_fetch_and_and
                self._array_fetch_and_or = int_fetch_and_or
                self._array_fetch_and_xor = int_fetch_and_xor
                self._array_bittest_and_set = int_bittest_and_set
                self._array_bittest_and_reset = int_bittest_and_reset

            else:
                raise ValueError("Input bytearray is longer than 8 bytes!")

        if mode in ('s', 'singleprocessing'):
            self.mode = 's'
            self.windows_unix_compatibility = False
            self.reference = ffi.new(self._type + " *")
            self.array_get_and_set(data)
        elif mode in ('m', 'multiprocessing'):
            self.mode = 'm'
            if sys.platform in ('darwin', 'linux'):
                void_pointer = lib.mmap(ffi.NULL, self.size, 3, mmap_flags, -1, 0)
                self.reference = ffi.cast(self._type + " *", void_pointer)
                self.array_get_and_set(data)
            else:
                multiprocessing_win32_init(self, windows_unix_compatibility, self.size)
                self.array_get_and_set(data)

        else:
            raise NotImplementedError("Unsupported mode!")

    if sys.platform in ("darwin", "linux"):
        def __del__(self):
            '''
                destructor
            '''
            if self.mode == 'm':
                lib.munmap(self.reference, self.size)
    else:
        def __del__(self):
            '''
                destructor
            '''
            multiprocessing_win32_del(self)

    if sys.platform in ("darwin", "linux"):
        def get_int(self) -> cython.size_t:
            """
            Get the integer representation from the bytearray,
            the whole array would be treated as a large integer

            :return: the integer representation
            """
            return self._array_load(self.reference)

        def get_bytes(self, trim=True) -> bytes:
            r"""
            Get all the bytes from the bytearray atomically
            :param trim: if True, the leading b'\\0' would be trimmed, by default: True

            :return: all the bytes in the bytearray
            """

            result = self._array_load(self.reference)
            if trim:
                result = int.to_bytes(result, length=self.size, byteorder='big').lstrip(b'\0')
            else:
                result = int.to_bytes(result, length=self.size, byteorder='big')
            return result

        def set_bytes(self, data: bytes):
            """
            Set the value in the bytearray,
            if the new data is longer than the original size of the array.
            it will expand the array accordingly which would lose atomicy.
            the size of the bytearray can be check with self.size

            :param data: input bytearray
            :return: None
            """

            desiredlength: cython.char
            integer: cython.size_t

            desiredlength = len(data)
            if 8 >= desiredlength > self.size:
                self.resize(desiredlength)
            elif desiredlength > 8:
                raise ValueError()
            integer = int.from_bytes(data, byteorder='big')
            newpointer = ffi.new(self._type + " *", integer)
            self._array_store(self.reference, newpointer)
            self.initial_length = desiredlength

        def _get_full_bytes(self):
            return self.get_bytes(trim=False)

        def resize(self, newlength: cython.char,
                   paddingdirection: str = 'right',
                   paddingbytes: bytes = b'\0',
                   trimming_direction: str = 'right'):
            r"""
            trim or pad the original contents in the bytearray
            to a new length, the new length should be no longer than 8 bytes,
            the original array wll be replaced with new array, if the original
            array is shared between threads/processes, other threads/processes
            will wouldn't be aware of the change, still use the old bytearray.

            :param newlength: the expected new length of the original bytes.
            :param paddingdirection: if longer than original, left or right sidethe original bytes should be padded, by default right side,use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
            :param paddingbytes: bytes to pad to the original bytes, by default b'\\0' can be multiple bytes like b'ab', will be padded to the original bytes in circulation until the expected length is reached.
            :param trimming_direction: if shorted than original, left or right side the original bytes should be padded,use 'right' or 'r' to specify right side,use 'left' or 'l' to specify the left side.
            :return: None
            """

            self.__init__(self.get_bytes(trim=False), mode=self.mode, length=newlength,
                          paddingdirection=paddingdirection,
                          paddingbytes=paddingbytes,
                          trimming_direction=trimming_direction)

        def array_store(self, i: self_atomic_bytearray):
            """
            Atomically store contents from another bytearray to this bytearray,
            if the other bytearray is different with this one in size , the function will fail.

            :param i: another bytearray to store its value to self
            :return: None
            """
            if self.size != i.size:
                raise ValueError("Input bytearray has different size!")
            self._array_store(self.reference, i.reference)
            self.initial_length = i.initial_length

        def array_get_and_set(self, data: bytes, trim=True) -> bytes:
            r"""
            Get and set atomically

            :param data: new data
            :param trim: whether of not to trim the returning b'\\0' when get, default True

            :return: the original bytes
            """

            integer: cython.size_t

            integer = int.from_bytes(data, byteorder='big')
            result = int.to_bytes(self._array_get_and_set(self.reference, integer),
                                  length=self.size, byteorder='big')
            self.initial_length = len(data)
            return result.lstrip(b'\0') if trim else result

        def array_shift(self, i: self_atomic_bytearray, j: self_atomic_bytearray):
            """
            Value exchange between 3 bytearrays in 2 groups atomically,
            the initial_length field will be updated but not atomically.
            store i in itself after store itself in j

            :param i: one atomic_bytearray
            :param j: another atomic_bytearray
            :return: None
            """
            if self.size != i.size:
                raise ValueError("Comparing bytearray i has different size!")
            if self.size != j.size:
                raise ValueError("Comparing bytearray j has different size!")
            self._array_shift(self.reference,
                              i.reference, j.reference)
            j.initial_length = self.initial_length
            self.initial_length = i.initial_length

        def array_compare_and_set(self, i: self_atomic_bytearray, n: bytes) -> bool:
            """
            Compare and set atomically,This compares the contents of self
            with the contents of i. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            the operation is a read and the current contents of itself are written into
            i.

            :param i: the bytearray to be compared with
            :param n: another bytes to be ready to set to self if comparision return True
            :return: if self is equal to i return True, else return False
            """
            integer: cython.size_t

            if self.size != i.size:
                raise ValueError("Comparing array has different size!")
            integer = int.from_bytes(n, byteorder='big')
            result = self._array_compare_and_set(self.reference,
                                                 i.reference, integer)
            if result:
                self.initial_length = len(n)

            return result

        def array_compare_and_set_value(self, i: bytes, n: bytes, trim=True) -> bytes:
            r"""
            Compare and swap atomically, This compares the contents of self
            with the contents of n. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            No operation will be performed.

            :param i: exchange value
            :param n: The value to be compared with
            :param trim: whether the returned bytes should be trimmed of tailing b'\\0'
            :return: the initial value of the self
            """

            integer_n: cython.size_t
            integer_i: cython.size_t

            if self.size != len(i) or len(i) != len(n):
                raise ValueError("array, i and n has different size!")

            integer_n = int.from_bytes(n, byteorder='big')

            #expected = ffi.new(self._type + " *", integer_n)
            integer_i = int.from_bytes(i, byteorder='big')

            # if length > self.size:
            #    raise ValueError("Input data too large, cannot set atomically!")
            #self._array_compare_and_set(self.reference,
            #                            expected, integer_i)

            result= self.decode(int.to_bytes(self._array_compare_and_set(self.reference,
                                        integer_i, integer_n),
                                            length=self.size, byteorder='big'))
            return result.lstrip(b'\0') if trim else result


        def array_add_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            Increment and fetch atomically

            :param n: bytes will be added to the array.
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_add_and_fetch(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_sub_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            Decrement and fetch atomically

            :param n: bytes will be subtracted from the array.
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_sub_and_fetch(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_and_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            Bitwise AND and fetch the result atomically

            :param n: the other operand of AND operation.
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_and_and_fetch(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_or_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            bitsise OR and fetch the result atomically

            :param n: the other operand of OR operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_or_and_fetch(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_xor_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            bitsise XOR and fetch the result atomically

            :param n: the other operand of XOR operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_xor_and_fetch(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_nand_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            bitsise NAND(AND first then NOT) and fetch the result atomically

            :param n:the other operand of NAND operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_nand_and_fetch(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_fetch_and_add(self, n: bytes, trim=True) -> bytes:
            r"""
            fetch and increment atomically

            :param n: the bytes will be added to the array
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True

            :return: the original contents of the bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_fetch_and_add(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_fetch_and_sub(self, n: bytes, trim=True) -> bytes:
            r"""
            fetch and decrement atomically

            :param n: the bytes will be substracted from the array
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_fetch_and_sub(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_fetch_and_and(self, n: bytes, trim=True) -> bytes:
            r"""
            Fetch then bitwise AND atomically

            :param n: the other operands of AND operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_fetch_and_and(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_fetch_and_or(self, n: bytes, trim=True) -> bytes:
            r"""
            Fetch then bitwise OR atomically

            :param n: the other operands of OR operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_fetch_and_or(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_fetch_and_xor(self, n: bytes, trim=True) -> bytes:
            r"""
            Fetch then bitwise XOR atomically

            :param n: the other operands of XOR operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_fetch_and_xor(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def array_fetch_and_nand(self, n: bytes, trim=True) -> bytes:
            r"""
            fetch then bitwise NAND(AND first then NOT) atomically

            :param n: the other operands of NAND operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """
            integer: cython.size_t

            integer = int.from_bytes(n, byteorder='big')
            result = int.to_bytes(self._array_fetch_and_nand(self.reference, integer),
                                  length=self.size, byteorder='big')
            return result.lstrip(b'\0') if trim else result

        def change_mode(self, newmode='m', windows_unix_compatibility: bool=False):
            """
            Switch between singleprocessing mode and multiprocessing mode,
            the contents will be copied , other threads/processes would not be aware of the change.

            :param newmode: the mode to change to, 'm' or 'multiprocessing' for multiproessing, 's' or 'singleprocessing' for singleprocessing. default 'm'
            :param windows_unix_compatibility: if change to multiprocessing mode, whether the source code should be compatible with windows x64
            :return: None
            """
            if newmode in ('m', 'multiprocessing'):
                if self.mode == 's':
                    data = self.get_bytes(trim=False)
                    void_pointer = lib.mmap(ffi.NULL, self.size, 3, mmap_flags, -1, 0)
                    self.reference = ffi.cast(self._type + " *", void_pointer)
                    self.set_bytes(data)

            elif newmode not in ('s', 'singleprocessing'):
                raise ValueError(
                    "newmode has the wrong value, should be 'm','s','multiprocessing' or 'singleprocessing'")

            elif self.mode == 'm':
                data = self.get_bytes(trim=False)
                self.reference = ffi.new(self._type + " *")
                self.set_bytes(data)
            self.mode = newmode
    else:
        def get_int(self) -> cython.size_t:
            """
            Get the integer representation from the bytearray,
            the whole array would be treated as a large integer

            :return: the integer representation
            """
            return array_get_int(self, self.reference)

        def get_bytes(self, trim=True) -> bytes:
            r"""
            Get all the bytes from the bytearray atomically
            :param trim: if True, the leading b'\\0' would be trimmed, by default: True

            :return: all the bytes in the bytearray
            """
            return array_get_bytes(self, self.reference, trim)

        def set_bytes(self, data: bytes):
            """
            Set the value in the bytearray,
            if the new data is longer than the original size of the array.
            it will expand the array accordingly which would lose atomicy.
            the size of the bytearray can be checked with self.size

            :param data: input bytearray
            :return: None
            """
            array_set_bytes(self, self.reference, data)

        def _get_full_bytes(self):
            return self.get_bytes(trim=False)

        def resize(self, newlength: cython.char,
                   paddingdirection: str = 'right',
                   paddingbytes: bytes = b'\0',
                   trimming_direction: str = 'right'):
            r"""
            trim or pad the original contents in the bytearray
            to a new length, the new length should be no longer than 8 bytes,
            the original array wll be replaced with new array, if the original
            array is shared between threads/processes, other threads/processes
            will wouldn't be aware of the change, still use the old bytearray.

            :param newlength: the expected new length of the original bytes.
            :param paddingdirection: if longer than original, left or right sidethe original bytes should be padded, by default right side,use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
            :param paddingbytes: bytes to pad to the original bytes, by default b'\\0' can be multiple bytes like b'ab', will be padded to the original bytes in circulation until the expected length is reached.
            :param trimming_direction: if shorted than original, left or right side the original bytes should be padded,use 'right' or 'r' to specify right side,use 'left' or 'l' to specify the left side.
            :return: None
            """

            self.__init__(self.get_bytes(trim=False), mode=self.mode, length=newlength,
                          paddingdirection=paddingdirection,
                          paddingbytes=paddingbytes,
                          trimming_direction=trimming_direction,
                          windows_unix_compatibility=self.windows_unix_compatibility)

        def array_bittest_and_set(self, offset: cython.char) -> cython.bint:
            """Tests the specified bit and sets it to 1 atomically

            :param offset: offset in the binary presenation of the bytearray
            :return: the initial value(True or False) of specified bit
            """

            return array_bittest_and_set(self, self.reference, offset)

        def array_bittest_and_reset(self, offset: cython.char) -> cython.bint:
            """Tests the specified bit and sets it to 0 atomically

            :param offset: offset in the binary presenation of the bytearray
            :return: the initial value(True or False) of specified bit
            """

            return array_bittest_and_reset(self, self.reference, offset)

        def array_get_and_set(self, data: bytes, trim=True) -> bytes:
            r"""
            Get and set atomically

            :param data: new data
            :param trim: whether of not to trim the returning b'\\0' when get, default True
            :return: the original bytes
            """

            return array_get_and_set(self, self.reference, data, trim)


        def array_compare_and_set_value(self, i: bytes, n: bytes, trim=True) -> bytes:
            """
            Compare and swap atomically, This compares the contents of self
            with the contents of n. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            No operation will be performed.

            :param i: exchange value
            :param n: The value to be compared with
            :param trim: whether the returned bytes should be trimmed of tailing b'\\0'
            :return: the initial value of the self
            """

            return array_compare_and_set_value(self, self.reference, i, n, trim)

        def array_add_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            Increment and fetch atomically

            :param n: bytes will be added to the array.
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """

            return array_add_and_fetch(self, self.reference, n, trim)

        def array_sub_and_fetch(self, n: bytes, trim=True) -> bytes:
            r"""
            Decrement and fetch atomically

            :param n: bytes will be subtracted from the array.
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the contents of resulted bytearray
            """

            return array_sub_and_fetch(self, self.reference, n, trim)

        def array_fetch_and_add(self, n: bytes, trim=True) -> bytes:
            r"""
            fetch and increment atomically

            :param n: the bytes will be added to the array
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """

            return array_fetch_and_add(self, self.reference, n, trim)

        def array_fetch_and_sub(self, n: bytes, trim=True) -> bytes:
            r"""
            fetch and decrement atomically

            :param n: the bytes will be substracted from the array
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """

            return array_fetch_and_sub(self, self.reference, n, trim)

        def array_fetch_and_and(self, n: bytes, trim=True) -> bytes:
            r"""
            Fetch then bitwise AND atomically

            :param n: the other operands of AND operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """

            return array_fetch_and_and(self, self.reference, n, trim)

        def array_fetch_and_or(self, n: bytes, trim=True) -> bytes:
            r"""
            Fetch then bitwise OR atomically

            :param n: the other operands of OR operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """

            return array_fetch_and_or(self, self.reference, n, trim)

        def array_fetch_and_xor(self, n: bytes, trim=True) -> bytes:
            r"""
            Fetch then bitwise XOR atomically

            :param n: the other operands of XOR operation
            :param trim: whether of not to trim the returning b'\\0' when fetch, default True
            :return: the original contents of the bytearray
            """

            return array_fetch_and_xor(self, self.reference, n, trim)

        def change_mode(self, newmode='m', windows_unix_compatibility: bool=False):
            """
            Switch between singleprocessing mode and multiprocessing mode,
            the contents will be copied , other threads/processes would not be aware of the change.

            :param newmode: the mode to change to, 'm' or 'multiprocessing' for multiproessing, 's' or 'singleprocessing' for singleprocessing. default 'm'
            :param windows_unix_compatibility: if change to multiprocessing mode, whether the source code should be compatible with windows x64
            :return: None
            """

            if newmode in ('m', 'multiprocessing'):
                if self.mode == 's':
                    data: bytes = self.get_bytes(trim=False)
                    multiprocessing_win32_init(self, windows_unix_compatibility, self.size)
                    self.set_bytes(data)

            elif newmode not in ('s', 'singleprocessing'):
                raise ValueError(
                    "newmode has the wrong value, should be 'm','s','multiprocessing' or 'singleprocessing'")

            elif self.mode == 'm':
                data: bytes = self.get_bytes(trim=False)
                self.windows_unix_compatibility = False
                self.reference = ffi.new(self._type + " *")
                self.set_bytes(data)

            self.mode = newmode

    value = property(fget=_get_full_bytes, fset=set_bytes, doc="same with get_bytes without trimming and set_bytes")
    int_value = property(fget=get_int, doc="same with get_int")
