import cython
import sys
import os
from typing import TypeVar

from shared_atomic.atomic_decryption_ import ffi
from shared_atomic.atomic_decryption_ import lib as lib_decryption


def sha1(message: bytes) -> str:
    """calculate sha1(internally used)
    :return: calculated sha1
    """
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")
    if lib_decryption.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib_decryption.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join(
            [i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")


if sha1(b'===randomstring===') != '===randomstringsha1===':
    raise ValueError("Didn't pass checksum1!")

if sys.platform in ("linux", "darwin", "win32"):
    import shared_atomic.shared_atomic_
    from shared_atomic.shared_atomic_ import lib
    from shared_atomic.shared_atomic_ import ffi

    with open(shared_atomic.shared_atomic_.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomiclibsha1===':
            raise ValueError("Didn't pass checksum2!")

    import shared_atomic.atomic_object
    with open(shared_atomic.atomic_object.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomic_object_libsha1===':
            raise ValueError("Didn't pass checksum3!")

    from shared_atomic.atomic_object import *
    from shared_atomic import mmap_flags
else:
    raise NotImplementedError("Unsupported platform!")

if sys.platform == "win32":
    import platform
    if platform.win32_ver()[0] +'.' + platform.win32_ver()[2] >= '10.SP0':
        from shared_atomic.shared_atomic_win10_ import lib as lib10
    with open(shared_atomic.shared_atomic_win10_.__file__, 'rb') as f:
        shared_atomic_win10_bytes = f.read()
        if sha1(shared_atomic_win10_bytes) != '===atomicwin10libsha1===':
            raise ValueError("Didn't pass checksum!")


self_atomic_int = TypeVar("self_atomic_int", bound="atomic_int")

class atomic_int(atomic_object):
    """int provide atomic operations, the int takes 8 bytes
        :attributes
            reference: readonly pointer to the integer\n
            value: read/write value of the integer\n
            mode: readonly 's' for single process, 'm' for multiprocessing, on windows platform, only singleprocessing is supported\n
            windows_unix_compatibility: readonly whether the source code should be compatible with windows x64
    """

    mode = cython.declare(str, visibility='readonly')
    windows_unix_compatibility = cython.declare(cython.bint, visibility='readonly')


    def __init__(self, value: cython.longlong, mode: str = 'singleprocessing',
                 windows_unix_compatibility: bool = False):
        """constructor of atomic_int
            :attributes
                value: read/write value of the integer\n
                mode: 's' or 'singleprocessing' for single process, 'm' or 'multiprocessing' for multiprocessing\n
                windows_unix_compatibility: whether the source code should be compatible with windows x64
        """
        super().__init__()
        self.size = 8
        if mode in ('s', 'singleprocessing'):
            self.mode = 's'
            if sys.platform in ('darwin', 'linux'):
                self.reference = ffi.new("ssize_t *")
                self.set(value)
            else:
                self.reference = ffi.new("long long *")
                self.windows_unix_compatibility = False
                self.set(value)
        elif mode in ('m', 'multiprocessing'):
            self.mode = 'm'
            if sys.platform in ('darwin', 'linux'):
                void_pointer = lib.mmap(ffi.NULL, 8, 3, mmap_flags, -1, 0)
                self.reference = ffi.cast("ssize_t *", void_pointer)
                self.windows_unix_compatibility = False
                self.set(value)
            else:
                multiprocessing_win32_init(self, windows_unix_compatibility, 8)
                self.set(value)
        else:
            raise NotImplementedError("Unsupported mode!")

    if sys.platform in ('darwin', 'linux'):
        def __del__(self):
            '''
                destructor
            '''
            if self.mode == 'm':
                lib.munmap(self.reference, 8)
    else:
        def __del__(self):
            '''
                destructor
            '''
            multiprocessing_win32_del(self)


    if sys.platform in ('darwin', 'linux'):
        def get(self) -> cython.longlong:
            '''
                getter

            :return: value
            '''
            return lib.ssize_t_load(self.reference)

        def set(self, value: cython.longlong):
            '''
                setter

            :param value: value to set
            :return: None
            '''
            lib.ssize_t_get_and_set(self.reference, value)

        def int_store(self, n: self_atomic_int):
            """Store value atomically

             :param n: atomic_int from which value is from
             :return: None
             """
            lib.ssize_t_store(self.reference, n.reference)

        def int_shift(self, n: self_atomic_int, r: self_atomic_int):
            """value exchange between 3 atomic_ints in 2 groups atomically, store n in v after store v in r

            :param n: atomic_int of n
            :param r: atomic_int of r
            :return: None
            """
            return lib.ssize_t_shift(self.reference, n.reference, r.reference)

        def int_and_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """bitwise AND and fetch the result atomically

            :param n: data to AND
            :return: resulted value
            """
            return lib.ssize_t_and_and_fetch(self.reference, n)

        def int_or_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """bitwise XOR and fetch the result atomically

            :param n: data to OR
            :return: resulted value
            """
            return lib.ssize_t_or_and_fetch(self.reference, n)

        def int_xor_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """bitwise XOR and fetch the result atomically

            :param n: data to XOR
            :return: resulted value
            """
            return lib.ssize_t_xor_and_fetch(self.reference, n)

        def int_nand_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """bitwise NAND and fetch the result atomically

            :param n: data to NAND
            :return: resulted value
            """
            return lib.ssize_t_nand_and_fetch(self.reference, n)


        def int_get_and_set(self, n: cython.longlong) -> cython.longlong:
            """get and set atomically

            :param n: value to set
            :return: original value
            """
            return lib.ssize_t_get_and_set(self.reference, n)

        def int_compare_and_set(self, e: self_atomic_int, n: cython.longlong) -> cython.bint:
            """Compare and set atomically. This compares the contents of self
            with the contents of e. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            the operation is a read and the current contents of self are written into e.

            :param e: atomic_int e
            :param n: value to be set
            :return: whether the contents of v and contents of e is the same
            """
            return lib.ssize_t_compare_and_set(self.reference, e.reference, n)

        def int_compare_and_set_value(self, e: cython.longlong, n: cython.longlong) -> cython.longlong:
            """Compare and set atomically. This compares the contents of self
            with the contents of e. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal, no operation will be taken.

            :param e: The exchange value
            :param n: The value to compare to
            :return: initial value
            """
            #expected = ffi.new("ssize_t *", n)
            return lib.ssize_t_compare_and_set_value(self.reference, e, n)
            #return expected[0]

        def int_add_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """increment and fetch atomically

            :param n: data to add
            :return: sum of the 2 values
            """
            return lib.ssize_t_add_and_fetch(self.reference, n)

        def int_sub_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """sub and fetch atomically

            :param n: data to subtract
            :return: difference of the 2 values
            """
            return lib.ssize_t_sub_and_fetch(self.reference, n)



        def int_fetch_and_add(self, n: cython.longlong) -> cython.longlong:
            """fetch and increment atomically

            :param n: data to add
            :return: original value in v
            """
            return lib.ssize_t_fetch_and_add(self.reference, n)

        def int_fetch_and_sub(self, n: cython.longlong) -> cython.longlong:
            """fetch and subtract atomically

            :param n: data to subtract
            :return: original value in v
            """
            return lib.ssize_t_fetch_and_sub(self.reference, n)

        def int_fetch_and_and(self, n: cython.longlong) -> cython.longlong:
            """fetch then bitwise AND atomically

            :param n: data to AND
            :return: the result
            """
            return lib.ssize_t_fetch_and_and(self.reference, n)

        def int_fetch_and_or(self, n: cython.longlong) -> cython.longlong:
            """fetch then bitwise OR atomically

            :param n: data to OR
            :return: the result
            """
            return lib.ssize_t_fetch_and_or(self.reference, n)

        def int_fetch_and_xor(self, n: cython.longlong) -> cython.longlong:
            """fetch then bitwise XOR atomically

            :param n: data to XOR
            :return: the result
            """
            return lib.ssize_t_fetch_and_xor(self.reference, n)

        def int_fetch_and_nand(self, n: cython.longlong) -> cython.longlong:
            """fetch then bitwise NAND atomically

            :param n: data to NAND
            :return: the result
            """
            return lib.ssize_t_fetch_and_nand(self.reference, n)
    else:
        def get(self) -> cython.longlong:
            '''
                getter

            :return: value
            '''
            return int_load(self.reference,
                            windows_unix_compatibility=self.windows_unix_compatibility
                            )

        def set(self, value: cython.longlong):
            '''
                setter

            :param value: value to set
            :return: None
            '''
            int_get_and_set(self.reference, value,
                            windows_unix_compatibility=self.windows_unix_compatibility
                            )

        def int_get_and_set(self, n: cython.longlong) -> cython.longlong:
            """get and set atomically

            :param n: value to set
            :return: original value
            """
            return int_get_and_set(self.reference, n,
                            windows_unix_compatibility=self.windows_unix_compatibility
                            )


        def int_compare_and_set_value(self, e: cython.longlong, n: cython.longlong) -> cython.longlong:
            """Compare and set atomically. This compares the contents of self
            with the contents of e. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal, no operation will be taken.

            :param e: The exchange value
            :param n: The value to compare to
            :return: initial value
            """
            return int_compare_and_set_value(self.reference, e, n,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )


        def int_add_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """increment and fetch atomically

            :param n: data to add
            :return: sum of the 2 values
            """
            return int_add_and_fetch(self.reference, n,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )

        def int_sub_and_fetch(self, n: cython.longlong) -> cython.longlong:
            """sub and fetch atomically

            :param n: data to subtract
            :return: difference of the 2 values
            """
            return int_sub_and_fetch(self.reference, n,
                                            windows_unix_compatibility=self.windows_unix_compatibility
                                            )

        def int_fetch_and_add(self, n: cython.longlong) -> cython.longlong:
            """fetch and increment atomically

            :param n: data to add
            :return: original value
            """
            return int_fetch_and_add(self.reference, n,
                                            windows_unix_compatibility=self.windows_unix_compatibility
                                            )

        def int_fetch_and_sub(self, n: cython.longlong) -> cython.longlong:
            """fetch and subtract atomically

            :param n: data to subtract
            :return: original value
            """
            return int_fetch_and_sub(self.reference, n,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )

        def int_fetch_and_and(self, n: cython.longlong) -> cython.longlong:
            """fetch then bitwise AND atomically

            :param n: data to AND
            :return: the result
            """
            return int_fetch_and_and(self.reference, n,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )

        def int_fetch_and_or(self, n: cython.longlong) -> cython.longlong:
            """fetch then bitwise OR atomically

            :param n: data to OR
            :return: the result
            """
            return int_fetch_and_or(self.reference, n,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )

        def int_fetch_and_xor(self, n: cython.longlong) -> cython.longlong:
            """fetch then bitwise XOR atomically

            :param n: data to XOR
            :return: the result
            """
            return int_fetch_and_xor(self.reference, n,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )

        def int_bittest_and_set(self, offset: cython.longlong) -> cython.bint:
            """Tests the specified bit and set it to 1 atomically

            :param offset: offset in the binary presenation of the 64bit integer
            :return: the initial value(True or False) of specified bit
            """
            return int_bittest_and_set(self.reference, offset,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )

        def int_bittest_and_reset(self, offset: cython.longlong) -> cython.bint:
            """Tests the specified bit and set it to 0 atomically

            :param offset: offset in the binary presenation of the 64bit integer
            :return: the initial value(True or False) of specified bit
            """
            return int_bittest_and_reset(self.reference, offset,
                                             windows_unix_compatibility=self.windows_unix_compatibility
                                             )


    value = property(fget=get, fset=set, doc="get and set for the integer number")
