import traceback
from pathlib import Path
import sys
import logging
from logging.handlers import RotatingFileHandler
logger=logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
if sys.platform == 'linux':
    handler = RotatingFileHandler(Path.joinpath(Path(__file__).parent, 'shared_atomic.log'),maxBytes=18*1024**3, backupCount=5)
else:
    raise NotImplementedError("Unsupported platform")
handler.setFormatter(logging.Formatter('%(name) %(asctime)s %(msecs)d %(levelname) %(process)d %(thread)d %(funcName)s %(lineno)d %(message)s'))
logger.addHandler(handler)

import cython
from shared_atomic.atomic_decryption_ import ffi, lib
import re

# ssllib = str(Path.joinpath(Path(__file__).parent, "lib/libssl.so.3"))
# cryptolib = str(Path.joinpath(Path(__file__).parent, "lib/libcrypto.so.3"))
# ssl1lib = str(Path.joinpath(Path(__file__).parent, "lib/libssl.so.1.1"))
# crypto1lib = str(Path.joinpath(Path(__file__).parent, "lib/libcrypto.so.1.1"))
decryptionlib = str(list(Path.joinpath(Path(__file__).parent, "lib").glob("libdecryption.*"))[0])
libpythonlib = str(list(Path.joinpath(Path(__file__).parent, "lib").glob("libpython*.so.*"))[0])
atomic_decryptionlib = str(list(Path(__file__).parent.glob("atomic_decryption_.*"))[0])

def sha1(message: bytes) -> str:
    """calculate sha1(internally used)
    :return: calculated sha1
    """
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")

    if lib.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")

def base64_decode(cipher_txt: bytes) -> bytes:
    """decode base64(internally used)

    :return: decoded value
    """
    cipher_len: cython.Py_ssize_t = len(cipher_txt)
    cipher_txt_char = ffi.new("unsigned char []", cipher_txt)
    content_len = ffi.new("int *")
    content_char = ffi.new("unsigned char [1000]")
    if lib.base64_decrypt(content_char, content_len, cipher_txt_char, cipher_len) == 1:
        if content_len[0] > 1000:
            content_char = ffi.new("unsigned char []", content_len[0])
            lib.base64_decrypt(content_char, content_len, cipher_txt_char, cipher_len)
        return b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(content_char, content_len[0])])
    else:
        raise ValueError("Checksum failed!")

def base64_encode(content_txt: bytes) -> bytes:
    """encode in  base64(internally used)

    :return: encoded value
    """
    content_len: cython.Py_ssize_t = len(content_txt)
    content_txt_char = ffi.new("unsigned char []", content_txt)
    cipher_len = ffi.new("int *")
    cipher_char = ffi.new("unsigned char [100000]")

    if lib.base64_encrypt(content_txt_char, content_len, cipher_char, cipher_len) == 1:
        if cipher_len[0] > 100000:
            cipher_char = ffi.new("unsigned char []", cipher_len[0])
            lib.base64_encrypt(content_txt_char, content_len, cipher_char, cipher_len)
        return b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(cipher_char, cipher_len[0]-1)])
    else:
        raise ValueError("Checksum failed!")

def main() -> cython.uchar:
    """Used internally to check the hardware information of the host

    :return: 0 if the hardware identification is successfully processed, >0 if not.
    """
    LOCK_SH: cython.char = 1
    LOCK_EX: cython.char = 2
    LOCK_NB: cython.char = 4
    LOCK_UN: cython.char = 8

    def checksum() -> cython.bint:

        if sha1(b'===randomstring===') != '===randomstringsha1===':
            return False

        # with open(ssllib, 'rb') as f:
        #     ssllib_bytes = f.read()
        #     if sha1(ssllib_bytes) != '===ssllibsha1===':
        #         return False
        # with open(cryptolib, 'rb') as f:
        #     cryptolib_bytes = f.read()
        #     if sha1(cryptolib_bytes) != '===cryptolibsha1===':
        #         return False
        # with open(ssl1lib, 'rb') as f:
        #     ssl1lib_bytes = f.read()
        #     if sha1(ssl1lib_bytes) != '===ssl1libsha1===':
        #         return False
        # with open(crypto1lib, 'rb') as f:
        #     crypto1lib_bytes = f.read()
        #     if sha1(crypto1lib_bytes) != '===crypto1libsha1===':
        #         return False
        with open(decryptionlib, 'rb') as f:
            decryptionlib_bytes = f.read()
            if sha1(decryptionlib_bytes) != '===decryptionlibsha1===':
                return False
        with open(atomic_decryptionlib, 'rb') as f:
            atomic_decryptionlib_bytes = f.read()
            if sha1(atomic_decryptionlib_bytes) != '===atomic_decryptionlibsha1===':
                return False
        return True

    system_info = {
        'smbios_type_id': 1,
        'system_manufacturer': None,
        'system_product_name': None,
        'system_serial_number': None,
        'system_uuid': None,
        'system_sku_number': None,
        'system_family': None,
        'smbios_major_version': None,
        'smbios_minor_version': None,
        'smbios_docrev_version': None,
        'baseboard_list': [],
        'bios_list': [],
        'chassis_list': [],
        'cpu_list': [],
        'cpu_cache_list': [],
        'memory_controller_list': [],
        'memory_module_list': [],
        'system_slot_list': [],
        'on_board_device_list': [],
        'on_board_device_extended_list': [],
        'physical_memory_array_list': [],
        'memory_device_set_list': [],
        'memory_device_set_id_list': []
    }

    class bios_info:
        smbios_type_id: cython.uchar = 0
        bios_vendor: str

        def __init__(self, bios_vendor: str):
            self.bios_vendor = bios_vendor

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + f'{self.bios_vendor}'

    class chassis_info:
        smbios_type_id: cython.uchar = 3
        chassis_handle: cython.ushort
        chassis_manufacturer: str
        chassis_type: cython.uchar
        chassis_version: str
        chassis_serial_number: str
        chassis_asset_tag: str
        chassis_oem_defined: cython.uint
        chassis_height: cython.uchar
        chassis_contained_element_count: cython.uchar
        chassis_contained_element_record_length: cython.uchar
        chassis_contained_elements: list
        chassis_sku_number: str

        def __init__(self, chassis_handle: cython.ushort,
                     chassis_manufacturer: str,
                     chassis_type: cython.uchar,
                     chassis_version: str,
                     chassis_serial_number: str,
                     chassis_asset_tag: str,
                     chassis_oem_defined: cython.uint,
                     chassis_height: cython.uchar,
                     chassis_contained_element_count: cython.uchar,
                     chassis_contained_element_record_length: cython.uchar,
                     chassis_contained_elements: list,
                     chassis_sku_number: str,
                     chassis_boardlist: list):
            self.chassis_handle = chassis_handle
            self.chassis_manufacturer = chassis_manufacturer
            self.chassis_type = chassis_type
            self.chassis_version = chassis_version
            self.chassis_serial_number = chassis_serial_number
            self.chassis_asset_tag = chassis_asset_tag
            self.chassis_oem_defined = chassis_oem_defined
            self.chassis_height = chassis_height
            self.chassis_contained_element_count = chassis_contained_element_count
            self.chassis_contained_element_record_length = chassis_contained_element_record_length
            self.chassis_contained_elements = chassis_contained_elements
            self.chassis_sku_number = chassis_sku_number
            self.chassis_boardlist = chassis_boardlist

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + f'{self.chassis_handle}' + '\2' + f'{self.chassis_manufacturer}' + '\2' + \
            f'{self.chassis_type}' + '\2' + f'{self.chassis_version}' + '\2' + f'{self.chassis_serial_number}' + '\2' +\
            f'{self.chassis_asset_tag}' + '\2' + \
            f'{self.chassis_oem_defined}' + '\2' +\
            f'{self.chassis_height}'+ '\2' + \
            f'{self.chassis_sku_number}'

    class chassis_contained_element:
        chassis_contained_element_type: cython.uchar
        chassis_contained_element_minimum_number: cython.uchar
        chassis_contained_element_maximum_number: cython.uchar
        chassis_contained_element_other_bytes: str

        def __init__(self, chassis_contained_element_type: cython.uchar,
                     chassis_contained_element_minimum_number: cython.uchar,
                     chassis_contained_element_maximum_number: cython.uchar,
                     chassis_contained_element_other_bytes: str):
            self.chassis_contained_element_type = chassis_contained_element_type
            self.chassis_contained_element_minimum_number = chassis_contained_element_minimum_number
            self.chassis_contained_element_maximum_number = chassis_contained_element_maximum_number
            self.chassis_contained_element_other_bytes = chassis_contained_element_other_bytes

        def __str__(self):
            return f'{self.chassis_contained_element_type}' + '\2' +\
            f'{self.chassis_contained_element_minimum_number}' + '\2' +\
            f'{self.chassis_contained_element_maximum_number}' + '\2' +\
            f'{self.chassis_contained_element_other_bytes}'

    class baseboard_info:
        smbios_type_id: cython.uchar = 2
        baseboard_handle: cython.ushort
        baseboard_manufacturer: str
        baseboard_product: str
        baseboard_version: str
        baseboard_serial_number: str
        baseboard_chassis_handle: cython.ushort
        baseboard_board_type: cython.uchar
        baseboard_number_of_contained_object_handles: cython.uchar
        baseboard_contained_object_handles: list

        def __init__(self, baseboard_handle: cython.ushort,
                     baseboard_manufacturer: str,
                     baseboard_product: str,
                     baseboard_version: str,
                     baseboard_serial_number: str,
                     baseboard_chassis_handle: cython.ushort,
                     baseboard_board_type: cython.uchar,
                     baseboard_number_of_contained_object_handles: cython.uchar,
                     baseboard_contained_object_handles: list):
            self.baseboard_manufacturer = baseboard_manufacturer
            self.baseboard_handle = baseboard_handle
            self.baseboard_product = baseboard_product
            self.baseboard_version = baseboard_version
            self.baseboard_serial_number = baseboard_serial_number
            self.baseboard_chassis_handle = baseboard_chassis_handle
            self.baseboard_board_type = baseboard_board_type
            self.baseboard_number_of_contained_object_handles = baseboard_number_of_contained_object_handles
            self.baseboard_contained_object_handles = baseboard_contained_object_handles

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + f'{self.baseboard_handle}' + '\2' + \
                   f'{self.baseboard_manufacturer}' + '\2' + f'{self.baseboard_product}' + '\2' + \
                   f'{self.baseboard_version}' + '\2' + \
                   f'{self.baseboard_serial_number}' + '\2' + \
                f'{self.baseboard_chassis_handle}' + '\2' +\
                f'{self.baseboard_board_type}' + '\2' + \
                f'{self.baseboard_number_of_contained_object_handles}' + '\2' + \
                '\4' + '\4'.join([ str(i) for i in self.baseboard_contained_object_handles ])

    class cpu_cache_info:
        smbios_type_id: cython.uchar = 7
        cc_handle: cython.ushort
        cc_socket_designation: str
        cc_cache_configuration: cython.ushort
        cc_maximum_cache_size: cython.ushort
        cc_maximum_cache_size2: cython.uint
        cc_installed_size: cython.ushort
        cc_installed_size2: cython.uint
        cc_supported_sram_type: cython.ushort
        cc_current_sram_type: cython.ushort
        cc_error_corrention_type: cython.uchar
        cc_system_cache_type: cython.uchar
        cc_associativity: cython.uchar

        def __init__(self,
                     cc_handle: cython.ushort,
                     cc_socket_designation: str,
                     cc_cache_configuration: cython.ushort,
                     cc_maximum_cache_size: cython.ushort,
                     cc_maximum_cache_size2: cython.uint,
                     cc_installed_size: cython.ushort,
                     cc_installed_size2: cython.uint,
                     cc_supported_sram_type: cython.ushort,
                     cc_current_sram_type: cython.ushort,
                     cc_error_corrention_type: cython.uchar,
                     cc_system_cache_type: cython.uchar,
                     cc_associativity: cython.uchar):
            self.cc_handle = cc_handle
            self.cc_socket_designation = cc_socket_designation
            self.cc_cache_configuration = cc_cache_configuration
            self.cc_maximum_cache_size = cc_maximum_cache_size
            self.cc_maximum_cache_size2 = cc_maximum_cache_size2
            self.cc_installed_size = cc_installed_size
            self.cc_installed_size2 = cc_installed_size2
            self.cc_supported_sram_type = cc_supported_sram_type
            self.cc_current_sram_type = cc_current_sram_type
            self.cc_error_corrention_type = cc_error_corrention_type
            self.cc_system_cache_type = cc_system_cache_type
            self.cc_associativity = cc_associativity

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.cc_handle}' + '\2' + \
                   f'{self.cc_socket_designation}' + '\2' + \
                   f'{self.cc_cache_configuration}' + '\2' + \
                   f'{self.cc_maximum_cache_size}' + '\2' + \
                   f'{self.cc_maximum_cache_size2}' + '\2' + \
                   f'{self.cc_installed_size}' + '\2' + \
                   f'{self.cc_installed_size2}' + '\2' + \
                   f'{self.cc_supported_sram_type}' + '\2' + \
                   f'{self.cc_current_sram_type}' + '\2' + \
                   f'{self.cc_error_corrention_type}' + '\2' + \
                   f'{self.cc_system_cache_type}' + '\2' + \
                   f'{self.cc_associativity}'

    class cpu_info:
        smbios_type_id: cython.uchar = 4
        cpu_handle: cython.ushort
        cpu_socket_designation: str
        cpu_upgrade: cython.uchar
        cpu_manufacturer: str
        cpu_processor_id: cython.size_t
        cpu_voltage: cython.uchar
        cpu_max_speed: cython.ushort
        cpu_status: cython.uchar
        cpu_serial_number: str
        cpu_asset_tag: str
        cpu_part_number: str
        cpu_core_count: cython.uchar
        cpu_core_count2: cython.ushort
        cpu_core_enabled: cython.uchar
        cpu_core_enabled2: cython.ushort
        cpu_thread_count: cython.uchar
        cpu_thread_count2: cython.ushort
        cpu_thread_enabled: cython.ushort
        cpu_l1_cache_handle: cython.ushort
        cpu_l2_cache_handle: cython.ushort
        cpu_l3_cache_handle: cython.ushort
        cpu_l1_cache: cpu_cache_info
        cpu_l2_cache: cpu_cache_info
        cpu_l3_cache: cpu_cache_info

        def __init__(self, cpu_handle
                     , cpu_socket_designation=None
                     , cpu_upgrade=None
                     , cpu_manufacturer=None
                     , cpu_processor_id=None
                     , cpu_voltage=None
                     , cpu_max_speed=None
                     , cpu_status=None
                     , cpu_serial_number=None
                     , cpu_asset_tag=None
                     , cpu_part_number=None
                     , cpu_core_count=None
                     , cpu_core_count2=None
                     , cpu_core_enabled=None
                     , cpu_core_enabled2=None
                     , cpu_thread_count=None
                     , cpu_thread_count2=None
                     , cpu_thread_enabled=None
                     , cpu_l1_cache_handle=None
                     , cpu_l2_cache_handle=None
                     , cpu_l3_cache_handle=None
                     , cpu_l1_cache=None
                     , cpu_l2_cache=None
                     , cpu_l3_cache=None):
            self.cpu_handle = cpu_handle
            self.cpu_socket_designation = cpu_socket_designation
            self.cpu_upgrade = cpu_upgrade
            self.cpu_manufacturer = cpu_manufacturer
            self.cpu_processor_id = cpu_processor_id
            self.cpu_voltage = cpu_voltage
            self.cpu_max_speed = cpu_max_speed
            self.cpu_status = cpu_status
            self.cpu_serial_number = cpu_serial_number
            self.cpu_asset_tag = cpu_asset_tag
            self.cpu_part_number = cpu_part_number
            self.cpu_core_count = cpu_core_count
            self.cpu_core_count2 = cpu_core_count2
            self.cpu_core_enabled = cpu_core_enabled
            self.cpu_core_enabled2 = cpu_core_enabled2
            self.cpu_thread_count = cpu_thread_count
            self.cpu_thread_count2 = cpu_thread_count2
            self.cpu_thread_enabled = cpu_thread_enabled
            self.cpu_l1_cache_handle = cpu_l1_cache_handle
            self.cpu_l2_cache_handle = cpu_l2_cache_handle
            self.cpu_l3_cache_handle = cpu_l3_cache_handle
            self.cpu_l1_cache = cpu_l1_cache
            self.cpu_l2_cache = cpu_l2_cache
            self.cpu_l3_cache = cpu_l3_cache

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.cpu_handle}' + '\2' + \
                   f'{self.cpu_socket_designation}' + '\2' + \
                   f'{self.cpu_upgrade}' + '\2' + \
                   f'{self.cpu_manufacturer}' + '\2' + \
                   f'{self.cpu_processor_id}' + '\2' + \
                   f'{self.cpu_voltage}' + '\2' + \
                   f'{self.cpu_max_speed:}' + '\2' + \
                   f'{self.cpu_status}' + '\2' + \
                   f'{self.cpu_serial_number}' + '\2' + \
                   f'{self.cpu_asset_tag}' + '\2' + \
                   f'{self.cpu_part_number}' + '\2' + \
                   f'{self.cpu_core_count}' + '\2' + \
                   f'{self.cpu_core_count2}' + '\2' + \
                   f'{self.cpu_core_enabled}' + '\2' + \
                   f'{self.cpu_core_enabled2}' + '\2' + \
                   f'{self.cpu_thread_count}' + '\2' + \
                   f'{self.cpu_thread_count2}' + '\2' + \
                   f'{self.cpu_thread_enabled}' + '\2' + \
                   f'{self.cpu_l1_cache_handle}' + '\2' + \
                   f'{self.cpu_l2_cache_handle}' + '\2' + \
                   f'{self.cpu_l3_cache_handle}' + '\2' + \
                   str(self.cpu_l1_cache) + '\2' + \
                   str(self.cpu_l2_cache) + '\2' + \
                   str(self.cpu_l3_cache)

    class memory_controller_info:
        smbios_type_id: cython.uchar = 5
        mc_handle: cython.ushort
        mc_error_detecting_method: cython.uchar
        mc_correcting_capability: cython.uchar
        mc_supported_interleave: cython.uchar
        mc_current_interleave: cython.uchar
        mc_maximum_memory_module_size: cython.uchar
        mc_supported_speeds: cython.ushort
        mc_supported_memory_types: cython.ushort
        mc_memory_module_voltage: cython.uchar
        mc_mumber_of_associated_memory_slots: cython.uchar
        mc_memory_module_configuration_handles: list
        mc_memory_module_list: list
        mc_memory_enabled_error_correcting_capabilities: cython.uchar

        def __init__(self,
                     mc_handle: cython.ushort,
                     mc_error_detecting_method: cython.uchar,
                     mc_correcting_capability: cython.uchar,
                     mc_supported_interleave: cython.uchar,
                     mc_current_interleave: cython.uchar,
                     mc_maximum_memory_module_size: cython.uchar,
                     mc_supported_speeds: cython.ushort,
                     mc_supported_memory_types: cython.ushort,
                     mc_memory_module_voltage: cython.uchar,
                     mc_mumber_of_associated_memory_slots: cython.uchar,
                     mc_memory_module_configuration_handles: list,
                     mc_memory_module_list: list,
                     mc_memory_enabled_error_correcting_capabilities: cython.uchar,
                     ):
            self.mc_handle = mc_handle
            self.mc_error_detecting_method = mc_error_detecting_method
            self.mc_correcting_capability = mc_correcting_capability
            self.mc_supported_interleave = mc_supported_interleave
            self.mc_current_interleave = mc_current_interleave
            self.mc_maximum_memory_module_size = mc_maximum_memory_module_size
            self.mc_supported_speeds = mc_supported_speeds
            self.mc_supported_memory_types = mc_supported_memory_types
            self.mc_memory_module_voltage = mc_memory_module_voltage
            self.mc_mumber_of_associated_memory_slots = mc_mumber_of_associated_memory_slots
            self.mc_memory_module_configuration_handles = mc_memory_module_configuration_handles
            self.mc_memory_module_list = mc_memory_module_list
            self.mc_memory_enabled_error_correcting_capabilities = mc_memory_enabled_error_correcting_capabilities

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.mc_handle}' + '\2' + \
                   f'{self.mc_error_detecting_method}' + '\2' + \
                   f'{self.mc_correcting_capability}' + '\2' + \
                   f'{self.mc_supported_interleave}' + '\2' + \
                   f'{self.mc_current_interleave:}' + '\2' + \
                   f'{self.mc_maximum_memory_module_size}' + '\2' + \
                   f'{self.mc_supported_speeds}' + '\2' + \
                   f'{self.mc_supported_memory_types}' + '\2' + \
                   f'{self.mc_memory_module_voltage}' + '\2' + \
                   f'{self.mc_mumber_of_associated_memory_slots}' + '\2' + \
                   '\4' + '\4'.join([str(i) for i in self.mc_memory_module_configuration_handles ]) + '\2' + \
                   '\4' + '\4'.join([str(i) for i in self.mc_memory_module_list]) + '\2' + \
                   f'{self.mc_memory_enabled_error_correcting_capabilities}'

    class memory_module_info:
        smbios_type_id: cython.uchar = 6
        mm_handle: cython.ushort
        mm_socket_designation: str
        mm_bank_connections: cython.uchar
        mm_current_memory_type: cython.ushort
        mm_installed_size: cython.uchar
        mm_enabled_size: cython.uchar
        mm_error_status: cython.uchar

        def __init__(self,
                     mm_handle=None,
                     mm_socket_designation=None,
                     mm_bank_connections=None,
                     mm_current_memory_type=None,
                     mm_installed_size=None,
                     mm_enabled_size=None,
                     mm_error_status=None):
            self.mm_handle = mm_handle
            self.mm_socket_designation = mm_socket_designation
            self.mm_bank_connections = mm_bank_connections
            self.mm_current_memory_type = mm_current_memory_type
            self.mm_installed_size = mm_installed_size
            self.mm_enabled_size = mm_enabled_size
            self.mm_error_status = mm_error_status

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.mm_handle}' + '\2' + \
                   f'{self.mm_socket_designation}' + '\2' + \
                   f'{self.mm_bank_connections}' + '\2' + \
                   f'{self.mm_current_memory_type}' + '\2' + \
                   f'{self.mm_installed_size:}' + '\2' + \
                   f'{self.mm_enabled_size}' + '\2' + \
                   f'{self.mm_error_status}'

    class system_slot_info:
        smbios_type_id: cython.uchar = 9
        ss_handle: cython.ushort
        ss_designation: str
        ss_type: cython.uchar
        ss_data_bus_width: cython.uchar
        ss_current_usage: cython.uchar
        ss_length: cython.uchar
        ss_id: cython.ushort
        ss_characteristics_1: cython.uchar
        ss_characteristics_2: cython.uchar
        ss_segment_group_number: cython.ushort
        ss_bus_number: cython.uchar
        ss_device_function_number: cython.uchar
        ss_peer_data_bus_width: cython.uchar
        ss_peer_grouping_count: cython.uchar
        ss_peer_groups: list
        ss_information: cython.uchar
        ss_physical_width: cython.uchar
        ss_pitch: cython.ushort
        ss_height: cython.uchar

        def __init__(self, ss_handle: cython.ushort
                     , ss_designation: str
                     , ss_type: cython.uchar
                     , ss_data_bus_width: cython.uchar
                     , ss_current_usage: cython.uchar
                     , ss_length: cython.uchar
                     , ss_id: cython.ushort
                     , ss_characteristics_1: cython.uchar
                     , ss_characteristics_2: cython.uchar
                     , ss_segment_group_number: cython.ushort
                     , ss_bus_number: cython.uchar
                     , ss_device_function_number: cython.uchar
                     , ss_peer_data_bus_width: cython.uchar
                     , ss_peer_grouping_count: cython.uchar
                     , ss_peer_groups: list
                     , ss_information: cython.uchar
                     , ss_physical_width: cython.uchar
                     , ss_pitch: cython.ushort
                     , ss_height: cython.uchar):
            self.ss_handle = ss_handle
            self.ss_designation = ss_designation
            self.ss_type = ss_type
            self.ss_data_bus_width = ss_data_bus_width
            self.ss_current_usage = ss_current_usage
            self.ss_length = ss_length
            self.ss_id = ss_id
            self.ss_characteristics_1 = ss_characteristics_1
            self.ss_characteristics_2 = ss_characteristics_2
            self.ss_segment_group_number = ss_segment_group_number
            self.ss_bus_number = ss_bus_number
            self.ss_device_function_number = ss_device_function_number
            self.ss_peer_data_bus_width = ss_peer_data_bus_width
            self.ss_peer_grouping_count = ss_peer_grouping_count
            self.ss_peer_groups = ss_peer_groups
            self.ss_information = ss_information
            self.ss_physical_width = ss_physical_width
            self.ss_pitch = ss_pitch
            self.ss_height = ss_height

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.ss_handle}' + '\2' + \
                   f'{self.ss_designation}' + '\2' + \
                   f'{self.ss_type}' + '\2' + \
                   f'{self.ss_data_bus_width}' + '\2' + \
                   f'{self.ss_current_usage:}' + '\2' + \
                   f'{self.ss_length}' + '\2' + \
                   f'{self.ss_id}' + '\2' + \
                   f'{self.ss_characteristics_1}' + '\2' + \
                   f'{self.ss_characteristics_2}' + '\2' + \
                   f'{self.ss_segment_group_number}' + '\2' + \
                   f'{self.ss_bus_number}' + '\2' + \
                   f'{self.ss_device_function_number:}' + '\2' + \
                   f'{self.ss_peer_data_bus_width}' + '\2' + \
                   f'{self.ss_peer_grouping_count}' + '\2' + \
                   '\4'+'\4'.join([ str(i) for i in self.ss_peer_groups ]) + '\2' + \
                   f'{self.ss_information}' + '\2' + \
                   f'{self.ss_physical_width}' + '\2' + \
                   f'{self.ss_pitch}' + '\2' + \
                   f'{self.ss_height}'

    class peer_group_info:
        segment_group_number: cython.ushort
        bus_number: cython.uchar
        device_function_number: cython.uchar
        data_bus_width: cython.uchar

        def __init__(self,
                     segment_group_number=None,
                     bus_number=None,
                     device_function_number=None,
                     data_bus_width=None):
            self.segment_group_number = segment_group_number
            self.bus_number = bus_number
            self.device_function_number = device_function_number
            self.data_bus_width = data_bus_width

        def __str__(self):
            return f'{self.segment_group_number}' + '\2' + \
                   f'{self.bus_number}' + '\2' + \
                   f'{self.device_function_number}' + '\2' + \
                   f'{self.data_bus_width}'

    class on_board_device_info:
        smbios_type_id: cython.uchar = 10
        device_type: cython.uchar
        device_description: str
        def __init__(self, device_type: cython.uchar, device_description: str):
            self.device_type = device_type
            self.device_description = device_description

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.device_type}' + '\2' + \
                   f'{self.device_description}'

    class on_board_device_extended_info:
        smbios_type_id: cython.uchar = 41
        reference_designation: str
        device_type: cython.uchar
        device_type_instance: cython.uchar
        segment_group_number: cython.ushort
        bus_number: cython.uchar
        device_function_number: cython.uchar

        def __init__(self, reference_designation=None,
                     device_type=None,
                     device_type_instance=None,
                     segment_group_number=None,
                     bus_number=None,
                     device_function_number=None):
            self.reference_designation = reference_designation
            self.device_type = device_type
            self.device_type_instance = device_type_instance
            self.segment_group_number = segment_group_number
            self.bus_number = bus_number
            self.device_function_number = device_function_number

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.reference_designation}' + '\2' + \
                   f'{self.device_type}' + '\2' + \
                   f'{self.device_type_instance}' + '\2' + \
                   f'{self.segment_group_number}' + '\2' + \
                   f'{self.bus_number}' + '\2' + \
                   f'{self.device_function_number}'

    class physical_memory_array_info:
        smbios_type_id: cython.uchar = 16
        pma_handle: cython.ushort
        pma_location: cython.uchar
        pma_use: cython.uchar
        pma_memory_error_correction: cython.uchar
        pma_slots_number: cython.ushort
        pma_maximum_capacity: cython.uint
        pma_extended_maximum_capacity: cython.size_t
        pma_memory_device_list: list

        def __init__(self,
                     pma_handle: cython.ushort,
                     pma_location: cython.uchar,
                     pma_use: cython.uchar,
                     pma_memory_error_correction: cython.uchar,
                     pma_slots_number: cython.ushort,
                     pma_maximum_capacity: cython.uint,
                     pma_extended_maximum_capacity: cython.size_t,
                     pma_memory_device_list: list):
            self.pma_handle = pma_handle
            self.pma_location = pma_location
            self.pma_use = pma_use
            self.pma_memory_error_correction = pma_memory_error_correction
            self.pma_slots_number = pma_slots_number
            self.pma_maximum_capacity = pma_maximum_capacity
            self.pma_extended_maximum_capacity = pma_extended_maximum_capacity
            self.pma_memory_device_list = pma_memory_device_list

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.pma_handle}' + '\2' + \
                   f'{self.pma_location}' + '\2' + \
                   f'{self.pma_use}' + '\2' + \
                   f'{self.pma_memory_error_correction}' + '\2' + \
                   f'{self.pma_slots_number}' + '\2' + \
                   f'{self.pma_maximum_capacity}' + '\2' + \
                   f'{self.pma_extended_maximum_capacity}' + '\2' + \
                   '\4'+'\4'.join([ str(i) for i in self.pma_memory_device_list ])

    class memory_device_set_info:
        memory_device_set_id: cython.uchar
        mds_memory_device_list: list

        def __init__(self, memory_device_set_id=None,
                     mds_memory_device_list=None):
            self.memory_device_set_id=memory_device_set_id
            self.mds_memory_device_list=mds_memory_device_list

        def __str__(self):
            return f'{self.memory_device_set_id}' + \
                   '\4' + '\4'.join([str(i) for i in self.mds_memory_device_list])

    class memory_device_info:
        smbios_type_id: cython.uchar = 17
        md_handle: cython.ushort
        pma_handle: cython.ushort
        md_memory_error_information_handle: cython.ushort
        md_total_width: cython.ushort
        md_data_width: cython.ushort
        md_form_factor: cython.uchar
        md_device_set_id: cython.uchar
        md_device_locator: str
        md_bank_locator: str
        md_memory_type: cython.uchar
        md_type_detail: cython.ushort
        md_max_speed: cython.ushort
        md_max_extended_speed: cython.uint
        md_manufacturer: str
        md_serial_number: str
        md_asset_tag: str
        md_part_number: str
        md_attributes: cython.uchar
        md_size: cython.ushort
        md_extended_size: cython.uint
        md_memory_technology: cython.uchar
        md_memory_operating_mode_capability: cython.ushort

        def __init__(self, md_handle=None,
                     pma_handle=None,
                     md_memory_error_information_handle=None,
                     md_total_width=None,
                     md_data_width=None,
                     md_form_factor=None,
                     md_device_set_id=None,
                     md_device_locator=None,
                     md_bank_locator=None,
                     md_memory_type=None,
                     md_type_detail=None,
                     md_max_speed=None,
                     md_max_extended_speed=None,
                     md_manufacturer=None,
                     md_serial_number=None,
                     md_asset_tag=None,
                     md_part_number=None,
                     md_attributes=None,
                     md_size=None,
                     md_extended_size=None,
                     md_memory_technology=None,
                     md_memory_operating_mode_capability=None):
            self.md_handle = md_handle
            self.pma_handle = pma_handle
            self.md_memory_error_information_handle = md_memory_error_information_handle
            self.md_total_width = md_total_width
            self.md_data_width = md_data_width
            self.md_form_factor = md_form_factor
            self.md_device_set_id = md_device_set_id
            self.md_device_locator = md_device_locator
            self.md_bank_locator = md_bank_locator
            self.md_memory_type = md_memory_type
            self.md_type_detail = md_type_detail
            self.md_max_speed = md_max_speed
            self.md_max_extended_speed = md_max_extended_speed
            self.md_manufacturer = md_manufacturer
            self.md_serial_number = md_serial_number
            self.md_asset_tag = md_asset_tag
            self.md_part_number = md_part_number
            self.md_attributes = md_attributes
            self.md_size = md_size
            self.md_extended_size = md_extended_size
            self.md_memory_technology = md_memory_technology
            self.md_memory_operating_mode_capability = md_memory_operating_mode_capability

        def __str__(self):
            return f'{self.smbios_type_id}' + '\2' + \
                   f'{self.md_handle}' + '\2' + \
                   f'{self.pma_handle}' + '\2' + \
                   f'{self.md_memory_error_information_handle}' + '\2' + \
                   f'{self.md_total_width}' + '\2' + \
                   f'{self.md_data_width}' + '\2' + \
                   f'{self.md_form_factor}' + '\2' + \
                   f'{self.md_device_set_id}' + '\2' + \
                   f'{self.md_device_locator}' + '\2' + \
                   f'{self.md_bank_locator}' + '\2' + \
                   f'{self.md_memory_type}' + '\2' + \
                   f'{self.md_type_detail}' + '\2' + \
                   f'{self.md_max_speed}' + '\2' + \
                   f'{self.md_max_extended_speed}' + '\2' + \
                   f'{self.md_manufacturer}' + '\2' + \
                   f'{self.md_serial_number}' + '\2' + \
                   f'{self.md_asset_tag}' + '\2' + \
                   f'{self.md_part_number}' + '\2' + \
                   f'{self.md_attributes}' + '\2' + \
                   f'{self.md_size}' + '\2' + \
                   f'{self.md_extended_size}' + '\2' + \
                   f'{self.md_memory_technology}' + '\2' + \
                   f'{self.md_memory_operating_mode_capability}'

    def parse_smbios(smbios: bytes,
                     major_version: cython.uchar,
                     minor_version: cython.uchar,
                     docrev_version: cython.uchar):
        i = 0
        system_info['smbios_major_version'] = f'{major_version}'
        system_info['smbios_minor_version'] = f'{minor_version}'
        system_info['smbios_docrev_version'] = f'{docrev_version}'
        while i < len(smbios):
            type = int.from_bytes(smbios[i:i + 1], byteorder='little')
            length = int.from_bytes(smbios[i + 1:i + 2], byteorder='little')
            handle = int.from_bytes(smbios[i + 2:i + 4], byteorder='little')
            length_include_strings = smbios[i + length:].find(b'\0\0') + 2 + length
            text_strings = smbios[i + length:i + length_include_strings].split(b'\0')
            if length_include_strings == -1:
                raise ValueError("Couldn't find end of text strings!")
            if type == 0:
                # bios
                if length >= 5:
                    system_info['bios_list'].append(bios_info(
                        bios_vendor=string_from_position(smbios, i, 4, 5, text_strings)
                    ))
            elif type == 1:
                # system
                if length >= 5:
                    system_info['system_manufacturer'] = string_from_position(smbios, i, 4, 5, text_strings)
                if length >= 6:
                    system_info['system_product_name'] = string_from_position(smbios, i, 5, 6, text_strings)
                if length >= 8:
                    system_info['system_serial_number'] = string_from_position(smbios, i, 7, 8, text_strings)
                if length >= 0x18:
                    system_info['system_uuid'] = smbios[i + 0x8:i + 0x18].hex()
                if length >= 0x1A:
                    system_info['system_sku_number'] = string_from_position(smbios, i, 0x19, 0x1A, text_strings)
                if length >= 0x1B:
                    system_info['system_family'] = string_from_position(smbios, i, 0x1A, 0x1B, text_strings)
            elif type == 2:
                # board
                baseboard_manufacturer = None
                if length >= 5:
                    baseboard_manufacturer = string_from_position(smbios, i, 4, 5, text_strings)
                baseboard_product = None
                if length >= 6:
                    baseboard_product = string_from_position(smbios, i, 5, 6, text_strings)
                baseboard_version = None
                if length >= 7:
                    baseboard_version = string_from_position(smbios, i, 6, 7, text_strings)
                baseboard_serial_number = None
                if length >= 8:
                    baseboard_serial_number = string_from_position(smbios, i, 7, 8, text_strings)
                baseboard_chassis_handle = None
                if length >= 0x0D:
                    baseboard_chassis_handle: cython.ushort = int.from_bytes(smbios[i + 0x0B:i + 0x0D],
                                                                             byteorder='little')
                baseboard_board_type = None
                if length >= 0x0E:
                    baseboard_board_type: cython.uchar = int.from_bytes(smbios[i + 0x0D:i + 0x0E], byteorder='little')
                baseboard_number_of_contained_object_handles = None
                if length >= 0x0F:
                    baseboard_number_of_contained_object_handles: cython.uchar = int.from_bytes(
                        smbios[i + 0x0E:i + 0x0F], byteorder='little')
                baseboard_contained_object_handles = []
                if length >= 0x0F:
                    j = 0x0F
                    while length >= j + 2:
                        contained_object_handle = int.from_bytes(smbios[j:j + 2], byteorder='little')
                        baseboard_contained_object_handles.append(contained_object_handle)
                        j += 2
                system_info['baseboard_list'].append(baseboard_info(
                    baseboard_handle=handle,
                    baseboard_manufacturer=baseboard_manufacturer,
                    baseboard_product=baseboard_product,
                    baseboard_version=baseboard_version,
                    baseboard_serial_number=baseboard_serial_number,
                    baseboard_chassis_handle=baseboard_chassis_handle,
                    baseboard_board_type=baseboard_board_type,
                    baseboard_number_of_contained_object_handles=baseboard_number_of_contained_object_handles,
                    baseboard_contained_object_handles=baseboard_contained_object_handles
                ))
            elif type == 3:
                # chassis
                chassis_manufacturer = None
                if length >= 5:
                    chassis_manufacturer = string_from_position(smbios, i, 4, 5, text_strings)
                chassis_type = None
                if length >= 6:
                    chassis_type = int.from_bytes(smbios[i + 5:i + 6], byteorder='little')
                chassis_version = None
                if length >= 7:
                    chassis_version = string_from_position(smbios, i, 6, 7, text_strings)
                chassis_serial_number = None
                if length >= 8:
                    chassis_serial_number = string_from_position(smbios, i, 7, 8, text_strings)
                chassis_asset_tag = None
                if length >= 9:
                    chassis_asset_tag = string_from_position(smbios, i, 8, 9, text_strings)
                chassis_oem_defined = None
                if length >= 0x11:
                    chassis_oem_defined = int.from_bytes(smbios[i + 0x0D:i + 0x11], byteorder='little')
                chassis_height = None
                if length >= 0x12:
                    chassis_height = int.from_bytes(smbios[i + 0x11:i + 0x12], byteorder='little')
                chassis_contained_element_count = 0
                if length >= 0x14:
                    chassis_contained_element_count = int.from_bytes(smbios[i + 0x13:i + 0x14], byteorder='little')
                chassis_contained_element_record_length = 0
                if length >= 0x15:
                    chassis_contained_element_record_length = int.from_bytes(smbios[i + 0x14:i + 0x15],
                                                                             byteorder='little')
                chassis_contained_elements_list = []
                if length >= 0x15 + chassis_contained_element_count * chassis_contained_element_record_length:
                    j = 0
                    while j < chassis_contained_element_count:
                        chassis_contained_element_bytes = smbios[
                                                          0x15 + j * chassis_contained_element_record_length:0x15 + (
                                                                      j + 1) * chassis_contained_element_record_length]
                        chassis_contained_elements_list.append(
                            chassis_contained_element(
                                chassis_contained_element_type=int.from_bytes(chassis_contained_element_bytes[0:1],
                                                                              byteorder='little'),
                                chassis_contained_element_minimum_number=int.from_bytes(
                                    chassis_contained_element_bytes[1:2], byteorder='little'),
                                chassis_contained_element_maximum_number=int.from_bytes(
                                    chassis_contained_element_bytes[2:3], byteorder='little'),
                                chassis_contained_element_other_bytes=chassis_contained_element_bytes[3:].hex()
                                if len(chassis_contained_element_bytes) >= 4 else None)
                            if major_version > 2 or (major_version == 2 and minor_version > 3) or
                               (major_version == 2 and minor_version == 3 and docrev_version >= 2)
                            else chassis_contained_element(
                                chassis_contained_element_other_bytes=chassis_contained_element_bytes.hex()))
                        j += 1
                chassis_sku_number = None
                if length >= 0x15 + chassis_contained_element_count * chassis_contained_element_record_length + 1:
                    chassis_sku_number = string_from_position(smbios, i,
                                                              0x15 + chassis_contained_element_count * chassis_contained_element_record_length,
                                                              0x15 + chassis_contained_element_count * chassis_contained_element_record_length + 1,
                                                              text_strings)
                system_info['chassis_list'].append(chassis_info(
                    chassis_handle=handle,
                    chassis_manufacturer=chassis_manufacturer,
                    chassis_type=chassis_type,
                    chassis_version=chassis_version,
                    chassis_serial_number=chassis_serial_number,
                    chassis_asset_tag=chassis_asset_tag,
                    chassis_oem_defined=chassis_oem_defined,
                    chassis_height=chassis_height,
                    chassis_contained_element_count=chassis_contained_element_count,
                    chassis_contained_element_record_length=chassis_contained_element_record_length,
                    chassis_contained_elements=chassis_contained_elements_list,
                    chassis_sku_number=chassis_sku_number,
                    chassis_boardlist=[]
                ))
            elif type == 4:
                # cpu
                cpu_socket_designation = None
                if length >= 5:
                    cpu_socket_designation = string_from_position(smbios, i, 4, 5, text_strings)
                cpu_upgrade = None
                if length >= 0x1A:
                    cpu_upgrade = int.from_bytes(smbios[i + 0x19:i + 0x1A], byteorder='little')
                cpu_manufacturer = None
                if length >= 8:
                    cpu_manufacturer = string_from_position(smbios, i, 7, 8, text_strings)
                cpu_processor_id = None
                if length >= 0x10:
                    cpu_processor_id = int.from_bytes(smbios[i + 8:i + 12], byteorder='little')
                cpu_voltage = None
                if length >= 0x12:
                    cpu_voltage = int.from_bytes(smbios[i + 0x11:i + 0x12], byteorder='little')
                cpu_max_speed = None
                if length >= 0x16:
                    cpu_max_speed = int.from_bytes(smbios[i + 0x14:i + 0x16], byteorder='little')
                cpu_status = None
                if length >= 0x19:
                    cpu_status = int.from_bytes(smbios[i + 0x18:i + 0x19], byteorder='little')
                cpu_serial_number = None
                if length >= 0x21:
                    cpu_serial_number = string_from_position(smbios, i, 0x20, 0x21, text_strings)
                cpu_asset_tag = None
                if length >= 0x22:
                    cpu_asset_tag = string_from_position(smbios, i, 0x21, 0x22, text_strings)
                cpu_part_number = None
                if length >= 0x23:
                    cpu_part_number = string_from_position(smbios, i, 0x22, 0x23, text_strings)
                cpu_core_count = None
                if length >= 0x24:
                    cpu_core_count = int.from_bytes(smbios[i + 0x23:i + 0x24], byteorder='little')
                cpu_core_count2 = None
                if length >= 0x2C:
                    cpu_core_count2 = int.from_bytes(smbios[i + 0x2A:i + 0x2C], byteorder='little')
                cpu_core_enabled = None
                if length >= 0x25:
                    cpu_core_enabled = int.from_bytes(smbios[i + 0x24:i + 0x25], byteorder='little')
                cpu_core_enabled2 = None
                if length >= 0x2E:
                    cpu_core_enabled2 = int.from_bytes(smbios[i + 0x2C:i + 0x2E], byteorder='little')
                cpu_thread_count = None
                if length >= 0x26:
                    cpu_thread_count = int.from_bytes(smbios[i + 0x25:i + 0x26], byteorder='little')
                cpu_thread_count2 = None
                if length >= 0x30:
                    cpu_thread_count2 = int.from_bytes(smbios[i + 0x2E:i + 0x30], byteorder='little')
                cpu_thread_enabled = None
                if length >= 0x32:
                    cpu_thread_enabled = int.from_bytes(smbios[i + 0x30:i + 0x32], byteorder='little')
                cpu_l1_cache_handle = None
                if length >= 0x1C:
                    cpu_l1_cache_handle = int.from_bytes(smbios[i + 0x1A:i + 0x1C], byteorder='little')
                cpu_l2_cache_handle = None
                if length >= 0x1E:
                    cpu_l2_cache_handle = int.from_bytes(smbios[i + 0x1C:i + 0x1E], byteorder='little')
                cpu_l3_cache_handle = None
                if length >= 0x20:
                    cpu_l3_cache_handle = int.from_bytes(smbios[i + 0x1E:i + 0x20], byteorder='little')
                system_info['cpu_list'].append(cpu_info(
                    cpu_handle=handle,
                    cpu_socket_designation=cpu_socket_designation,
                    cpu_upgrade=cpu_upgrade,
                    cpu_manufacturer=cpu_manufacturer,
                    cpu_processor_id=cpu_processor_id,
                    cpu_voltage=cpu_voltage,
                    cpu_max_speed=cpu_max_speed,
                    cpu_status=cpu_status,
                    cpu_serial_number=cpu_serial_number,
                    cpu_asset_tag=cpu_asset_tag,
                    cpu_part_number=cpu_part_number,
                    cpu_core_count=cpu_core_count,
                    cpu_core_count2=cpu_core_count2,
                    cpu_core_enabled=cpu_core_enabled,
                    cpu_core_enabled2=cpu_core_enabled2,
                    cpu_thread_count=cpu_thread_count,
                    cpu_thread_count2=cpu_thread_count2,
                    cpu_thread_enabled=cpu_thread_enabled,
                    cpu_l1_cache_handle=cpu_l1_cache_handle,
                    cpu_l2_cache_handle=cpu_l2_cache_handle,
                    cpu_l3_cache_handle=cpu_l3_cache_handle,
                ))
            elif type == 5:
                # memory_controller
                '''
                mc_error_detecting_method = None
                if length >= 5:
                    mc_error_detecting_method = int.from_bytes(smbios[i + 0x04:i + 0x05], byteorder='little')
                mc_correcting_capability = None
                if length >= 6:
                    mc_correcting_capability = int.from_bytes(smbios[i + 0x05:i + 0x06], byteorder='little')
                mc_supported_interleave = None
                if length >= 7:
                    mc_supported_interleave = int.from_bytes(smbios[i + 0x06:i + 0x07], byteorder='little')
                mc_current_interleave = None
                if length >= 8:
                    mc_current_interleave = int.from_bytes(smbios[i + 0x07:i + 0x08], byteorder='little')
                mc_maximum_memory_module_size = None
                if length >= 9:
                    mc_maximum_memory_module_size = int.from_bytes(smbios[i + 0x08:i + 0x09], byteorder='little')
                mc_supported_speeds = None
                if length >= 0x0B:
                    mc_supported_speeds = int.from_bytes(smbios[i + 0x09:i + 0x0B], byteorder='little')
                mc_supported_memory_types = None
                if length >= 0x0D:
                    mc_supported_memory_types = int.from_bytes(smbios[i + 0x0B:i + 0x0D], byteorder='little')
                mc_memory_module_voltage = None
                if length >= 0x0E:
                    mc_memory_module_voltage = int.from_bytes(smbios[i + 0x0D:i + 0x0E], byteorder='little')
                mc_mumber_of_associated_memory_slots = 0
                if length >= 0x0F:
                    mc_mumber_of_associated_memory_slots = int.from_bytes(smbios[i + 0x0E:i + 0x0F], byteorder='little')
                mc_memory_module_configuration_handles = []
                if length >= 0x0F + mc_mumber_of_associated_memory_slots * 2:
                    j = 0
                    while j < mc_mumber_of_associated_memory_slots:
                        mc_memory_module_configuration_handles.append(
                            int.from_bytes(smbios[i + 0x0F + j * 2: i + 0x0F + (j + 1) * 2], byteorder='little'))
                        j += 1
                mc_memory_enabled_error_correcting_capabilities = None
                if length >= 0x0F + mc_mumber_of_associated_memory_slots * 2 + 1:
                    mc_memory_enabled_error_correcting_capabilities = int.from_bytes(
                        smbios[
                        i + 0x0F + mc_mumber_of_associated_memory_slots * 2: i + 0x0F + mc_mumber_of_associated_memory_slots * 2 + 1],
                        byteorder='little')
                system_info['memory_controller_list'].append(memory_controller_info(mc_handle=handle,
                                                                                    mc_error_detecting_method=mc_error_detecting_method,
                                                                                    mc_correcting_capability=mc_correcting_capability,
                                                                                    mc_supported_interleave=mc_supported_interleave,
                                                                                    mc_current_interleave=mc_current_interleave,
                                                                                    mc_maximum_memory_module_size=mc_maximum_memory_module_size,
                                                                                    mc_supported_speeds=mc_supported_speeds,
                                                                                    mc_supported_memory_types=mc_supported_memory_types,
                                                                                    mc_memory_module_voltage=mc_memory_module_voltage,
                                                                                    mc_mumber_of_associated_memory_slots=mc_mumber_of_associated_memory_slots,
                                                                                    mc_memory_module_configuration_handles=mc_memory_module_configuration_handles,
                                                                                    mc_memory_module_list=[],
                                                                                    mc_memory_enabled_error_correcting_capabilities=mc_memory_enabled_error_correcting_capabilities))
                '''
                pass
            elif type == 6:
                # memory_module
                mm_socket_designation = None
                if length >= 5:
                    mm_socket_designation = string_from_position(smbios, i, 4, 5, text_strings)
                mm_bank_connections = None
                if length >= 6:
                    mm_bank_connections = int.from_bytes(smbios[i + 5:i + 6], byteorder='little')
                mm_current_memory_type = None
                if length >= 9:
                    mm_current_memory_type = int.from_bytes(smbios[i + 7:i + 9], byteorder='little')
                mm_installed_size = None
                if length >= 0x0A:
                    mm_installed_size = int.from_bytes(smbios[i + 0x09:i + 0x0A], byteorder='little')
                mm_enabled_size = None
                if length >= 0x0B:
                    mm_enabled_size = int.from_bytes(smbios[i + 0x0A:i + 0x0B], byteorder='little')
                mm_error_status = None
                if length >= 0x0C:
                    mm_error_status = int.from_bytes(smbios[i + 0x0B:i + 0x0C], byteorder='little')
                system_info['memory_module_list'].append(memory_module_info(mm_handle=handle,
                                   mm_socket_designation=mm_socket_designation,
                                   mm_bank_connections=mm_bank_connections,
                                   mm_current_memory_type=mm_current_memory_type,
                                   mm_installed_size=mm_installed_size,
                                   mm_enabled_size=mm_enabled_size,
                                   mm_error_status=mm_error_status))
            elif type == 7:
                '''
                # cpu_cache
                cc_socket_designation = None
                if length >= 5:
                    cc_socket_designation = string_from_position(smbios, i, 4, 5, text_strings)
                cc_cache_configuration = None
                if length >= 7:
                    cc_cache_configuration = int.from_bytes(smbios[i + 5:i + 7], byteorder='little')
                cc_maximum_cache_size = None
                if length >= 9:
                    cc_maximum_cache_size = int.from_bytes(smbios[i + 7:i + 8], byteorder='little')
                cc_maximum_cache_size2 = None
                if length >= 0x17:
                    cc_maximum_cache_size2 = int.from_bytes(smbios[i + 0x13:i + 0x17], byteorder='little')
                cc_installed_size = None
                if length >= 0x0B:
                    cc_installed_size = int.from_bytes(smbios[i + 9:i + 0x0B], byteorder='little')
                cc_installed_size2 = None
                if length >= 0x1B:
                    cc_installed_size2 = int.from_bytes(smbios[i + 0x17:i + 0x1B], byteorder='little')
                cc_supported_sram_type = None
                if length >= 0x0D:
                    cc_supported_sram_type = int.from_bytes(smbios[i + 0x0B:i + 0x0D], byteorder='little')
                cc_current_sram_type = None
                if length >= 0x0F:
                    cc_current_sram_type = int.from_bytes(smbios[i + 0x0D:i + 0x0F], byteorder='little')
                cc_error_corrention_type = None
                if length >= 0x11:
                    cc_error_corrention_type = int.from_bytes(smbios[i + 0x10:i + 0x11], byteorder='little')
                cc_system_cache_type = None
                if length >= 0x12:
                    cc_system_cache_type = int.from_bytes(smbios[i + 0x11:i + 0x12], byteorder='little')
                cc_associativity = None
                if length >= 0x13:
                    cc_associativity = int.from_bytes(smbios[i + 0x12:i + 0x13], byteorder='little')
                system_info['cpu_cache_list'].append(cpu_cache_info(cc_handle=handle
                                                                    , cc_socket_designation=cc_socket_designation
                                                                    , cc_cache_configuration=cc_cache_configuration
                                                                    , cc_maximum_cache_size=cc_maximum_cache_size
                                                                    , cc_maximum_cache_size2=cc_maximum_cache_size2
                                                                    , cc_installed_size=cc_installed_size
                                                                    , cc_installed_size2=cc_installed_size2
                                                                    , cc_supported_sram_type=cc_supported_sram_type
                                                                    , cc_current_sram_type=cc_current_sram_type
                                                                    , cc_error_corrention_type=cc_error_corrention_type
                                                                    , cc_system_cache_type=cc_system_cache_type
                                                                    , cc_associativity=cc_associativity))
                '''
                pass
            elif type == 9:
                '''
                ss_designation = None
                if length >= 5:
                    ss_designation = string_from_position(smbios, i, 4, 5, text_strings)
                ss_type = None
                if length >= 6:
                    ss_type = int.from_bytes(smbios[i + 5:i + 6], byteorder='little')
                ss_data_bus_Width = None
                if length >= 7:
                    ss_data_bus_Width = int.from_bytes(smbios[i + 6:i + 7], byteorder='little')
                ss_current_usage = None
                if length >= 8:
                    ss_current_usage = int.from_bytes(smbios[i + 7:i + 8], byteorder='little')
                ss_length = None
                if length >= 9:
                    ss_length = int.from_bytes(smbios[i + 8:i + 9], byteorder='little')
                ss_id = None
                if length >= 0x0B:
                    ss_id = int.from_bytes(smbios[i + 9:i + 0x0B], byteorder='little')
                ss_characteristics_1 = None
                if length >= 0x0C:
                    ss_characteristics_1 = int.from_bytes(smbios[i + 0x0B:i + 0x0C], byteorder='little')
                ss_characteristics_2 = None
                if length >= 0x0D:
                    ss_characteristics_2 = int.from_bytes(smbios[i + 0x0C:i + 0x0D], byteorder='little')
                ss_segment_group_number = None
                if length >= 0x0F:
                    ss_segment_group_number = int.from_bytes(smbios[i + 0x0D:i + 0x0F], byteorder='little')
                ss_bus_number = None
                if length >= 0x10:
                    ss_bus_number = int.from_bytes(smbios[i + 0x0F:i + 0x10], byteorder='little')
                ss_device_function_number = None
                if length >= 0x11:
                    ss_device_function_number = int.from_bytes(smbios[i + 0x10:i + 0x11], byteorder='little')
                ss_peer_data_bus_width = None
                if length >= 0x12:
                    ss_peer_data_bus_width = int.from_bytes(smbios[i + 0x11:i + 0x12], byteorder='little')
                ss_peer_grouping_count = 0
                if length >= 0x13:
                    ss_peer_grouping_count = int.from_bytes(smbios[i + 0x12:i + 0x13], byteorder='little')
                ss_peer_groups = []
                if length >= 0x13 + 5 * ss_peer_grouping_count:
                    j = 0
                    while j <= ss_peer_groups:
                        segment_group_number = int.from_bytes(smbios[i + 0x13 + j * 5: i + 0x13 + j * 5 + 2], byteorder='little')
                        bus_number = int.from_bytes(smbios[i + 0x13 + j * 5 + 2: i + 0x13 + j * 5 + 3], byteorder='little')
                        device_function_number = int.from_bytes(smbios[i + 0x13 + j * 5 + 3: i + 0x13 + j * 5 + 4], byteorder='little')
                        data_bus_width = int.from_bytes(smbios[i + 0x13 + j * 5 + 4: i + 0x13 + j * 5 + 5], byteorder='little')
                        ss_peer_groups.append(peer_group_info(segment_group_number=segment_group_number,
                                                              bus_number=bus_number,
                                                              device_function_number=device_function_number,
                                                              data_bus_width=data_bus_width))
                        j += 1
                ss_information = None
                if length >= 0x14 + 5 * ss_peer_grouping_count:
                    ss_information = int.from_bytes(smbios[i + 0x13 + 5 * ss_peer_grouping_count:i + 0x14 + 5 * ss_peer_grouping_count], byteorder='little')
                ss_physical_width = None
                if length >= 0x15 + 5 * ss_peer_grouping_count:
                    ss_physical_width = int.from_bytes(smbios[i + 0x14 + 5 * ss_peer_grouping_count:i + 0x15 + 5 * ss_peer_grouping_count], byteorder='little')
                ss_pitch = None
                if length >= 0x17 + 5 * ss_peer_grouping_count:
                    ss_pitch = int.from_bytes(smbios[i + 0x15 + 5 * ss_peer_grouping_count:i + 0x17 + 5 * ss_peer_grouping_count], byteorder='little')
                ss_height = None
                if length >= 0x18 + 5 * ss_peer_grouping_count:
                    ss_height = int.from_bytes(smbios[i + 0x17 + 5 * ss_peer_grouping_count:i + 0x18 + 5 * ss_peer_grouping_count], byteorder='little')
                system_info['system_slot_list'].append(system_slot_info(ss_handle=handle
                                                                   , ss_designation=ss_designation
                                                                   , ss_type=ss_type
                                                                   , ss_data_bus_width=ss_data_bus_Width
                                                                   , ss_current_usage=ss_current_usage
                                                                   , ss_length=ss_length
                                                                   , ss_id=ss_id
                                                                   , ss_characteristics_1=ss_characteristics_1
                                                                   , ss_characteristics_2=ss_characteristics_2
                                                                   , ss_segment_group_number=ss_segment_group_number
                                                                   , ss_bus_number=ss_bus_number
                                                                   , ss_device_function_number=ss_device_function_number
                                                                   , ss_peer_data_bus_width=ss_peer_data_bus_width
                                                                   , ss_peer_grouping_count=ss_peer_grouping_count
                                                                   , ss_peer_groups=ss_peer_groups
                                                                   , ss_information=ss_information
                                                                   , ss_physical_width=ss_physical_width
                                                                   , ss_pitch=ss_pitch
                                                                   , ss_height=ss_height))
            '''
                pass
            elif type == 10:
                '''
                num_of_devices = int((length-4)/2)
                for j in range(num_of_devices):
                    device_type = int.from_bytes(smbios[i + 4 + 2 * j:i + 5 + 2 * j], byteorder='little')
                    device_description=string_from_position(smbios, i, 5 + 2 * j, 6 + 2 * j, text_strings)
                    system_info['on_board_device_list'].append(on_board_device_info(
                        device_type=device_type,
                        device_description=device_description
                    ))
                '''
                pass
            elif type == 16:
                # Physical_memory_array
                '''
                pma_location = None
                if length >= 5:
                    pma_location: cython.ushort = int.from_bytes(smbios[i + 4:i + 5],byteorder='little')
                pma_use = None
                if length >= 6:
                    pma_use: cython.ushort = int.from_bytes(smbios[i + 5:i + 6],byteorder='little')
                pma_memory_error_correction = None
                if length >= 6:
                    pma_memory_error_correction: cython.ushort = int.from_bytes(smbios[i + 6:i + 7],byteorder='little')
                pma_slots_number = None
                if length >= 0x0F:
                    pma_slots_number: cython.ushort = int.from_bytes(smbios[i + 0x0D:i + 0x0F],byteorder='little')
                pma_maximum_capacity = None
                if length >= 0x0B:
                    pma_maximum_capacity: cython.uint = int.from_bytes(smbios[i + 0x07:i + 0x0B],byteorder='little')
                pma_extended_maximum_capacity = None
                if length >= 0x17:
                    pma_extended_maximum_capacity: cython.size_t = int.from_bytes(smbios[i + 0x0F:i + 0x17],byteorder='little')
                system_info['physical_memory_array_list'].append(physical_memory_array_info(
                    pma_handle=handle,
                    pma_location=pma_location,
                    pma_use=pma_use,
                    pma_memory_error_correction=pma_memory_error_correction,
                    pma_slots_number=pma_slots_number,
                    pma_maximum_capacity=pma_maximum_capacity,
                    pma_extended_maximum_capacity=pma_extended_maximum_capacity,
                    pma_memory_device_list=[]
                ))
                '''
                pass
            elif type == 17:
                pma_handle = None
                if length >= 0x06:
                    pma_handle: cython.ushort = int.from_bytes(smbios[i + 0x04:i + 0x06], byteorder='little')
                md_memory_error_information_handle = None
                if length >= 0x08:
                    md_memory_error_information_handle: cython.ushort = int.from_bytes(smbios[i + 0x06:i + 0x08], byteorder='little')
                md_total_width = None
                if length >= 0x0A:
                    md_total_width: cython.ushort = int.from_bytes(smbios[i + 0x08:i + 0x0A], byteorder='little')
                md_data_width = None
                if length >= 0x0A:
                    md_data_width: cython.ushort = int.from_bytes(smbios[i + 0x0A:i + 0x0C], byteorder='little')
                md_form_factor = None
                if length >= 0x0F:
                    md_form_factor: cython.uchar = int.from_bytes(smbios[i + 0x0E:i + 0x0F], byteorder='little')
                md_device_set_id = None
                if length >= 0x10:
                    md_device_set_id: cython.uchar = int.from_bytes(smbios[i + 0x0F:i + 0x10], byteorder='little')
                md_device_locator = None
                if length >= 0x11:
                    md_device_locator: cython.uchar = string_from_position(smbios, i, 0x10, 0x11, text_strings)
                md_bank_locator = None
                if length >= 0x12:
                    md_bank_locator: cython.uchar = string_from_position(smbios, i, 0x11, 0x12, text_strings)
                md_memory_type = None
                if length >= 0x13:
                    md_memory_type: cython.uchar = int.from_bytes(smbios[i + 0x12:i + 0x13], byteorder='little')
                md_type_detail = None
                if length >= 0x15:
                    md_type_detail: cython.ushort = int.from_bytes(smbios[i + 0x13:i + 0x15], byteorder='little')
                md_max_speed = None
                if length >= 0x17:
                    md_max_speed: cython.ushort = int.from_bytes(smbios[i + 0x15:i + 0x17], byteorder='little')
                md_max_extended_speed = None
                if length >= 0x58:
                    md_max_extended_speed: cython.uint = int.from_bytes(smbios[i + 0x54:i + 0x58], byteorder='little')
                md_manufacturer = None
                if length >= 0x18:
                    md_manufacturer: cython.uint = string_from_position(smbios, i, 0x17, 0x18, text_strings)
                md_serial_number = None
                if length >= 0x19:
                    md_serial_number: cython.uint = string_from_position(smbios, i, 0x18, 0x19, text_strings)
                md_asset_tag = None
                if length >= 0x1A:
                    md_asset_tag: cython.uchar = int.from_bytes(smbios[i + 0x19:i + 0x1A], byteorder='little')
                md_part_number = None
                if length >= 0x1B:
                    md_part_number: cython.uchar = string_from_position(smbios, i, 0x1A, 0x1B, text_strings)
                md_attributes = None
                if length >= 0x1C:
                    md_attributes: cython.uint = int.from_bytes(smbios[i + 0x1B:i + 0x1C], byteorder='little')
                md_size = None
                if length >= 0x0E:
                    md_size: cython.ushort = int.from_bytes(smbios[i + 0x0C:i + 0x0E], byteorder='little')
                md_extended_size = None
                if length >= 0x20:
                    md_extended_size: cython.uint = int.from_bytes(smbios[i + 0x1C:i + 0x20], byteorder='little')
                md_memory_technology = None
                if length >= 0x13:
                    md_memory_technology: cython.uint = int.from_bytes(smbios[i + 0x12:i + 0x13], byteorder='little')
                md_memory_operating_mode_capability = None
                if length >= 0x2B:
                    md_memory_operating_mode_capability: cython.ushort = int.from_bytes(smbios[i + 0x29: i + 0x2B], byteorder='little')
                mm = memory_device_info(md_handle = handle
                        ,pma_handle = pma_handle
                        ,md_memory_error_information_handle = md_memory_error_information_handle
                        ,md_total_width = md_total_width
                        ,md_data_width = md_data_width
                        ,md_form_factor = md_form_factor
                        ,md_device_set_id = md_device_set_id
                        ,md_device_locator = md_device_locator
                        ,md_bank_locator = md_bank_locator
                        ,md_memory_type = md_memory_type
                        ,md_type_detail = md_type_detail
                        ,md_max_speed = md_max_speed
                        ,md_max_extended_speed = md_max_extended_speed
                        ,md_manufacturer = md_manufacturer
                        ,md_serial_number = md_serial_number
                        ,md_asset_tag = md_asset_tag
                        ,md_part_number = md_part_number
                        ,md_attributes = md_attributes
                        ,md_size = md_size
                        ,md_extended_size = md_extended_size
                        ,md_memory_technology = md_memory_technology
                        ,md_memory_operating_mode_capability = md_memory_operating_mode_capability)

                if md_device_set_id not in system_info['memory_device_set_id_list']:
                    system_info['memory_device_set_id_list'].append(md_device_set_id)
                    system_info['memory_device_set_list'].append(memory_device_set_info(
                        memory_device_set_id=md_device_set_id,
                        mds_memory_device_list=[mm]
                    ))
                else:
                    for memory_device_set in system_info['memory_device_set_list']:
                        if memory_device_set.memory_device_set_id == md_device_set_id:
                            memory_device_set.mds_memory_device_list.append(mm)
            elif type == 42:
                '''
                reference_designation = None
                if length >= 0x05:
                    reference_designation: cython.uchar = string_from_position(smbios, i, 0x04, 0x05, text_strings)
                device_type = None
                if length >= 0x06:
                    device_type: cython.uchar = int.from_bytes(smbios[i + 0x05:i + 0x06], byteorder='little')
                device_type_instance = None
                if length >= 0x07:
                    device_type_instance: cython.uchar = int.from_bytes(smbios[i + 0x06:i + 0x07], byteorder='little')
                segment_group_number = None
                if length >= 0x09:
                    segment_group_number: cython.ushort = int.from_bytes(smbios[i + 0x07:i + 0x09], byteorder='little')
                bus_number = None
                if length >= 0x0A:
                    bus_number: cython.uchar = int.from_bytes(smbios[i + 0x09:i + 0x0A], byteorder='little')
                device_function_number = None
                if length >= 0x0B:
                    device_function_number: cython.uchar = int.from_bytes(smbios[i + 0x0A:i + 0x0B], byteorder='little')

                system_info['on_board_device_extended_list'].append(on_board_device_extended_info(
                    reference_designation=reference_designation,
                    device_type=device_type,
                    device_type_instance=device_type_instance,
                    segment_group_number=segment_group_number,
                    bus_number=bus_number,
                    device_function_number=device_function_number
                ))
                '''
                pass
            else:
                # other
                pass
            i += length_include_strings

        pass

    def string_from_position(smbios, i, start, end, text_strings) -> str:
        str_num = int.from_bytes(smbios[i + start:i + end], byteorder='little')
        if str_num != 0:
            return text_strings[str_num - 1].decode()

    def parse_memory(bits: cython.char) -> tuple:
        with open('/dev/mem', 'rb') as f:
            f.seek(0xF0000)
            search_range = f.read(65535)
            if bits == 64:
                anchor_bytes = b'_SM3_'
            elif bits == 32:
                anchor_bytes = b'_SM_'
            else:
                raise ValueError("Unsupported bits number!")
            anchor_address: cython.Py_ssize_t = search_range.find(anchor_bytes)
            if anchor_address != -1 and bits == 64:
                smbios64_address: cython.size_t = anchor_address + 0xF0000
                # smbios64_address_ffi = ffi.new('int', smbios64_address)
                # smbios64_address = ffi.cast('char *', smbios64_address_ffi)

                f.seek(smbios64_address)
                smbios_entry_point = f.read(24)
                major_version = int.from_bytes(smbios_entry_point[7:8], byteorder='little')
                minor_version = int.from_bytes(smbios_entry_point[8:9], byteorder='little')
                docrev_version = int.from_bytes(smbios_entry_point[9:0x0A], byteorder='little')

                structure_table_length = int.from_bytes(smbios_entry_point[0x0C:0x0C + 4], byteorder='little')
                structure_table_address = int.from_bytes(smbios_entry_point[0x10:0x10 + 8], byteorder='little')
                f.seek(structure_table_address)
                return f.read(structure_table_length), major_version, minor_version, docrev_version
                # smbios_mmap = lib.mmap(structure_table_address, structure_table_length, 3, mmap_flags, -1, 0)
                # smbios_char_pointer = ffi.cast('char *', smbios_mmap)
                # return ffi.buffer(smbios_char_pointer, structure_table_length)[:]
            else:
                if anchor_address != -1 and bits == 32:
                    smbios32_address: cython.size_t = anchor_address + 0xF0000
                    # smbios32_address_ffi = ffi.new('int', smbios32_address)
                    # smbios32_address = ffi.cast('char *', smbios32_address_ffi)
                    f.seek(smbios32_address)
                    smbios_entry_point = f.read(31)
                    major_version = int.from_bytes(smbios_entry_point[6:7], byteorder='little')
                    minor_version = int.from_bytes(smbios_entry_point[7:8], byteorder='little')
                    structure_table_length = int.from_bytes(smbios_entry_point[0x16:0x16 + 2], byteorder='little')
                    structure_table_address = int.from_bytes(smbios_entry_point[0x18:0x18 + 4], byteorder='little')
                    f.seek(structure_table_address)
                    # smbios_mmap = lib.mmap(structure_table_address, structure_table_length, 3, mmap_flags, -1, 0)
                    # smbios_char_pointer = ffi.cast('char *', smbios_mmap)
                    return f.read(structure_table_length), major_version, minor_version, 0
                else:
                    raise NotImplementedError("Anchor string not found!")

    def parse_dmi() -> tuple:
        with open('/sys/firmware/dmi/tables/DMI', 'rb') as f:
            smbios = f.read()
            with open('/sys/firmware/dmi/tables/smbios_entry_point', 'rb') as f:
                smbios_entry_point = f.read()
                if smbios_entry_point.find(b'_SM_') != -1:
                    major_version = int.from_bytes(smbios_entry_point[6:7], byteorder='little')
                    minor_version = int.from_bytes(smbios_entry_point[7:8], byteorder='little')
                    return smbios, 32, major_version, minor_version, 0
                elif smbios_entry_point.find(b'_SM3_') != -1:
                    major_version = int.from_bytes(smbios_entry_point[7:8], byteorder='little')
                    minor_version = int.from_bytes(smbios_entry_point[8:9], byteorder='little')
                    docrev_version = int.from_bytes(smbios_entry_point[9:0x0A], byteorder='little')
                    return smbios, 64, major_version, minor_version, docrev_version

    def parse_directory():
        pass

    def aes_cbc_decryptor(aeskey: bytes, nonce_cipher: bytes, cbc_iv: bytes) -> bytes:

        aeskey_char = ffi.new("unsigned char []", aeskey)
        cbc_iv_char = ffi.new("unsigned char []", cbc_iv)
        nonce_cipher_char = ffi.new("unsigned char []", nonce_cipher)

        nonce_txt = ffi.new("unsigned char [1000]")
        nonce_len = ffi.new("int *")



        # encryptor = Cipher(algorithms.AES(zipkey), modes.CBC(iv)).encryptor()
        if lib.aes_cbc_decrypt(aeskey_char, cbc_iv_char, 16, nonce_txt, nonce_len, nonce_cipher_char, ffi.sizeof(nonce_cipher_char)-1) == 1:
            #print("nonce_len:" + f'{nonce_len[0]}', file=sys.stderr)
            if nonce_len[0] <= 16:
                nonce_txt_bytes = b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(nonce_txt, nonce_len[0])])
            else:
                nonce_txt = ffi.new("unsigned char []", nonce_len[0])
                lib.aes_cbc_encrypt(aeskey, cbc_iv, 16, nonce_txt, nonce_len, nonce_cipher, nonce_len)
                nonce_txt_bytes = b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(nonce_txt, nonce_len[0])])
        else:
            #out = open("/home/xiquanren/aes_cbc_python.txt", "ab")
            #out.write(b"aeskey_char: " + aeskey_char + b"\n")
            #out.write(b"nonce_cipher: " + nonce_cipher + b"\n")
            #out.write(b"cbc_iv: " + cbc_iv + b"\n")
            #out.close()
            pass
        return nonce_txt_bytes[0:12]

    def aes_gcm_decryptor(aeskey: bytes, cipher_txt: bytes, nonce: bytes) -> bytes:
        aeskey_char = ffi.new("unsigned char []", aeskey)
        nonce_char = ffi.new("unsigned char []", nonce)
        cipher_txt_char = ffi.new("unsigned char []", cipher_txt)

        content = ffi.new("unsigned char [100000]")
        content_len = ffi.new("int *")

        if lib.aes_gcm_decrypt(aeskey_char, nonce_char, ffi.sizeof(nonce_char)-1, content, content_len, cipher_txt_char, ffi.sizeof(cipher_txt_char)-1) == 1:
            if content_len[0] <= 100000:
                content_bytes = b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(content, content_len[0])])
            else:
                content = ffi.new("unsigned char []", content_len[0])
                lib.aes_gcm_encrypt(nonce, len(nonce), content, content_len, cipher_txt, len(cipher_txt))
                content_bytes = b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(content, content_len[0])])
        return content_bytes

    def aes_gcm_encryptor(aeskey: bytes, content: bytes, nonce: bytes) -> bytes:

        aeskey_char = ffi.new("unsigned char []", aeskey)
        content_char = ffi.new("unsigned char []", content)
        nonce_char = ffi.new("unsigned char []", nonce)

        cipher_txt = ffi.new("unsigned char [100000]")
        cipher_len = ffi.new("int *")

        if lib.aes_gcm_encrypt(aeskey_char, nonce_char, ffi.sizeof(nonce_char)-1, content_char, ffi.sizeof(content_char) - 1, cipher_txt, cipher_len) == 1:
            if cipher_len[0] <= 100000:
                cipher_txt_bytes = b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(cipher_txt, cipher_len[0])])
            else:
                cipher_txt = ffi.new("unsigned char []", cipher_len[0])
                lib.aes_gcm_encrypt(aeskey_char, nonce_char, ffi.sizeof(nonce_char)-1, content_char, ffi.sizeof(content_char) - 1, cipher_txt, cipher_len)
                cipher_txt_bytes = b''.join([i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(cipher_txt, cipher_len[0])])
        return cipher_txt_bytes

    def read_aesgcm_keyfile() -> tuple:
        keyfile = Path.joinpath(Path(__file__).parent, 'atomic_keyfile.dat')
        zipkey = base64_decode(b'7UYBBl5mAQpA+NAbZGVTt6cLGPjxvCcACyQAl/kQY20=')
        #aesgcm = AESGCM(zipkey)
        with open(keyfile, 'rb') as k:
            file_content_length = int.from_bytes(k.read(8), byteorder='big')
            content_cipher = k.read(file_content_length)
            iv = content_cipher[:16]
            #decryptor = Cipher(algorithms.AES(zipkey), modes.CBC(iv)).decryptor()
            nonce_cipher_bytes_length: cython.size_t = int.from_bytes(content_cipher[16:24], byteorder='big')
            #nonce = decryptor.update(content_cipher[24:24+nonce_cipher_bytes_length]) + decryptor.finalize()

            nonce_cipher = content_cipher[24:24+nonce_cipher_bytes_length]
            nonce = aes_cbc_decryptor(zipkey, nonce_cipher, iv)
            content = aes_gcm_decryptor(zipkey, content_cipher[24+nonce_cipher_bytes_length:], nonce)
            return iv, nonce, nonce_cipher, content.split(rb'^-------------------------^')

    if not checksum():
        logger.error("Checksum failed!")
        return 255

    try:
        smbios_dmi, bits, dmi_major_version, dmi_minor_version, dmi_docrev_version = parse_dmi()
    except IOError as e:
        logging.error("could not parse hardware information 1!")
        return 1
    except PermissionError as f:
        logging.error("could not parse hardware information 2!")
        return 2

    try:
        smbios_memory, memory_major_version, memory_minor_version, memory_docrev_version = parse_memory(bits)
    except IOError as e:
        logging.error("could not parse hardware information 3!")
        return 3
    except PermissionError as f:
        logging.error("could not parse hardware information 4!")
        return 4


    if smbios_dmi != smbios_memory or dmi_major_version != memory_major_version or \
            dmi_minor_version != memory_minor_version or dmi_docrev_version != memory_docrev_version:
        raise ValueError("hardware information not match 1!")

    other_bits = {32, 64} - {bits}
    try:
        smbios_memory_other_bits, memory_major_version_other_bits, memory_minor_version_other_bits, memory_docrev_version_other_bits =\
            parse_memory(list(other_bits)[0])
    except NotImplementedError as e:
        if e.args[0] == 'Anchor string not found!':
            pass
        else:
            raise e

    #print(5, file=sys.stderr)
    keyfile_list = read_aesgcm_keyfile()
    nonce = keyfile_list[1]
    zipkey = base64_decode(b'7UYBBl5mAQpA+NAbZGVTt6cLGPjxvCcACyQAl/kQY20=')
    parse_smbios(smbios_memory, memory_major_version, memory_minor_version, memory_docrev_version)

    final_str = '{'
    for key, value in system_info.items():
        if type(value) == list:
            str_value = '\3' + '\3'.join([str(i) for i in value])
        else:
            str_value = str(value)
        final_str += "'" + str(key) + "':'" + str_value + "',"
    final_str += '}'
    #print(6, file=sys.stderr)

    #print("nonce: " + nonce.decode('latin1'))
    #print("final_str: " + final_str)
    #print("aes_gcm_encryptor: " + aes_gcm_encryptor(zipkey, final_str.encode(), nonce).decode('latin1'))
    myprint = lambda objects: print(*objects, sep='', end='', file=sys.stdout, flush=True)
    myprint(base64_encode(aes_gcm_encryptor(zipkey, final_str.encode(), nonce)).decode())

    #print(7, file=sys.stderr)

    return 0
    '''
    f = open('/Users/philren/PycharmProjects/shared_atomic_private/sbios.dump', 'rb')
    smbios = f.read()

    
    parse_smbios(smbios, 2, 7, 0)
    str(system_info)

    
    print(final_str)
    '''
