from shared_atomic.atomic_decryption_ import ffi
from shared_atomic.atomic_decryption_ import lib as lib_decryption
from collections import namedtuple
import cython
import sys
import os
from functools import wraps


def sha1(message: bytes) -> str:
    """calculate sha1(internally used)

    :return: calculated sha1
    """
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")
    if lib_decryption.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib_decryption.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join(
            [i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")


if sha1(b'===randomstring===') != '===randomstringsha1===':
    raise ValueError("Didn't pass checksum1!")

if sys.platform in ("win32", "linux", "darwin"):
    import shared_atomic.shared_atomic_
    from shared_atomic.shared_atomic_ import lib as lib
    from shared_atomic.shared_atomic_ import ffi

    with open(shared_atomic.shared_atomic_.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomiclibsha1===':
            raise ValueError("Didn't pass checksum2!")

    pointer = ffi.CData
else:
    raise NotImplementedError("Unsupported platform!")

if sys.platform == "win32":
    import platform

    if platform.win32_ver()[0] + '.' + platform.win32_ver()[2] >= '10.SP0':
        from shared_atomic.shared_atomic_win10_ import lib as lib10
    with open(shared_atomic.shared_atomic_win10_.__file__, 'rb') as f:
        shared_atomic_win10_bytes = f.read()
        if sha1(shared_atomic_win10_bytes) != '===atomicwin10libsha1===':
            raise ValueError("Didn't pass checksum!")

def check_length(length: cython.char):
    """check the length of the operations

    :return: None, raise exception if not
    """
    set_lengths = frozenset({1, 2, 4, 8})
    if length not in set_lengths:
        raise ValueError('length except 1,2,4,8 cannot used for atomic operations! Please use memmove instead')


class atomic_object:
    """parent class of atomic types, other classes inherited this

        :attributes
        file_mapping: internal use\n
        file_mapping_name: internal use\n
        size: readonly size of the object\n
        reference: readonly pointer to the atomic object\n
        current_process_handle: readonly size of the list\n
        windows_unix_compatibility: readonly whether the source code should be compatible with windows x64
        mode: readonly 's' for single process, 'm' for multiprocessing, on windows platform, only singleprocessing is supported\n
    """
    #file_mapping: cython.longlong
    #file_mapping_name: bytes
    #size: cython.longlong
    #current_process_handle: cython.longlong
    #windows_unix_compatibility: cython.bint
    #creation_pid: cython.longlong
    #mode: str
    def __init__(self):
        pass

    def signed2unsigned(self, input: cython.longlong) -> cython.size_t:
        r"""
        transform of signed integer to unsigned integer,

        :param input: value to be change
        :return: the unsigned integer
        """
        if input >= 0:
            return input
        else:
            return input + 2 ** (self.size * 8)

    def unsigned2signed(self, input: cython.size_t) -> cython.longlong:
        r"""
        transform of unsigned integer to signed integer,

        :param input: value to be change
        :return: the signed integer
        """
        if input < 2 ** (self.size * 8 - 1):
            return input
        else:
            return input - 2 ** (self.size * 8)

class reference:
    """class of reference generated for multiprocessing mode

        :attributes
        file_mapping: internal use\n
        reference: readonly reference used in the subprocesses\n
    """
    file_mapping: cython.longlong
    reference: cython.longlong

    def __init__(self, file_mapping, reference):
        self.file_mapping = file_mapping
        self.reference = reference


if sys.platform in ("linux", "darwin"):
    def int_store(v: pointer, n: pointer):
        """Store value atomically

         :param v: the pointer to set
         :param n: the pointer from value to set
         :return: None
         """
        lib.ssize_t_store(v, n)


    def int_shift(v: pointer, n: pointer, r: pointer):
        """value exchange between 3 pointers in 2 groups atomically, store n in v after store v in r

        :param v: pointer of v
        :param n: pointer of n
        :param r: pointer of r
        :return: None
        """
        return lib.ssize_t_shift(v, n, r)


    def int_get_and_set(v: pointer, n: cython.longlong) -> cython.longlong:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.ssize_t_get_and_set(v, n)


    def int_compare_and_set(v: pointer, e: pointer, n: cython.longlong) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        the operation is a read and the current contents of v are written into e.

        :param v: pointer of v
        :param e: pointer of e
        :param n: value to be set
        :return: whether the contents of v and contents of e is the same
        """
        return lib.ssize_t_compare_and_set(v, e, n)


    def int_compare_and_set_value(v: pointer, e: cython.longlong, n: cython.longlong) -> cython.longlong:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """
        #expected = ffi.new("ssize_t *", n)
        return lib.ssize_t_compare_and_set(v, e, n)
        #return expected[0]


    def int_add_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """increment and fetch atomically

        :param v: atomic_int to add to and get
        :param n: data to add
        :return: sum of the 2 values
        """
        return lib.ssize_t_add_and_fetch(v, n)


    def int_sub_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """sub and fetch atomically

        :param v: pointer of value to subtract and get
        :param n: data to subtract
        :return: difference of the 2 values
        """
        return lib.ssize_t_sub_and_fetch(v, n)


    def int_and_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """bitwise AND and fetch the result atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: resulted value
        """
        return lib.ssize_t_and_and_fetch(v, n)


    def int_or_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """bitwise XOR and fetch the result atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: resulted value
        """
        return lib.ssize_t_or_and_fetch(v, n)


    def int_xor_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """bitwise XOR and fetch the result atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: resulted value
        """
        return lib.ssize_t_xor_and_fetch(v, n)


    def int_nand_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """bitwise NAND and fetch the result atomically

        :param v: pointer of value to NAND to
        :param n: data to NAND
        :return: resulted value
        """
        return lib.ssize_t_nand_and_fetch(v, n)


    def int_fetch_and_add(v: pointer, n: cython.longlong) -> cython.longlong:
        """increment and fetch atomically

        :param v: pointer of value to add to
        :param n: data to add
        :return: original value in v
        """
        return lib.ssize_t_fetch_and_add(v, n)


    def int_fetch_and_sub(v: pointer, n: cython.longlong) -> cython.longlong:
        """subtract and fetch atomically

        :param v: pointer of value to subtract from
        :param n: data to subtract
        :return: original value in v
        """
        return lib.ssize_t_fetch_and_sub(v, n)


    def int_fetch_and_and(v: pointer, n: cython.longlong) -> cython.longlong:
        """fetch then bitwise AND atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: the result
        """
        return lib.ssize_t_fetch_and_and(v, n)


    def int_fetch_and_or(v: pointer, n: cython.longlong) -> cython.longlong:
        """fetch then bitwise OR atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: the result
        """
        return lib.ssize_t_fetch_and_or(v, n)


    def int_fetch_and_xor(v: pointer, n: cython.longlong) -> cython.longlong:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.ssize_t_fetch_and_xor(v, n)


    def int_fetch_and_nand(v: pointer, n: cython.longlong) -> cython.longlong:
        """fetch then bitwise NAND atomically

        :param v: pointer of value to NAND to
        :param n: data to NAND
        :return: the result
        """
        return lib.ssize_t_fetch_and_nand(v, n)


    def uint_store(v: pointer, n: pointer):
        """Store value atomically

         :param v: the pointer to set
         :param n: the pointer from value to set
         :return: None
         """
        lib.size_t_store(v, n)


    def uint_shift(v: pointer, n: pointer, r: pointer):
        """value exchange between 3 pointers in 2 groups atomically, store n in v after store v in r

        :param v: pointer of v
        :param n: pointer of n
        :param r: pointer of r
        :return: None
        """
        return lib.size_t_shift(v, n, r)


    def uint_get_and_set(v: pointer, n: cython.size_t) -> cython.size_t:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.size_t_get_and_set(v, n)


    def uint_compare_and_set(v: pointer, e: pointer, n: cython.size_t) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        the operation is a read and the current contents of v are written into e.

        :param v: pointer of v
        :param e: pointer of e
        :param n: value to be set
        :return: whether the contents of v and contents of e is the same
        """
        return lib.size_t_compare_and_set(v, e, n)


    def uint_compare_and_set_value(v: pointer, e: cython.size_t, n: cython.size_t) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """
        return lib.size_t_compare_and_set_value(v, e, n)


    def uint_add_and_fetch(v: pointer, n: cython.size_t) -> cython.size_t:
        """increment and fetch atomically

        :param v: pointer of value to add to and get
        :param n: data to add
        :return: sum of the 2 values
        """
        return lib.size_t_add_and_fetch(v, n)


    def uint_sub_and_fetch(v: pointer, n: cython.size_t) -> cython.size_t:
        """sub and fetch atomically

        :param v: pointer of value to subtract and get
        :param n: data to subtract
        :return: difference of the 2 values
        """
        return lib.size_t_sub_and_fetch(v, n)


    def uint_and_and_fetch(v: pointer, n: cython.size_t) -> cython.size_t:
        """bitwise AND and fetch the result atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: resulted value
        """
        return lib.size_t_and_and_fetch(v, n)


    def uint_or_and_fetch(v: pointer, n: cython.size_t) -> cython.size_t:
        """bitwise XOR and fetch the result atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: resulted value
        """
        return lib.size_t_or_and_fetch(v, n)


    def uint_xor_and_fetch(v: pointer, n: cython.size_t) -> cython.size_t:
        """bitwise XOR and fetch the result atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: resulted value
        """
        return lib.size_t_xor_and_fetch(v, n)


    def uint_nand_and_fetch(v: pointer, n: cython.size_t) -> cython.size_t:
        """bitwise NAND and fetch the result atomically

        :param v: pointer of value to NAND to
        :param n: data to NAND
        :return: resulted value
        """
        return lib.size_t_nand_and_fetch(v, n)


    def uint_fetch_and_add(v: pointer, n: cython.size_t) -> cython.size_t:
        """increment and fetch atomically

        :param v: pointer of value to add to
        :param n: data to add
        :return: original value in v
        """
        return lib.size_t_fetch_and_add(v, n)


    def uint_fetch_and_sub(v: pointer, n: cython.size_t) -> cython.size_t:
        """subtract and fetch atomically

        :param v: pointer of value to subtract from
        :param n: data to subtract
        :return: original value in v
        """
        return lib.size_t_fetch_and_sub(v, n)


    def uint_fetch_and_and(v: pointer, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise AND atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: the result
        """
        return lib.size_t_fetch_and_and(v, n)


    def uint_fetch_and_or(v: pointer, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise OR atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: the result
        """
        return lib.size_t_fetch_and_or(v, n)


    def uint_fetch_and_xor(v: pointer, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.size_t_fetch_and_xor(v, n)


    def uint_fetch_and_nand(v: pointer, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise NAND atomically

        :param v: pointer of value to NAND to
        :param n: data to NAND
        :return: the result
        """
        return lib.size_t_fetch_and_nand(v, n)

    def bool_load(v: pointer) -> cython.bint:
        """get atomically

        :param v: the pointer to the object
        :return: original value
        """
        return lib.bool_load(v)

    def bool_store(v: pointer, n: pointer):
        """Store value atomically

        :param v: the pointer to set
        :param n: the pointer from value to set
        :return: None
        """
        lib.bool_store(v, n)


    def bool_shift(v: pointer, n: pointer, r: pointer):
        """value exchange between 3 pointers in 2 groups atomically, store n in v after store v in r

        :param v: pointer of v
        :param n: pointer of n
        :param r: pointer of r
        :return: None
        """
        lib.bool_shift(v, n, r)


    def bool_get_and_set(v: pointer, n: cython.bint) -> cython.bint:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.bool_get_and_set(v, n)


    def bool_compare_and_set(v: pointer, e: pointer, n: cython.bint) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        the operation is a read and the current contents of v are written into
        e.
        :param v: pointer of v
        :param e: pointer of e
        :param n: value to be set
        :return: whether the contents of v and contents of e is the same
        """
        return lib.bool_compare_and_set(v, e, n)


    def bool_compare_and_set_value(v: pointer, e: cython.bint, n: cython.bint) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """
        #expected = ffi.new("_Bool *", n)
        return lib.bool_compare_and_set(v, e, n)

    def float_store(v: pointer, n: pointer):
        """Store value atomically

         :param v: the pointer to set
         :param n: the pointer from value to set
         :return: None
         """
        lib.double_store(v, n)

else:
    def multiprocessing_win32_init(windows_atomic_object: atomic_object, windows_unix_compatibility,
                                   size: cython.size_t):

        """Initilize the atomic_object for multiprocessing mode.

         :param windows_atomic_object: the atomic_object
         :param windows_unix_compatibility: whether the source code should be compatible with windows x64
         :param size: size of the atomic_object
         :return: None
         """
        windows_atomic_object.mode = 'm'
        windows_atomic_object.windows_unix_compatibility = windows_unix_compatibility
        windows_atomic_object.file_mapping_name = f'{id(windows_atomic_object)}'.encode()
        windows_atomic_object.creation_pid = os.getpid()
        if windows_unix_compatibility:
            windows_atomic_object.file_mapping = lib10.CreateFileMappingAtomic(size, windows_atomic_object.file_mapping_name)
            if windows_atomic_object.file_mapping == ffi.cast("INT_PTR", ffi.NULL):
                raise OSError("Open SharedMemory Failed1!")
            windows_atomic_object.reference = windows_atomic_object.file_mapping_name
        else:
            windows_atomic_object.file_mapping = lib10.CreateFileMappingAtomic(size, windows_atomic_object.file_mapping_name)
            if windows_atomic_object.file_mapping == ffi.cast("INT_PTR", ffi.NULL):
                raise OSError("Open SharedMemory Failed2!")
            windows_atomic_object.current_process_handle = lib10.GetCurrentProcessAtomic()
            windows_atomic_object.reference = lib10.MapViewOfFileAtomic(windows_atomic_object.file_mapping,
                                                                        windows_atomic_object.current_process_handle,
                                                                        ffi.cast("INT_PTR", ffi.NULL),
                                                                        0,
                                                                        0)
            if windows_atomic_object.reference == ffi.cast("INT_PTR", ffi.NULL):
                raise OSError("Open SharedMemory Failed3!")

    def multiprocessing_win32_del(windows_atomic_object: atomic_object):
        """delete the atomic_object in multiproessing mode.

         :param windows_atomic_object: the atomic_object
         :return: None
         """
        if windows_atomic_object.mode == 'm' and windows_atomic_object.creation_pid == os.getpid():
            if not windows_atomic_object.windows_unix_compatibility:
                if lib10.UnmapViewOfFileAtomic(windows_atomic_object.reference, False) == 0:
                    raise OSError("Close SharedMemory Failed1!")
                if lib10.CloseHandleAtomic(windows_atomic_object.current_process_handle) == 0:
                    raise OSError("Close SharedMemory Failed2!")
            if lib10.CloseHandleAtomic(windows_atomic_object.file_mapping) == 0:
                raise OSError("Close SharedMemory Failed3!")


    def get_reference(a: atomic_object) -> reference:
        """get refernce from the shared atomic_object in multiprocessing imcapatible mode.

         :param a: atomic_object
         :return: reference can be distributed in subprocesses
         """
        file_mapping = lib10.OpenFileMappingAtomic(a.file_mapping_name)
        if file_mapping == ffi.cast("INT_PTR", ffi.NULL):
            raise OSError("Open SharedMemory Failed4!")

        process_handle = lib10.GetCurrentProcessAtomic()
        pointer = lib10.MapViewOfFileAtomic(file_mapping,
                                            process_handle,
                                            ffi.cast("INT_PTR", ffi.NULL),
                                            0,
                                            0)
        if pointer == ffi.cast("INT_PTR", ffi.NULL):
            raise OSError("Open SharedMemory Failed5!")

        if lib10.CloseHandleAtomic(process_handle) == 0:
            raise OSError("Open SharedMemory Failed6!")
        return reference(file_mapping, pointer)


    def release_reference(r: reference):
        """release refernce from the reference in multiprocessing imcapatible mode.

         :param r: reference distributed
         :return: None
         """
        if lib10.UnmapViewOfFileAtomic(r.reference, False) == 0:
            raise OSError("release reference Failed1!")
        if lib10.CloseHandleAtomic(r.file_mapping) == 0:
            raise OSError("release reference Failed2!")

    def windows_wrapper(size: int = None):
        """wrapper function  for windows atomic functions.

         :param size: size of the object
         :return: decorated functions
         """
        def outer(f):
            @wraps(f)
            def inner(*args, **kwargs):
                windows_unix_compatibility_inner = kwargs.pop('windows_unix_compatibility', False)
                offset = kwargs.pop('offset', 0)
                # print(kwargs)
                if not windows_unix_compatibility_inner:
                    if size == 8:
                        temp_ptr = ffi.cast("long long *", args[0]) + offset
                    elif size == 4:
                        temp_ptr = ffi.cast("long *", args[0]) + offset
                    elif size == 2:
                        temp_ptr = ffi.cast("short *", args[0]) + offset
                    elif size == 1:
                        temp_ptr = ffi.cast("char *", args[0]) + offset
                    return f(*((temp_ptr,) + args[1:]), **kwargs)
                else:
                    file_mapping: cython.longlong = lib10.OpenFileMappingAtomic(args[0])

                    if file_mapping == ffi.cast("INT_PTR", ffi.NULL):
                        raise OSError("Open SharedMemory Failed6!")

                    current_process_handle: cython.longlong = lib10.GetCurrentProcessAtomic()
                    if size == 8:
                        mv_pointer = ffi.cast('long long *',
                                              lib10.MapViewOfFileAtomic(file_mapping, current_process_handle,
                                                                        ffi.cast("INT_PTR", ffi.NULL),
                                                                        0, 0)) + offset
                    elif size == 4:
                        mv_pointer = ffi.cast('long *',
                                              lib10.MapViewOfFileAtomic(file_mapping, current_process_handle,
                                                                        ffi.cast("INT_PTR", ffi.NULL),
                                                                        0, 0)) + offset
                    elif size == 2:
                        mv_pointer = ffi.cast('short *',
                                              lib10.MapViewOfFileAtomic(file_mapping, current_process_handle,
                                                                        ffi.cast("INT_PTR", ffi.NULL),
                                                                        0, 0)) + offset
                    elif size == 1:
                        mv_pointer = ffi.cast('char *',
                                              lib10.MapViewOfFileAtomic(file_mapping, current_process_handle,
                                                                        ffi.cast("INT_PTR", ffi.NULL),
                                                                        0, 0)) + offset
                    if mv_pointer == ffi.cast("INT_PTR", ffi.NULL):
                        raise OSError("Open SharedMemory Failed7!")

                    result = f(mv_pointer, *(args[1:]), **kwargs)

                    if lib10.UnmapViewOfFileAtomic(ffi.cast('long long ', mv_pointer), False) == 0:
                        raise OSError("Open SharedMemory Failed8!")

                    if lib10.CloseHandleAtomic(current_process_handle) == 0:
                        raise OSError("Open SharedMemory Failed9!")

                    if lib10.CloseHandleAtomic(file_mapping) == 0:
                        raise OSError("Open SharedMemory Failed10!")

                    return result

            return inner
        return outer

    @windows_wrapper(size=8)
    def int_load(v: pointer) -> cython.longlong:
        """get atomically

        :param v: pointer of value to get
        :return: original value
        """
        return lib.ssize_t_load(v)


    @windows_wrapper(size=8)
    def int_get_and_set(v: pointer, n: cython.longlong) -> cython.longlong:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.ssize_t_get_and_set(v, n)


    @windows_wrapper(size=8)
    def int_compare_and_set_value(v: pointer, e: cython.longlong, n: cython.longlong) -> cython.longlong:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """
        return lib.ssize_t_compare_and_set_value(v, e, n)


    @windows_wrapper(size=8)
    def int_add_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """increment and fetch atomically

        :param v: atomic_int to add to and get
        :param n: data to add
        :return: sum of the 2 values
        """
        return lib.ssize_t_add_and_fetch(v, n)


    @windows_wrapper(size=8)
    def int_sub_and_fetch(v: pointer, n: cython.longlong) -> cython.longlong:
        """sub and fetch atomically

        :param v: pointer of value to subtract and get
        :param n: data to subtract
        :return: difference of the 2 values
        """
        return lib.ssize_t_sub_and_fetch(v, n)


    @windows_wrapper(size=8)
    def int_fetch_and_add(v: pointer, n: cython.longlong) -> cython.longlong:
        """increment and fetch atomically

        :param v: pointer of value to add to
        :param n: data to add
        :return: original value in v
        """
        return lib.ssize_t_fetch_and_add(v, n)


    @windows_wrapper(size=8)
    def int_fetch_and_sub(v: pointer, n: cython.longlong) -> cython.longlong:
        """subtract and fetch atomically

        :param v: pointer of value to subtract from
        :param n: data to subtract
        :return: original value in v
        """
        return lib.ssize_t_fetch_and_sub(v, n)


    @windows_wrapper(size=8)
    def int_fetch_and_and(v: pointer, n: cython.longlong) -> cython.longlong:
        """fetch then bitwise AND atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: the result
        """
        return lib.ssize_t_fetch_and_and(v, n)


    @windows_wrapper(size=8)
    def int_fetch_and_or(v: pointer, n: cython.longlong) -> cython.longlong:
        """fetch then bitwise OR atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: the result
        """
        return lib.ssize_t_fetch_and_or(v, n)


    @windows_wrapper(size=8)
    def int_fetch_and_xor(v: pointer, n: cython.longlong) -> cython.longlong:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.ssize_t_fetch_and_xor(v, n)


    @windows_wrapper(size=8)
    def int_bittest_and_set(v: pointer, offset: cython.longlong) -> cython.bint:
        """Tests the specified bit and set it to 1 atomically

        :param offset: offset in the binary presenation of the 64bit integer
        :return: the initial value(True or False) of specified bit"""
        return lib.ssize_t_bittest_and_set(v, offset)


    @windows_wrapper(size=8)
    def int_bittest_and_reset(v: pointer, offset: cython.longlong) -> cython.bint:
        """Tests the specified bit and set it to 0 atomically

        :param offset: offset in the binary presenation of the 64bit integer
        :return: the initial value(True or False) of specified bit
        """
        return lib.ssize_t_bittest_and_reset(v, offset)


    @windows_wrapper(size=4)
    def long_load(v: pointer) -> cython.long:
        """get and set atomically

        :param v: pointer of value to get
        :return: original value
        """
        return lib.long_load(v)


    @windows_wrapper(size=4)
    def long_get_and_set(v: pointer, n: cython.long) -> cython.long:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.long_get_and_set(v, n)


    @windows_wrapper(size=4)
    def long_compare_and_set_value(v: pointer, e: cython.long, n: cython.long) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """
        return lib.long_compare_and_set_value(v, e, n)


    @windows_wrapper(size=4)
    def long_add_and_fetch(v: pointer, n: cython.long) -> cython.long:
        """increment and fetch atomically

        :param v: A pointer to the destination value
        :param n: data to add
        :return: sum of the 2 values
        """
        return lib.long_add_and_fetch(v, n)


    @windows_wrapper(size=4)
    def long_sub_and_fetch(v: pointer, n: cython.long) -> cython.long:
        """sub and fetch atomically

        :param v: pointer of value to subtract and get
        :param n: data to subtract
        :return: difference of the 2 values
        """
        return lib.long_sub_and_fetch(v, n)


    @windows_wrapper(size=4)
    def long_fetch_and_add(v: pointer, n: cython.long) -> cython.long:
        """increment and fetch atomically

        :param v: pointer of value to add to
        :param n: data to add
        :return: original value in v
        """
        return lib.long_fetch_and_add(v, n)


    @windows_wrapper(size=4)
    def long_fetch_and_sub(v: pointer, n: cython.long) -> cython.long:
        """subtract and fetch atomically

        :param v: pointer of value to subtract from
        :param n: data to subtract
        :return: original value in v
        """
        return lib.long_fetch_and_sub(v, n)


    @windows_wrapper(size=4)
    def long_fetch_and_and(v: pointer, n: cython.long) -> cython.long:
        """fetch then bitwise AND atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: the result
        """
        return lib.long_fetch_and_and(v, n)


    @windows_wrapper(size=4)
    def long_fetch_and_or(v: pointer, n: cython.long) -> cython.long:
        """fetch then bitwise OR atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: the result
        """
        return lib.long_fetch_and_or(v, n)


    @windows_wrapper(size=4)
    def long_fetch_and_xor(v: pointer, n: cython.long) -> cython.long:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.long_fetch_and_xor(v, n)


    @windows_wrapper(size=4)
    def long_bittest_and_set(v: pointer, offset: cython.long) -> cython.bint:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.long_bittest_and_set(v, offset)


    @windows_wrapper(size=4)
    def long_bittest_and_reset(v: pointer, offset: cython.long) -> cython.bint:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.long_bittest_and_reset(v, offset)


    @windows_wrapper(size=2)
    def short_load(v: pointer) -> cython.short:
        """get atomically

        :param v: pointer of value to get and set
        :return: original value
        """
        return lib.short_load(v)


    @windows_wrapper(size=2)
    def short_get_and_set(v: pointer, n: cython.short) -> cython.short:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.short_get_and_set(v, n)


    @windows_wrapper(size=2)
    def short_compare_and_set_value(v: pointer, e: cython.short, n: cython.short) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """
        return lib.short_compare_and_set_value(v, e, n)


    @windows_wrapper(size=2)
    def short_add_and_fetch(v: pointer, n: cython.short) -> cython.short:
        """increment and fetch atomically(Not implemented)

        :param v: atomic_int to add to and get
        :param n: data to add
        :return: sum of the 2 values
        """
        raise NotImplementedError("short_add_and_fetch No implementation on windows!")


    @windows_wrapper(size=2)
    def short_sub_and_fetch(v: pointer, n: cython.short) -> cython.short:
        """sub and fetch atomically(Not implemented)

        :param v: pointer of value to subtract and get
        :param n: data to subtract
        :return: difference of the 2 values
        """
        raise NotImplementedError("short_sub_and_fetch No implementation on windows!")


    @windows_wrapper(size=2)
    def short_fetch_and_add(v: pointer, n: cython.short) -> cython.short:
        """increment and fetch atomically

        :param v: pointer of value to add to
        :param n: data to add
        :return: original value in v
        """
        return lib.short_fetch_and_add(v, n)


    @windows_wrapper(size=2)
    def short_fetch_and_sub(v: pointer, n: cython.short) -> cython.short:
        """subtract and fetch atomically

        :param v: pointer of value to subtract from
        :param n: data to subtract
        :return: original value in v
        """
        return lib.short_fetch_and_sub(v, n)


    @windows_wrapper(size=2)
    def short_fetch_and_and(v: pointer, n: cython.short) -> cython.short:
        """fetch then bitwise AND atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: the result
        """
        return lib.short_fetch_and_and(v, n)


    @windows_wrapper(size=2)
    def short_fetch_and_or(v: pointer, n: cython.short) -> cython.short:
        """fetch then bitwise OR atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: the result
        """
        return lib.short_fetch_and_or(v, n)


    @windows_wrapper(size=2)
    def short_fetch_and_xor(v: pointer, n: cython.short) -> cython.short:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.short_fetch_and_xor(v, n)


    @windows_wrapper(size=1)
    def byte_load(v: pointer) -> cython.char:
        """get atomically

        :param v: pointer of value to get and set
        :return: original value
        """
        return int.from_bytes(
            lib.byte_load(v),
            byteorder='big', signed=True)


    @windows_wrapper(size=1)
    def byte_get_and_set(v: pointer, n: cython.char) -> cython.char:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return int.from_bytes(
            lib.byte_get_and_set(v, int.to_bytes(n, length=1, byteorder='big', signed=True)),
            byteorder='big', signed=True)


    @windows_wrapper(size=1)
    def byte_compare_and_set_value(v: pointer, e: cython.char, n: cython.char) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """

        return int.from_bytes(
            lib.byte_compare_and_set_value(v,
                                           int.to_bytes(e, length=1, byteorder='big', signed=True),
                                           int.to_bytes(n, length=1, byteorder='big', signed=True)),
            byteorder='big', signed=True)


    @windows_wrapper(size=1)
    def byte_add_and_fetch(v: pointer, n: cython.char) -> cython.char:
        """increment and fetch atomically(Not implemented)

        :param v: atomic_int to add to and get
        :param n: data to add
        :return: sum of the 2 values
        """
        raise NotImplementedError("No implementation on windows!")


    @windows_wrapper(size=1)
    def byte_sub_and_fetch(v: pointer, n: cython.char) -> cython.char:
        """sub and fetch atomically(Not implemented)

        :param v: pointer of value to subtract and get
        :param n: data to subtract
        :return: difference of the 2 values
        """
        raise NotImplementedError("No implementation on windows!")


    @windows_wrapper(size=1)
    def byte_fetch_and_add(v: pointer, n: cython.char) -> cython.char:
        """increment and fetch atomically

        :param v: pointer of value to add to
        :param n: data to add
        :return: original value in v
        """
        return int.from_bytes(
            lib.byte_fetch_and_add(v, int.to_bytes(n, length=1, byteorder='big', signed=True)),
            byteorder='big', signed=True)


    @windows_wrapper(size=1)
    def byte_fetch_and_sub(v: pointer, n: cython.char) -> cython.char:
        """subtract and fetch atomically

        :param v: pointer of value to subtract from
        :param n: data to subtract
        :return: original value in v
        """
        return int.from_bytes(
            lib.byte_fetch_and_sub(v, int.to_bytes(n, length=1, byteorder='big', signed=True)),
            byteorder='big', signed=True)


    @windows_wrapper(size=1)
    def byte_fetch_and_and(v: pointer, n: cython.char) -> cython.char:
        """fetch then bitwise AND atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: the result
        """
        return int.from_bytes(
            lib.byte_fetch_and_and(v, int.to_bytes(n, length=1, byteorder='big', signed=True)),
            byteorder='big', signed=True)


    @windows_wrapper(size=1)
    def byte_fetch_and_or(v: pointer, n: cython.char) -> cython.char:
        """fetch then bitwise OR atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: the result
        """
        return int.from_bytes(
            lib.byte_fetch_and_or(v, int.to_bytes(n, length=1, byteorder='big', signed=True)),
            byteorder='big', signed=True)


    @windows_wrapper(size=1)
    def byte_fetch_and_xor(v: pointer, n: cython.char) -> cython.char:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return int.from_bytes(
            lib.byte_fetch_and_xor(v, int.to_bytes(n, length=1, byteorder='big', signed=True)),
            byteorder='big', signed=True)


    @windows_wrapper(size=4)
    def bool_load(v: pointer) -> cython.bint:
        """get atomically

        :param v: the pointer to the object
        :return: original value
        """
        return bool(lib.bool_load(v))


    @windows_wrapper(size=4)
    def bool_get_and_set(v: pointer, n: cython.bint) -> cython.bint:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.bool_get_and_set(v, n)


    @windows_wrapper(size=4)
    def bool_compare_and_set_value(v: pointer, e: cython.bint, n: cython.bint) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal, no operation will be taken.

        :param v: A pointer to the destination value
        :param e: The exchange value
        :param n: The value to compare to v
        :return: initial value of the v
        """
        return lib.bool_compare_and_set_value(v, e, n)


    def get_bool(v: pointer) -> cython.bint:
        """get the bool value from the long pointer.

        :param v: A pointer to the destination value
        :return:  value of the v
        """
        result = ffi.cast("long *", v)
        return bool(result[0])


    def get_int(v: pointer) -> cython.ssize_t:
        """get the int value from the long long pointer.

        :param v: A pointer to the destination value
        :return:  value of the v
        """
        result = ffi.cast("long long *", v)
        return result[0]


    from typing import TypeVar

    type_atomic_bytearray = TypeVar("type_atomic_bytearray", bound="shared_atomic.atomic_bytearray")
    type_atomic_set = TypeVar("type_atomic_set", bound="shared_atomic.atomic_set")
    type_atomic_list = TypeVar("type_atomic_list", bound="shared_atomic.atomic_list")
    type_atomic_string = TypeVar("type_atomic_string", bound="shared_atomic.atomic_string")
    type_atomic_shared_memory = TypeVar("type_atomic_shared_memory", bound="shared_atomic.atomic_shared_memory")


    def array_bittest_and_set(array: type_atomic_bytearray, reference, offset: cython.char) -> cython.bint:
        r"""
        bittest_and_set the original contents in the bytearray

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param offset: offset from counted by bits from the left.
        :return: original value(True or False) at the offset
        """
        if array.size not in (4, 8):
            raise NotImplementedError("No implementation for bytearray other than 4 or 8 bytes!")
        return bool(array._array_bittest_and_set(reference, offset,
                                                 windows_unix_compatibility=array.windows_unix_compatibility
                                                 ))


    def array_bittest_and_reset(array: type_atomic_bytearray, reference, offset: cython.char) -> cython.bint:
        r"""
        bittest_and_reset the original contents in the bytearray

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param offset: offset from counted by bits from the left.
        :return: original value(True or False) at the offset
        """
        if array.size not in (4, 8):
            raise NotImplementedError("No implementation for bytearray other than 4 or 8 bytes!")
        return bool(array._array_bittest_and_reset(reference, offset,
                                                   windows_unix_compatibility=array.windows_unix_compatibility
                                                   ))


    def array_get_and_set(array: type_atomic_bytearray, reference, data: bytes, trim=True) -> bytes:
        r"""
        Get and set atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param data: new data
        :param trim: whether of not to trim the returning b'\\0' when get, default True
        :return: the original bytes
        """

        integer: cython.longlong

        integer = int.from_bytes(data, byteorder='big', signed=True)
        result = int.to_bytes(
            array._array_get_and_set(reference, integer, windows_unix_compatibility=array.windows_unix_compatibility
                                     ),
            length=array.size, byteorder='big', signed=True)
        array.initial_length = len(data)
        return result.lstrip(b'\0') if trim else result


    def array_compare_and_set_value(array: type_atomic_bytearray, reference, i: bytes, n: bytes, trim=True) -> bytes:
        """
        Compare and set atomically,This compares the contents of self
        with the contents of i. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        the operation is a read and the current contents of itself are written into i.

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param i: The exchange value
        :param n: The value to compare to
        :param trim: whether the returned bytes should be trimmed of tailing b'\0'
        :return: Original value
        """
        integer_i: cython.longlong
        integer_n: cython.longlong

        if array.size != len(i) or len(i) != len(n):
            raise ValueError("array, i and n has different size!")
        integer_i = int.from_bytes(i, byteorder='big', signed=True)
        integer_n = int.from_bytes(n, byteorder='big', signed=True)
        result = array._array_compare_and_set_value(reference,
                                                    integer_i, integer_n,
                                                    windows_unix_compatibility=array.windows_unix_compatibility
                                                    )

        result = int.to_bytes(result, length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_add_and_fetch(array: type_atomic_bytearray, reference, n: bytes, trim=True) -> bytes:
        r"""
        Increment and fetch atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param n: bytes will be added to the array.
        :param trim: whether of not to trim the returning b'\\0' when fetch, default True
        :return: the contents of resulted bytearray
        """
        integer: cython.longlong

        integer = int.from_bytes(n, byteorder='big', signed=True)
        result = int.to_bytes(array._array_add_and_fetch(reference, integer,
                                                         windows_unix_compatibility=array.windows_unix_compatibility
                                                         ),
                              length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_sub_and_fetch(array: type_atomic_bytearray, reference, n: bytes, trim=True) -> bytes:
        r"""
        Decrement and fetch atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param n: bytes will be subtracted from the array.
        :param trim: whether of not to trim the returning b'\\0' when fetch, default True
        :return: the contents of resulted bytearray
        """
        integer: cython.longlong

        integer = int.from_bytes(n, byteorder='big', signed=True)
        result = int.to_bytes(array._array_sub_and_fetch(reference, integer,
                                                         windows_unix_compatibility=array.windows_unix_compatibility
                                                         ),
                              length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_fetch_and_add(array: type_atomic_bytearray, reference, n: bytes, trim=True) -> bytes:
        r"""
        fetch and increment atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param n: the bytes will be added to the array
        :param trim: whether of not to trim the returning b'\\0' when fetch, default True
        :return: the original contents of the bytearray
        """
        integer: cython.longlong

        integer = int.from_bytes(n, byteorder='big', signed=True)
        result = int.to_bytes(array._array_fetch_and_add(reference, integer,
                                                         windows_unix_compatibility=array.windows_unix_compatibility
                                                         ),
                              length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_fetch_and_sub(array: type_atomic_bytearray, reference, n: bytes, trim=True) -> bytes:
        r"""
        fetch and decrement atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param n: the bytes will be substracted from the array
        :param trim: whether of not to trim the returning b'\\0' when fetch, default True
        :return: the original contents of the bytearray
        """
        integer: cython.longlong

        integer = int.from_bytes(n, byteorder='big', signed=True)
        result = int.to_bytes(array._array_fetch_and_sub(reference, integer,
                                                         windows_unix_compatibility=array.windows_unix_compatibility
                                                         ),
                              length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_fetch_and_and(array: type_atomic_bytearray, reference, n: bytes, trim=True) -> bytes:
        r"""
        Fetch then bitwise AND atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param n: the other operands of AND operation
        :param trim: whether of not to trim the returning b'\\0' when fetch, default True
        :return: the original contents of the bytearray
        """
        integer: cython.longlong

        integer = int.from_bytes(n, byteorder='big', signed=True)
        result = int.to_bytes(array._array_fetch_and_and(reference, integer,
                                                         windows_unix_compatibility=array.windows_unix_compatibility
                                                         ),
                              length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_fetch_and_or(array: type_atomic_bytearray, reference, n: bytes, trim=True) -> bytes:
        r"""
        Fetch then bitwise OR atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param n: the other operands of OR operation
        :param trim: whether of not to trim the returning b'\\0' when fetch, default True
        :return: the original contents of the bytearray
        """
        integer: cython.longlong

        integer = int.from_bytes(n, byteorder='big', signed=True)
        result = int.to_bytes(array._array_fetch_and_or(reference, integer,
                                                        windows_unix_compatibility=array.windows_unix_compatibility
                                                        ),
                              length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_fetch_and_xor(array: type_atomic_bytearray, reference, n: bytes, trim=True) -> bytes:
        r"""
        Fetch then bitwise XOR atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param n: the other operands of XOR operation
        :param trim: whether of not to trim the returning b'\\0' when fetch, default True
        :return: the original contents of the bytearray
        """
        integer: cython.longlong

        integer = int.from_bytes(n, byteorder='big', signed=True)
        result = int.to_bytes(array._array_fetch_and_xor(reference, integer,
                                                         windows_unix_compatibility=array.windows_unix_compatibility
                                                         ),
                              length=array.size, byteorder='big', signed=True)
        return result.lstrip(b'\0') if trim else result


    def array_get_int(array: type_atomic_bytearray, reference) -> cython.size_t:
        """
        Get the integer representation from the bytearray,

        the whole array would be treated as a large integer
        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :return: the integer representation
        """
        return array.signed2unsigned(array._array_load(reference,
                                                       windows_unix_compatibility=array.windows_unix_compatibility
                                                       ))


    def array_get_bytes(array: type_atomic_bytearray, reference, trim=True) -> bytes:
        r"""
        Get all the bytes from the bytearray atomically

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param trim: if True, the leading b'\\0' would be trimmed, by default: True

        :return: all the bytes in the bytearray
        """

        result = array._array_load(reference,
                                   windows_unix_compatibility=array.windows_unix_compatibility
                                   )
        if trim:
            result = int.to_bytes(result, length=array.size, byteorder='big', signed=True).lstrip(b'\0')
        else:
            result = int.to_bytes(result, length=array.size, byteorder='big', signed=True)
        return result


    def array_set_bytes(array: type_atomic_bytearray, reference, data: bytes):
        """
        Set the value in the bytearray,
        if the new data is longer than the original size of the array.
        it will expand the array accordingly which would lose atomicy.
        the size of the bytearray can be checked with array.size

        :param array: target array to change.
        :param reference: reference distributed from the parent process.
        :param data: input bytearray
        :return: None
        """

        desiredlength: cython.char
        integer: cython.longlong
        integer = int.from_bytes(data, byteorder='big', signed=True)
        desiredlength = len(data)
        if 8 >= desiredlength > array.size:
            array.resize(desiredlength)
            array._array_get_and_set(array.reference, integer,
                                     windows_unix_compatibility=array.windows_unix_compatibility
                                     )
            array.initial_length = desiredlength
            return
        elif desiredlength > 8:
            raise ValueError("data too long for atomic_array!")
        array._array_get_and_set(reference, integer,
                                 windows_unix_compatibility=array.windows_unix_compatibility
                                 )
        array.initial_length = desiredlength


    def set_get_int(set: type_atomic_set, reference) -> cython.size_t:
        """Get the whole integer representation from the set,
        the whole set would be treated as a large integer

        :param set: target set to change.
        :param reference: reference distributed from the parent process.
        :return: the integer representation
        """
        return set.signed2unsigned(set._array_load(reference,
                                                   windows_unix_compatibility=set.windows_unix_compatibility

                                                   ))


    def set_set_int(set: type_atomic_set, reference, integer: cython.size_t):
        """
        Set the whole integer representation from the set,
        the whole set would be treated as a large integer

        :param set: target set to change.
        :param reference: reference distributed from the parent process.
        :return: None
        """
        set._array_get_and_set(reference, set.unsigned2signed(integer),
                               windows_unix_compatibility=set.windows_unix_compatibility

                               )


    def set_get_set(set: type_atomic_set, reference) -> set:
        r"""
        Get the set atomically
        :param set: target set to change.
        :param reference: reference distributed from the parent process.

        :return: the set
        """
        result = int.to_bytes(set._array_load(reference,
                                              windows_unix_compatibility=set.windows_unix_compatibility

                                              ),
                              length=set.size, byteorder='big', signed=True)
        return set.decode(result)


    def set_set_set(set: type_atomic_set, reference, data: set):
        """
        Set the value in the set,
        if the new data is longer than the original size of the set.
        it will expand the set accordingly which would lose atomicy.
        the size of the set can be check with self.size

        :param set: target set to change.
        :param reference: reference distributed from the parent process.
        :param data: input set
        :return: None
        """
        integer: cython.size_t
        desiredlength: cython.char

        integer, desiredlength = set.encode(data)

        if 8 >= desiredlength > set.size:
            set.__init__(data, mode=set.mode, encoding=set.encoding,
                         windows_unix_compatibility=set.windows_unix_compatibility)
        else:
            set._array_get_and_set(reference, set.unsigned2signed(integer),
                                   windows_unix_compatibility=set.windows_unix_compatibility

                                   )


    def set_get_and_set(set: type_atomic_set, reference, data: set) -> set:
        r"""
        Get and set atomically

        :param set: target set to change.
        :param reference: reference distributed from the parent process.
        :param data: new data set
        :return: the original set
        """
        integer: cython.size_t
        desiredlength: cython.char

        integer, length = set.encode(data)
        # if length > self.size:
        #    raise ValueError("Input data too long cannot be set atomically!")
        result = int.to_bytes(set._array_get_and_set(reference, set.unsigned2signed(integer),
                                                     windows_unix_compatibility=set.windows_unix_compatibility

                                                     ),
                              length=set.size, byteorder='big', signed=True)
        return set.decode(result)


    def set_compare_and_set_value(set: type_atomic_set, reference, i: set, n: set) -> cython.set:
        """
        Compare and set atomically,This compares the contents of self
        with the contents of i. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        the operation is a read and the current contents of itself are written into i.

        :param set: target set to change.
        :param reference: reference distributed from the parent process.
        :param i: The exchange value
        :param n: The value to compare to
        :return: Original value
        """
        integer_i: cython.size_t
        integer_n: cython.size_t

        length_i: cython.char
        length_n: cython.char

        integer_n, length_n = set.encode(n)
        integer_i, length_i = set.encode(i)

        if set.size < length_i or set.size < length_n:
            raise ValueError("set i or set n is longer than set.size!")

        # if length > self.size:
        #    raise ValueError("Input data too large, cannot set atomically!")
        result = int.to_bytes(set._array_compare_and_set_value(reference,
                                                               set.unsigned2signed(integer_i),
                                                               set.unsigned2signed(integer_n),
                                                               windows_unix_compatibility=set.windows_unix_compatibility

                                                               ),
                              length=set.size, byteorder='big', signed=True)
        return set.decode(result)


    def list_get_int(list: type_atomic_list, reference) -> cython.size_t:
        """
        Get the whole integer representation from the list,
        the whole set would be treated as a large integer

        :param list: target list to change.
        :param reference: reference distributed from the parent process.
        :return: the integer representation
        """
        return list.signed2unsigned(list._array_load(reference,
                                                     windows_unix_compatibility=list.windows_unix_compatibility

                                                     ))


    def list_set_int(list: type_atomic_list, reference, integer: cython.size_t):
        """
        Set the whole integer representation from the set,
        the whole set would be treated as a large integer

        :param list: target list to change.
        :param reference: reference distributed from the parent process.
        :return: None
        """
        list._array_get_and_set(reference, list.unsigned2signed(integer),
                                windows_unix_compatibility=list.windows_unix_compatibility

                                )


    def list_get_list(list: type_atomic_list, reference) -> list:
        r"""
        Get the list atomically
        :param list: target list to change.
        :param reference: reference distributed from the parent process.
        :return: the list
        """
        result = int.to_bytes(list._array_load(reference,
                                               windows_unix_compatibility=list.windows_unix_compatibility

                                               ),
                              length=list.size, byteorder='big', signed=True)
        return list.decode(result)


    def list_set_list(list: type_atomic_list, reference, data: list):
        """
        Set the value in the list,
        if the new data is longer than the original size of the list.
        it will expand the set accordingly which would lose atomicy.
        the size of the set can be checked with list.size
        :param list: target list to change.
        :param reference: reference distributed from the parent process.
        :param data: input list
        :return: None
        """
        integer: cython.size_t
        desiredlength: cython.char

        integer, desiredlength = list.encode(data)

        if 8 >= desiredlength > list.size:
            list.__init__(data, mode=list.mode, encoding=list.encoding,
                          windows_unix_compatibility=list.windows_unix_compatibility)
        else:
            list._array_get_and_set(reference, list.unsigned2signed(integer),
                                    windows_unix_compatibility=list.windows_unix_compatibility
                                    )


    def list_get_and_set(list: type_atomic_list, reference, data: list) -> list:
        r"""
        Get and set atomically
        :param list: target list to change.
        :param reference: reference distributed from the parent process.
        :param data: new data list
        :return: the original list
        """
        integer: cython.size_t
        desiredlength: cython.char

        integer, length = list.encode(data)
        # if length > self.size:
        #    raise ValueError("Input data too long cannot be set atomically!")
        result = int.to_bytes(list._array_get_and_set(reference, list.unsigned2signed(integer),
                                                      windows_unix_compatibility=list.windows_unix_compatibility
                                                      ),
                              length=list.size, byteorder='big', signed=True)
        return list.decode(result)


    def list_compare_and_set_value(list: type_atomic_list, reference, i: list, n: list) -> list:
        """
        Compare and set atomically,This compares the contents of self
        with the contents of i. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        the operation is a read and the current contents of itself are written into i.

        :param list: target list to change.
        :param reference: reference distributed from the parent process.
        :param i: The exchange value
        :param n: The value to compare to
        :return: Original value
        """
        integer_n: cython.size_t
        integer_i: cython.size_t
        length_n: cython.char
        length_i: cython.char

        integer_n, length_n = list.encode(n)
        integer_i, length_i = list.encode(i)

        if list.size < length_i or list.size < length_n:
            raise ValueError("set i or set n is longer than set.size!")

        # if length > self.size:
        #    raise ValueError("Input data too large, cannot set atomically!")
        result = list._array_compare_and_set_value(reference
                                                   , list.unsigned2signed(integer_i), list.unsigned2signed(integer_n),
                                                   windows_unix_compatibility=list.windows_unix_compatibility
                                                   )
        return list.decode(int.to_bytes(result, length=list.size, byteorder='big', signed=True))


    def string_get_int(string: type_atomic_list, reference) -> cython.size_t:
        """
        Get the integer representation from the string atomically,
        the whole string would be treated as a large integer

        :param string: target string to change.
        :param reference: reference distributed from the parent process.
        :return: the integer representation
        """
        return string.signed2unsigned(string._array_load(reference,
                                                         windows_unix_compatibility=string.windows_unix_compatibility
                                                         ))


    def string_set_int(string: type_atomic_list, reference, input: cython.size_t):
        """
        Set the integer representation of the string atomically,
        the whole array would be treated as a large integer

        :param string: target string to change.
        :param reference: reference distributed from the parent process.
        :return: None
        """
        string._array_get_and_set(reference, string.unsigned2signed(input),
                                  windows_unix_compatibility=string.windows_unix_compatibility
                                  )


    def string_set_bytes(string: type_atomic_list, reference, data: bytes):
        """
            Set the bytes value in the string,
            if the new data is longer than the original size of the string.
            it will expand the string accordingly which would lose atomicy.
            the size of the string can be check with self.size

            :param string: target string to change.
            :param reference: reference distributed from the parent process.
            :param data: input string
            :return: None
        """

        desiredlength: cython.char
        integer: cython.size_t

        desiredlength = len(data)

        if 7 >= desiredlength > string.size - 1:
            string.resize(desiredlength)
            integer = string._byte2int(data)
            string_set_int(string, string.reference, integer)
            return
        elif desiredlength > 7:
            raise ValueError("data too long for atomic_string!")
        else:
            integer = string._byte2int(data)
            string_set_int(string, reference, integer)


    def string_get_string(string: type_atomic_string, reference) -> str:
        r"""
         Get all the bytes from the string atomically

        :param string: target string to change.
        :param reference: reference distributed from the parent process.
        :return: all the full string
        """

        result_bytes = int.to_bytes(string._array_load(reference,
                                                       windows_unix_compatibility=string.windows_unix_compatibility
                                                       ), length=string.size, byteorder='big',
                                    signed=True)
        length = int.from_bytes(result_bytes[0:1], byteorder='big')
        return (result_bytes[1:length + 1]).decode(string.encoding)


    def string_set_string(string: type_atomic_string, reference, data: str):
        """
            Set the value in the string,
            if the new data is longer than the original size of the string.
            it will expand the string accordingly which would lose atomicy.
            the size of the string can be checked with self.size

            :param string: target string to change.
            :param reference: reference distributed from the parent process.
            :param data: input string
            :return: None
        """
        data_bytes = data.encode(string.encoding)
        string_set_bytes(string, reference, data_bytes)


    def string_get_and_set(string: type_atomic_string, reference, data: str) -> str:
        r"""
        Get and set atomically

        :param string: target string to change.
        :param reference: reference distributed from the parent process.
        :param data: new data
        :return: the original string
        """
        integer: cython.size_t
        result_length: cython.char

        integer = string._str2int(data)
        result = int.to_bytes(string._array_get_and_set(reference, string.unsigned2signed(integer),
                                                        windows_unix_compatibility=string.windows_unix_compatibility
                                                        ),
                              length=string.size, byteorder='big', signed=True)
        result_length = int.from_bytes(result[:1], byteorder='big')
        result_all = result[1:result_length + 1]
        return result_all.decode(string.encoding)


    def string_compare_and_set_value(string: type_atomic_string, reference, i: str, n: str) -> str:
        """
        Compare and swap atomically, This compares the contents of self
        with the contents of n. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        No operation is performed.

        :param string: target string to change.
        :param reference: reference distributed from the parent process.
        :param i: exchange value
        :param n: The value to be compared with
        :return: the initial value of self
        """
        integer_n: cython.size_t
        integer_i: cython.size_t

        integer_n = string._str2int(n)
        integer_i = string._str2int(i)

        result_bytes = int.to_bytes(string._array_compare_and_set_value(reference,
                                                                        string.unsigned2signed(integer_i),
                                                                        string.unsigned2signed(integer_n),
                                                                        windows_unix_compatibility=string.windows_unix_compatibility
                                                                        ), length=string.size,
                                    byteorder='big', signed=True)
        length = int.from_bytes(result_bytes[0:1], byteorder='big')
        return (result_bytes[1:length + 1]).decode(string.encoding)



    def shared_memory_offset_get(memory: type_atomic_shared_memory, reference, offset: cython.longlong = 0,
                                 length: cython.char = 1) -> bytes:
        """Atomically get the bytes at specific offset given specific length

        :param memory: target shared memory to change.
        :param reference: reference distributed from the parent process.
        :param offset: the offset inside the shared memory starting from 0, including the operation length
        :param length: the length of bytes should be retrieved
        :return: bytes at specific offset.
        """
        if offset < memory.size:
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            get_operation_length: cython.char = memory.runtime_properties._get_operation_length
            if offset % get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            result_atomic: cython.longlong = memory.runtime_properties._array_load(
                reference,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                offset=offset // get_operation_length)
            result = int.to_bytes(result_atomic, length=get_operation_length, byteorder='big', signed=True)[-length:]
            memory.runtime_properties = None
            return result

        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_get_and_set(memory: type_atomic_shared_memory, reference, value: bytes,
                                         offset: cython.longlong = 0) -> bytes:
        """Atomically set the bytes at specific offset given specific length

        :param memory: target shared memory to change.
        :param reference: reference distributed from the parent process.
        :param value: new value in bytes
        :param offset: the offset inside the shared memory starting from 0, including the operation length
        :return: bytes at specific offset previously.
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            result = int.to_bytes(
                memory.runtime_properties._array_get_and_set(
                    reference,
                    int.from_bytes(value, byteorder='big',signed=True),
                    windows_unix_compatibility=memory.windows_unix_compatibility
                    ,
                    offset=offset // memory.runtime_properties._get_operation_length),
                length=length, byteorder='big', signed=True)
            memory.runtime_properties = None
            return result


        else:
            raise ValueError("Offset overflow!")

    @cython.returns(bytes)
    @cython.ccall
    def shared_memory_offset_compare_and_set_value(memory: type_atomic_shared_memory, reference, i: bytes,
                                                   n: bytes,
                                                   offset: cython.size_t = 0) -> bytes:
        """Compare and set atomically. This compares the contents of pointer with the contents of self at offset. If equal, the operation is a read-modify-write operation that writes bytes parameter value into self. If they are not equal, the operation is a read and the current contents of self are written into pointer.

        :param memory: target shared memory to change.
        :param reference: reference distributed from the parent process.
        :param i: exchange value
        :param n: The value to be compared with
        :param offset: the offset inside the shared memory starting from 0 you need to compare and set,
        :return: the initial value of memory
        """
        if offset < memory.size:
            length: cython.char = len(i)
            if length != len(n):
                raise ValueError(" i and n are different in length!")
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            n_integer: cython.longlong = int.from_bytes(n, byteorder='big', signed=True)
            i_integer: cython.longlong = int.from_bytes(i, byteorder='big', signed=True)

            result:bytes = memory.runtime_properties._array_compare_and_set_value(
                        reference,
                        i_integer,
                        n_integer,
                        windows_unix_compatibility=memory.windows_unix_compatibility
                        ,
                        offset=offset // memory.runtime_properties._get_operation_length).\
                    to_bytes(length=length, byteorder='big',signed=True)
            memory.runtime_properties = None
            return result
        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_add_and_fetch(memory: type_atomic_shared_memory, reference, value: bytes,
                                           offset: cython.size_t = 0) -> bytes:
        """increment and fetch atomically

        :param memory: target shared memory to change.
        :param reference: reference distributed from the parent process.
        :param value: data to add
        :param offset: the offset inside the shared memory starting from 0 you need to add,
        :return: sum of the 2 values
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            integer: cython.longlong = int.from_bytes(value, byteorder='big', signed=True)
            result = memory.runtime_properties._array_add_and_fetch(
                reference,
                integer,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                offset=offset // memory.runtime_properties._get_operation_length).\
                    to_bytes(length=length, byteorder='big',signed=True)
            memory.runtime_properties = None
            return result
        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_sub_and_fetch(memory: type_atomic_shared_memory, reference, value: bytes,
                                           offset: cython.size_t = 0) -> bytes:
        """substract and fetch atomically

        :param memory: target shared memory to change.
        :param reference: reference distributed from the parent process.
        :param value: data to substract
        :param offset: the offset inside the shared memory starting from 0 you need to add,
        :return: difference of the 2 values
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            integer: cython.longlong = int.from_bytes(value, byteorder='big', signed=True)
            result = memory.runtime_properties._array_sub_and_fetch(
                reference,
                integer,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                offset=offset // memory.runtime_properties._get_operation_length). \
                to_bytes(length=length, byteorder='big', signed=True)
            memory.runtime_properties = None
            return result
        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_fetch_and_add(memory: type_atomic_shared_memory, reference, value: bytes,
                                           offset: cython.size_t = 0) -> bytes:
        """fetch and increment atomically

        :param memory: target shared memory to change.
        :param reference: reference distributed from the parent process.
        :param value: data to add
        :param offset: the offset inside the shared memory starting from 0 you need to add to,
        :return: original value
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            integer: cython.longlong = int.from_bytes(value, byteorder='big', signed=True)
            result = memory.runtime_properties._array_fetch_and_add(
                reference,
                integer,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                offset=offset // memory.runtime_properties._get_operation_length). \
                to_bytes(length=length, byteorder='big', signed=True)
            memory.runtime_properties = None
            return result

        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_fetch_and_sub(memory: type_atomic_shared_memory, reference, value: bytes,
                                           offset: cython.size_t = 0) -> bytes:
        """fetch and substract atomically

        :param value: data to add
        :param offset: the offset inside the shared memory starting from 0 you need to substract from,
        :return: original value
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            integer: cython.longlong = int.from_bytes(value, byteorder='big', signed=True)
            result = memory.runtime_properties._array_fetch_and_sub(
                reference,
                integer,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                offset=offset // memory.runtime_properties._get_operation_length
            ). \
                to_bytes(length=length, byteorder='big', signed=True)
            memory.runtime_properties = None
            return result
        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_fetch_and_and(memory: type_atomic_shared_memory, reference, value: bytes,
                                           offset: cython.size_t = 0) -> bytes:
        """fetch then bitwise AND atomically

        :param value: value to AND to
        :param offset: the offset inside the shared memory starting from 0 you need to AND to,
        :return: original value
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            integer: cython.longlong = int.from_bytes(value, byteorder='big', signed=True)
            result = memory.runtime_properties._array_fetch_and_and(
                reference,
                integer,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                    offset=offset // memory.runtime_properties._get_operation_length). \
                to_bytes(length=length, byteorder='big', signed=True)
            memory.runtime_properties = None
            return result
        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_fetch_and_or(memory: type_atomic_shared_memory, reference, value: bytes,
                                          offset: cython.size_t = 0) -> bytes:
        """fetch then bitwise OR atomically

        :param value: value to OR with
        :param offset: the offset inside the shared memory starting from 0 you need to OR with,
        :return: original value
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            integer: cython.longlong = int.from_bytes(value, byteorder='big', signed=True)
            result = memory.runtime_properties._array_fetch_and_or(
                reference,
                integer,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                    offset=offset // memory.runtime_properties._get_operation_length). \
                to_bytes(length=length, byteorder='big', signed=True)
            memory.runtime_properties = None
            return result
        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_fetch_and_xor(memory: type_atomic_shared_memory, reference, value: bytes,
                                           offset: cython.size_t = 0) -> bytes:
        """fetch then bitwise XOR atomically

        :param value: value to XOR with
        :param offset: the offset inside the shared memory starting from 0 you need to XOR with,
        :return: original value
        """
        if offset < memory.size:
            length: cython.char = len(value)
            check_length(length)
            memory.set_atomic_methods(length)
            offset = memory.size - offset - length
            if offset % memory.runtime_properties._get_operation_length != 0:
                raise NotImplementedError("offset should be multiple of length to finish the atomic operation!")
            integer: cython.longlong = int.from_bytes(value, byteorder='big', signed=True)
            result = memory.runtime_properties._array_fetch_and_xor(
                reference,
                integer,
                windows_unix_compatibility=memory.windows_unix_compatibility
                ,
                    offset=offset // memory.runtime_properties._get_operation_length). \
                to_bytes(length=length, byteorder='big', signed=True)
            memory.runtime_properties = None
            return result

        else:
            raise ValueError("Offset overflow!")

    def shared_memory_offset_bittest_and_set(memory: type_atomic_shared_memory, reference, left_offset: cython.size_t) -> cython.bint:
        """test and set the bit at offset atomically

        :param left_offset: the offset inside the shared memory starting from 0 you need to set from,
        :return: original value
        """

        if left_offset < memory.size * 8:
            if memory.size >= 4:
                offset = memory.size * 8 - left_offset
                division, remainder = divmod(offset, 32)
                return bool(long_bittest_and_set(reference, 32 - remainder,
                                                 windows_unix_compatibility=memory.windows_unix_compatibility,
                                                 offset=division))
            else:
                raise NotImplementedError("size too small, not supported!")
        else:
            raise ValueError("Offset overflow!")


    def shared_memory_offset_bittest_and_reset(memory: type_atomic_shared_memory, reference, left_offset: cython.size_t) -> cython.bint:
        """test and reset the bit at offset atomically

        :param left_offset: the offset inside the shared memory starting from 0 you need to set from,
        :return: original value
        """
        if left_offset < memory.size * 8:
            if memory.size >= 4:
                offset = memory.size * 8 - left_offset
                division, remainder = divmod(offset, 32)
                return bool(long_bittest_and_reset(reference, 32 - remainder,
                                                   windows_unix_compatibility=memory.windows_unix_compatibility,
                                                   offset=division))
            else:
                raise NotImplementedError("size too small, not supported!")
        else:
            raise ValueError("Offset overflow!")