import sys
import ctypes
import os
import cython
from typing import Iterable
from typing import TypeVar
import math
from bitarray import bitarray
from bitarray.util import ba2hex, ba2int, int2ba

from shared_atomic.atomic_decryption_ import ffi
from shared_atomic.atomic_decryption_ import lib as lib_decryption


def sha1(message: bytes) -> str:
    """calculate sha1(internally used)
    :return: calculated sha1
    """
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")
    if lib_decryption.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib_decryption.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join(
            [i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")


if sha1(b'===randomstring===') != '===randomstringsha1===':
    raise ValueError("Didn't pass checksum!")

if sys.platform in ("linux", "darwin", "win32"):
    import shared_atomic.shared_atomic_
    from shared_atomic.shared_atomic_ import lib
    from shared_atomic.shared_atomic_ import ffi

    with open(shared_atomic.shared_atomic_.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomiclibsha1===':
            raise ValueError("Didn't pass checksum!")

    import shared_atomic.atomic_object
    with open(shared_atomic.atomic_object.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomic_object_libsha1===':
            raise ValueError("Didn't pass checksum3!")

    from shared_atomic.atomic_object import *
    from shared_atomic import mmap_flags

else:
    raise NotImplementedError("Unsupported platform!")

if sys.platform == "win32":
    import platform

    if platform.win32_ver()[0] + '.' + platform.win32_ver()[2] >= '10.SP0':
        from shared_atomic.shared_atomic_win10_ import lib as lib10
    with open(shared_atomic.shared_atomic_win10_.__file__, 'rb') as f:
        shared_atomic_win10_bytes = f.read()
        if sha1(shared_atomic_win10_bytes) != '===atomicwin10libsha1===':
            raise ValueError("Didn't pass checksum!")

    from shared_atomic.atomic_object import *

class atomic_set(atomic_object):
    """
    set provide atomic operations, the set should be no longer than 8 bytes
    :attributes
        encoding: readonly string character set\n
        initial_length: readonly total length in bits\n
        value: read/write value of the bytearray\n
        int_value: readonly int value\n
        size: readonly size of the set\n
        mode: readonly 's' for single process, 'm' for multiprocessing, on windows platform, only singleprocessing is supported\n
        windows_unix_compatibility: readonly whether the source code should be compatible with windows x64
    """
    encoding = cython.declare(str, visibility='readonly')
    size = cython.declare(cython.char, visibility='readonly')
    initial_length = cython.declare(cython.char, visibility='readonly')
    mode = cython.declare(str,visibility='readonly')
    windows_unix_compatibility = cython.declare(cython.bint, visibility='readonly')

    def __init__(self, initial: Iterable,
                 encoding='utf-8',
                 mode='singleprocessing',
                 windows_unix_compatibility: bool = False):
        r"""
        constructor to initialize the set,
        the set should be no longer than 8 bytes

        :param initial: initial value of the set, if the initial value is longer than 8 bytes, please specify the trimming target length, or else it would fail.
        :param mode: the mode in which the set will be shared. 'singleprocessing' or 's' for single process, 'multiprocessing' or 'm' for multiprocessing, on windows platform, only singleprocessing is supported, setting it to 'm' or 'multiprocessing' will be ignored.
        :param encoding: , character set, default 'utf-8'
        """
        super().__init__()

        full_data: cython.size_t
        self.initial_length: cython.char

        self.encoding = encoding
        initial_set = set(initial)

        full_data, self.initial_length = self.encode(initial_set)


        if self.initial_length <= 4:
            if sys.platform in ('darwin', 'linux'):
                self._type = "unsigned int"
                self.size = 4
                self._array_load = lib.uint_load
                self._array_store = lib.uint_store
                self._array_get_and_set = lib.uint_get_and_set
                self._array_shift = lib.uint_shift
                self._array_compare_and_set = lib.uint_compare_and_set
                self._array_compare_and_set_value = lib.uint_compare_and_set_value
                self._array_add_and_fetch = lib.uint_add_and_fetch
                self._array_sub_and_fetch = lib.uint_sub_and_fetch
                self._array_and_and_fetch = lib.uint_and_and_fetch
                self._array_or_and_fetch = lib.uint_or_and_fetch
                self._array_xor_and_fetch = lib.uint_xor_and_fetch
                self._array_nand_and_fetch = lib.uint_nand_and_fetch
                self._array_fetch_and_add = lib.uint_fetch_and_add
                self._array_fetch_and_sub = lib.uint_fetch_and_sub
                self._array_fetch_and_and = lib.uint_fetch_and_and
                self._array_fetch_and_or = lib.uint_fetch_and_or
                self._array_fetch_and_xor = lib.uint_fetch_and_xor
                self._array_fetch_and_nand = lib.uint_fetch_and_nand
            elif sys.platform == 'win32':
                self._type = "long"
                self.size = 4
                self._array_load = long_load
                self._array_get_and_set = long_get_and_set
                self._array_compare_and_set_value = long_compare_and_set_value
                self._array_add_and_fetch = long_add_and_fetch
                self._array_sub_and_fetch = long_sub_and_fetch
                self._array_fetch_and_add = long_fetch_and_add
                self._array_fetch_and_sub = long_fetch_and_sub
                self._array_fetch_and_and = long_fetch_and_and
                self._array_fetch_and_or = long_fetch_and_or
                self._array_fetch_and_xor = long_fetch_and_xor
                self._array_bittest_and_set = long_bittest_and_set
                self._array_bittest_and_reset = long_bittest_and_reset

        elif self.initial_length <= 8:
            if sys.platform in ('darwin', 'linux'):
                self._type = "unsigned long long"
                self.size = 8
                self._array_load = lib.ulonglong_load
                self._array_store = lib.ulonglong_store
                self._array_get_and_set = lib.ulonglong_get_and_set
                self._array_shift = lib.ulonglong_shift
                self._array_compare_and_set = lib.ulonglong_compare_and_set
                self._array_compare_and_set_value = lib.ulonglong_compare_and_set_value
                self._array_add_and_fetch = lib.ulonglong_add_and_fetch
                self._array_sub_and_fetch = lib.ulonglong_sub_and_fetch
                self._array_and_and_fetch = lib.ulonglong_and_and_fetch
                self._array_or_and_fetch = lib.ulonglong_or_and_fetch
                self._array_xor_and_fetch = lib.ulonglong_xor_and_fetch
                self._array_nand_and_fetch = lib.ulonglong_nand_and_fetch
                self._array_fetch_and_add = lib.ulonglong_fetch_and_add
                self._array_fetch_and_sub = lib.ulonglong_fetch_and_sub
                self._array_fetch_and_and = lib.ulonglong_fetch_and_and
                self._array_fetch_and_or = lib.ulonglong_fetch_and_or
                self._array_fetch_and_xor = lib.ulonglong_fetch_and_xor
                self._array_fetch_and_nand = lib.ulonglong_fetch_and_nand
            elif sys.platform == 'win32':
                self._type = "ssize_t"
                self.size = 8
                self._array_load = int_load
                self._array_get_and_set = int_get_and_set
                self._array_compare_and_set_value = int_compare_and_set_value
                self._array_add_and_fetch = int_add_and_fetch
                self._array_sub_and_fetch = int_sub_and_fetch
                self._array_fetch_and_add = int_fetch_and_add
                self._array_fetch_and_sub = int_fetch_and_sub
                self._array_fetch_and_and = int_fetch_and_and
                self._array_fetch_and_or = int_fetch_and_or
                self._array_fetch_and_xor = int_fetch_and_xor
                self._array_bittest_and_set = int_bittest_and_set
                self._array_bittest_and_reset = int_bittest_and_reset

        else:
            raise ValueError("Input set too long! It should be no more than 8 bytes!")

        if mode in ('s', 'singleprocessing'):
            self.mode = 's'
            self.windows_unix_compatibility = False
            self.reference = ffi.new(self._type + " *")
            self._array_get_and_set(self.reference, self.unsigned2signed(full_data) if sys.platform == 'win32' else full_data)

        elif mode in ('m', 'multiprocessing'):
            if sys.platform in ('darwin', 'linux'):
                self.mode = 'm'
                void_pointer = lib.mmap(ffi.NULL, self.size, 3, mmap_flags, -1, 0)
                self.reference = ffi.cast(self._type + " *", void_pointer)
                self._array_get_and_set(self.reference, full_data)
            else:
                multiprocessing_win32_init(self, windows_unix_compatibility, self.size)
                self.set_int(full_data)

        else:
            raise NotImplementedError("Unsupported mode!")



    if sys.platform in ("linux", "darwin"):
        def __del__(self):
            '''
                destructor
            '''
            if self.mode == 'm':
                lib.munmap(self.reference, self.size)

    else:
        def __del__(self):
            '''
                destructor
            '''
            multiprocessing_win32_del(self)

    def decode(self, bits_in_bytes: bytes) -> set:
        """Function to decode the bytes to set

        :param bits_in_bytes:  bytes needs to be decoded
        :param encoding: character encoding
        :return: the decoded set
        """
        bits = bitarray()
        bits.frombytes(bits_in_bytes)
        data_continue_flag: cython.bint = True
        i: cython.char = 0
        kind: cython.char
        length: cython.char
        start_point: cython.char

        meta_list = []
        while not bits[0]:
            bits <<= 1
        bits <<= 1
        while data_continue_flag:
            kind = ba2int(bits[i:i + 2])
            if kind != 0:
                length = ba2int(bits[i + 2:i + 5])
                meta_list.append({'length': length, 'kind': kind})
                if not bits[i + 5]:
                    data_continue_flag = False
                i += 6

            else:
                meta_list.append({'length': 1, 'kind': 0})
                if not bits[i + 2]:
                    data_continue_flag = False
                i += 3

        result = set()

        for data_dict in meta_list:
            start_point = i
            kind = data_dict['kind']
            length = data_dict['length']

            if kind == 0:
                result.add(True if bits[start_point] else False)
                i += 1
            elif kind == 1:
                result.add(ba2int(bits[start_point:start_point + length]))
                i += length
            elif kind == 2:
                str_bytes = bytes.fromhex(ba2hex(bits[start_point:start_point + length * 8]))
                result.add(str_bytes.decode(self.encoding))
                i += length * 8
            elif kind == 3:
                result.add(bytes.fromhex(ba2hex(bits[start_point:start_point + length * 8])))
                i += length * 8
            else:
                raise ('Type not supported!')
        return result

    def encode(self, input_set: set) -> (cython.size_t, cython.char):
        """function to encode the input_set with specific character encoding

        :param input_set: input set
        :param encoding: character encoding
        :return: (data in integer representation, total length in bits)
        """
        accumulated_length: cython.char = 0
        item_num: cython.char = 0
        data_bitarray = bitarray()
        data_prefix: cython.size_t = 1
        length: cython.char
        total_length: cython.char
        data_int: cython.size_t
        full_data: cython.size_t

        for i in input_set:
            hash(i)
            if item_num != 0:
                data_prefix += 1
            item_num += 1

            if isinstance(i, bool):
                length = 1
                data_prefix, accumulated_length = self.shift_and_add(data_prefix,
                                                                     accumulated_length,
                                                                     length, 0)
                data_bitarray += [i]
            elif isinstance(i, int):
                # length = uint(max(math.ceil(math.log2(i+1)),1))
                b = int2ba(i)
                length = len(b)
                data_prefix, accumulated_length = self.shift_and_add(data_prefix,
                                                                     accumulated_length,
                                                                     length, 1)

                # b.frombytes(i.to_bytes(length=max(math.ceil(length.value/8),1), byteorder='big'))
                # c = b[-length.value:]
                data_bitarray += b

            elif isinstance(i, str):
                input_bytes = i.encode(encoding=self.encoding)
                length = len(input_bytes)
                data_prefix, accumulated_length = self.shift_and_add(data_prefix,
                                                                     accumulated_length,
                                                                     length, 2)
                b = bitarray()
                b.frombytes(input_bytes)
                data_bitarray += b
            elif isinstance(i, bytes):
                length = len(i)
                data_prefix, accumulated_length = self.shift_and_add(data_prefix,
                                                                     accumulated_length,
                                                                     length, 3)
                b = bitarray()
                b.frombytes(i)
                data_bitarray += b
            else:
                raise ('Type not supported!')

        total_length = math.ceil((len(f'{data_prefix:b}') + accumulated_length) / 8)
        if total_length > 8:
            raise ValueError("Total Length exceeds 8 bytes!")
        # data_int = int.from_bytes(data_bitarray.tobytes(), byteorder='big')
        data_int = ba2int(data_bitarray)
        full_data = (data_prefix << len(data_bitarray)) + data_int
        return full_data, total_length

    def shift_and_add(self, data_prefix: cython.size_t, accumulate_length: cython.char,
                      input_length: cython.char, kind: cython.char) -> (cython.size_t, cython.char):
        """Function to finish the shift and add of the data prefix

        :param data_prefix: integer for the data_prefix
        :param accumulate_length: total data length in bits accumulated so far
        :param input_length: length of the data segment in bits
        :param kind: sorts of data
        :return: (data_prefix, accumulate_length)
        """

        if kind in (2, 3):
            accumulate_length += input_length * 8
        else:
            accumulate_length += input_length

        if kind != 0:
            data_prefix <<= 6
            data_prefix += (kind << 4) + (input_length << 1)
        else:
            data_prefix <<= 3
            # self.atomic.uint_add_and_fetch(byref(data_prefix),0)

        return data_prefix, accumulate_length

    if sys.platform in ("linux", "darwin"):
        def get_int(self) -> cython.size_t:
            """
            Get the whole integer representation from the set,
            the whole set would be treated as a large integer

            :return: the integer representation
            """
            return self._array_load(self.reference)

        def set_int(self, integer: cython.size_t):
            """
            Set the whole integer representation from the set,
            the whole set would be treated as a large integer

            :return: None
            """
            newpointer = ffi.new(self._type + " *", integer)
            self._array_store(self.reference, newpointer)

        def get_set(self) -> set:
            r"""
            Get the set atomically

            :return: the set
            """
            result = int.to_bytes(self._array_load(self.reference),
                                  length=self.size, byteorder='big')
            return self.decode(result)

        def set_set(self, data: set):
            """
            Set the value in the set,
            if the new data is longer than the original size of the set.
            it will expand the set accordingly which would lose atomicy.
            the size of the set can be check with self.size

            :param data: input set
            :return: None
            """
            integer: cython.size_t
            desiredlength: cython.char

            integer, desiredlength = self.encode(data)

            if 8 >= desiredlength > self.size:
                self.__init__({b'\0\0\0\0\0\0'}, mode=self.mode, encoding=self.encoding)
            if sys.platform in ('darwin', 'linux'):
                newpointer = ffi.new(self._type + " *", integer)
            elif sys.platform == 'win32':
                newpointer = self._type(integer)
            self._array_store(self.reference, newpointer)

        def set_store(self, i):
            """
            Atomically store contents from another set to the this set,
            if the other set is different with this one in size , the function will fail.

            :param i: another set to store its value to self
            :return: None
            """
            if self.size != i.size:
                raise ValueError("Input set has different size!")
            self._array_store(self.reference, i.reference)

        def set_get_and_set(self, data: set) -> set:
            r"""
            Get and set atomically

            :param data: new data set
            :return: the original set
            """
            integer: cython.size_t
            desiredlength: cython.char

            integer, length = self.encode(data)
            # if length > self.size:
            #    raise ValueError("Input data too long cannot be set atomically!")
            result = int.to_bytes(self._array_get_and_set(self.reference, integer),
                                  length=self.size, byteorder='big')
            return self.decode(result)

        def set_shift(self, i, j):
            """
            Value exchange between 3 pointers in 2 groups atomically,
            store i in itself after store itself in j

            :param i: one atomic_set
            :param j: another atomic_set
            :return: None
            """
            if self.size != i.size:
                raise ValueError("Comparing set i has different size!")
            if self.size != j.size:
                raise ValueError("Comparing set j has different size!")
            self._array_shift(self.reference,
                              i.reference, j.reference)

        def set_compare_and_set(self, i, n: set) -> cython.bint:
            """
            Compare and set atomically, This compares the contents of self
            with the contents of i. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            the operation is a read and the current contents of itself are written into
            i.

            :param i: the set to be compared with
            :param n: another bytes to be ready to set to self if comparision return True
            :return: if self is equal to i return True, else return False
            """
            integer: cython.size_t
            length: cython.char

            if self.size != i.size:
                raise ValueError("Comparing string has different size!")
            integer, length = self.encode(n)

            # if length > self.size:
            #    raise ValueError("Input data too large, cannot set atomically!")
            result = self._array_compare_and_set(self.reference,
                                                 i.reference, integer)
            return result

        def set_compare_and_set_value(self, i:set, n: set) -> set:
            """
            Compare and swap atomically, This compares the contents of self
            with the contents of n. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            No operation is performed.

            :param i: exchange value
            :param n: The value to be compared with
            :return: the initial value of self
            """
            integer_n: cython.size_t
            integer_i: cython.size_t
            length_n: cython.char
            length_i: cython.char

            integer_n, length_n = self.encode(n)
            integer_i, length_i = self.encode(i)


            #try:
            #    expected = ffi.new(self._type + " *", integer_n)
            #    temp = ffi.new(self._type + " *", integer_i)
            #if length_n > self.size * 8 or length_i > self.size * 8:
            #   raise ValueError("Numeric Overflow! Please check if i and n is too long for "+f'{self.size}' + " bytes !")


            # if length > self.size:
            #    raise ValueError("Input data too large, cannot set atomically!")

            return self.decode(int.to_bytes(self._array_compare_and_set_value(self.reference,
                                        integer_i, integer_n),
                                  length=self.size, byteorder='big'))

        def reencode(self, newencode: str):
            """
            Change the encoding of the string, if the original size is not enough,
            it will elongate the string, if 8 bytes are not enough, it will fail.

            :param newencode: new encoding, such as 'utf-8', 'utf-16-le'
            :return: None
            """
            data = self.get_set()
            prev_encoding = self.encoding
            self.encoding = newencode
            try:
                self.set_set(data)
            except ValueError:
                self.encoding = prev_encoding
                self.set_set(data)
                raise ValueError("New encoding took larger space than 8 bytes, cannot elongate!")

        def change_mode(self, newmode='m'):
            """
            Switch between singleprocessing mode and multiprocessing mode,
            the function doesn't exists on windows, since only single processing mode
            is supported on windows platform.
            the contents will be copied , other threads/processes would not be aware of the change.

            :param newmode: the mode to change to, 'm' or 'multiprocessing' for multiproessing, 's' or 'singleprocessing' for singleprocessing. default 'm'
            :return: None
            """
            if newmode in ('m', 'multiprocessing'):
                if self.mode == 's':
                    data: cython.size_t = self.get_int()
                    void_pointer = lib.mmap(ffi.NULL, self.size, 3, mmap_flags, -1, 0)
                    self.reference = ffi.cast(self._type + " *", void_pointer)

                    self.set_int(data)

            elif newmode not in ('s', 'singleprocessing'):
                raise ValueError(
                    "newmode has the wrong value, should be 'm','s','multiprocessing' or 'singleprocessing'")

            elif self.mode == 'm':
                data: cython.size_t = self.get_int()
                self.reference = ffi.new(self._type + " *")
                self.set_int(data)
            self.mode = newmode
    else:
        def get_int(self) -> cython.size_t:
            """
            Get the whole integer representation from the set,
            the whole set would be treated as a large integer

            :return: the integer representation
            """
            return set_get_int(self, self.reference)

        def set_int(self, integer: cython.size_t):
            """
            Set the whole integer representation from the set,
            the whole set would be treated as a large integer

            :return: None
            """
            set_set_int(self, self.reference, integer)

        def get_set(self) -> set:
            r"""
            Get the set atomically

            :return: the set
            """
            return set_get_set(self, self.reference)

        def set_set(self, data: set):
            """
            Set the value in the set,
            if the new data is longer than the original size of the set.
            it will expand the set accordingly which would lose atomicy.
            the size of the set can be check with self.size

            :param data: input set
            :return: None
            """
            set_set_set(self, self.reference, data)

        def set_get_and_set(self, data: set) -> set:
            r"""
            Get and set atomically

            :param data: new data set
            :return: the original set
            """
            return set_get_and_set(self, self.reference, data)

        def set_compare_and_set_value(self, i: set, n: set) -> cython.set:
            """
            Compare and set atomically, This compares the contents of self
            with the contents of i. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            the operation is a read and the current contents of itself are written into
            i.

            :param i: the set to be compared with
            :param n: another bytes to be ready to set to self if comparision return True
            :return: if self is equal to i return True, else return False
            """
            return set_compare_and_set_value(self, self.reference, i, n)

        def reencode(self, newencode: str):
            """
            Change the encoding of the string, if the original size is not enough,
            it will elongate the string, if 8 bytes are not enough, it will fail.

            :param newencode: new encoding, such as 'utf-8', 'utf-16-le'
            :return: None
            """
            data = self.get_set()
            prev_encoding = self.encoding
            self.encoding = newencode
            try:
                self.set_set(data)
            except ValueError:
                self.encoding = prev_encoding
                self.set_set(data)
                raise ValueError("New encoding took larger space than 8 bytes, cannot elongate!")

        def change_mode(self, newmode='m', windows_unix_compatibility: bool=False):
            """
            Switch between singleprocessing mode and multiprocessing mode,
            the function doesn't exists on windows, since only single processing mode
            is supported on windows platform.
            the contents will be copied , other threads/processes would not be aware of the change.

            :param newmode: the mode to change to, 'm' or 'multiprocessing' for multiproessing, 's' or 'singleprocessing' for singleprocessing. default 'm'
            :return: None
            """
            if newmode in ('m', 'multiprocessing'):
                if self.mode == 's':
                    data: cython.size_t = self.get_int()
                    multiprocessing_win32_init(self, windows_unix_compatibility, self.size)
                    self.set_int(data)

            elif newmode not in ('s', 'singleprocessing'):
                raise ValueError(
                    "newmode has the wrong value, should be 'm','s','multiprocessing' or 'singleprocessing'")

            elif self.mode == 'm':
                data: cython.size_t = self.get_int()
                self.windows_unix_compatibility = False
                self.reference = ffi.new(self._type + " *")
                self.set_int(data)

            self.mode = newmode

    value = property(fget=get_set, fset=set_set, doc="same with get_set and set_set")
    int_value = property(fget=get_int, doc="same with get_int")
