import cython
import ctypes
from typing import TypeVar
import sys
from shared_atomic.atomic_decryption_ import ffi
from shared_atomic.atomic_decryption_ import lib as lib_decryption


def sha1(message: bytes) -> str:
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")
    if lib_decryption.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib_decryption.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join(
            [i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")

if sha1(b'===randomstring===') != '===randomstringsha1===':
    raise ValueError("Didn't pass checksum!")

if sys.platform in ("linux", "darwin", "win32"):
    import shared_atomic.shared_atomic_
    from shared_atomic.shared_atomic_ import lib
    from shared_atomic.shared_atomic_ import ffi

    with open(shared_atomic.shared_atomic_.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomiclibsha1===':
            raise ValueError("Didn't pass checksum!")

    import shared_atomic.atomic_object
    from shared_atomic.atomic_object import *
    with open(shared_atomic.atomic_object.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomic_object_libsha1===':
            raise ValueError("Didn't pass checksum3!")

    from shared_atomic.atomic_object import *

    from shared_atomic import mmap_flags

else:
    raise NotImplementedError("Unsupported platform!")

if sys.platform == "win32":
    import platform

    if platform.win32_ver()[0] + '.' + platform.win32_ver()[2] >= '10.SP0':
        from shared_atomic.shared_atomic_win10_ import lib as lib10
    with open(shared_atomic.shared_atomic_win10_.__file__, 'rb') as f:
        shared_atomic_win10_bytes = f.read()
        if sha1(shared_atomic_win10_bytes) != '===atomicwin10libsha1===':
            raise ValueError("Didn't pass checksum!")

self_atomic_string = TypeVar("self_atomic_string", bound="atomic_string")



class atomic_string(atomic_object):
    """
    string provide atomic operations, the string should be no longer than 8 bytes
    :attributes
        encoding: readonly string character set\n
        size: readonly size of the string\n
        initial_byte_length: readonly initial input length as number of bytes it took\n
        value: read/write value of the string\n
        mode: readonly 's' for single process, 'm' for multiprocessing, on windows platform, only singleprocessing is supported\n
        windows_unix_compatibility: readonly whether the source code should be compatible with windows x64
    """

    encoding = cython.declare(str, visibility='readonly')
    size = cython.declare(cython.char, visibility='readonly')
    initial_length = cython.declare(cython.char, visibility='readonly')
    mode = cython.declare(str,visibility='readonly')
    windows_unix_compatibility = cython.declare(cython.bint,visibility='public')

    def __init__(self, initial: str,
                 mode: str = 'singleprocessing',
                 length: int = None,
                 paddingdirection: str = 'right',
                 paddingstr: str = ' ',
                 trimming_direction: str = 'right',
                 encoding='utf-8',
                 windows_unix_compatibility: bool = False):
        r"""
        constructor to initialize the string,
        the string should be no longer than 8 bytes

        :param initial: initial value of the string, if the initial value is longer than 8 bytes, please specify the trimming target length, or else it would fail.
        :param mode: the mode in which the string will be shared. 'singleprocessing' or 's' for single process, 'multiprocessing' or 'm' for multiprocessing, on windows platform, only singleprocessing is supported, setting it to 'm' or 'multiprocessing' will be ignored.
        :param length: the expected length after padding/trimming for the input value, if not specified, no padding or trimming performed, use original value.
        :param paddingdirection: right, or left side the padding bytes would be added if not specified, pad to the right side, use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
        :param paddingstr: string to pad to the original bytes, by default '\\0' can be multiple bytes like b'ab', will be padded to the original bytes in circulation until the expected length is reached.
        :param trimming_direction: if initial bytes are longer, on which side the bytes will be trimmed. By default, on the right side, use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
        :param encoding: charactor encoding to store the string, use 'utf-8' by default.
        :param windows_unix_compatibility: whether the source code should be compatible with windows x64
        """
        super().__init__()
        input_length: cython.char
        self.initial_byte_length: cython.char
        self.size: cython.char

        self.encoding = encoding
        input_length = len(initial.encode(encoding=self.encoding))

        if length is not None:
            if length > 7:
                raise ValueError("length is longer than expected, "
                                 "should be less than 9 bytes!")

        elif input_length > 7 and length is None:
            raise ValueError("input string is longer than expected, "
                             "should be less than 9 bytes!")

        if length is not None:
            if input_length > length:
                if trimming_direction in ('r', 'right'):
                    data = initial.encode(self.encoding)[:length].decode(self.encoding, 'ignore')
                elif trimming_direction in ('l', 'left'):
                    data = initial[::-1].encode(self.encoding)[:length].decode(self.encoding, 'ignore')[::-1]

            elif input_length < length:
                paddingbytes = paddingstr.encode(self.encoding)
                devision, remaining = divmod(length - input_length, len(paddingbytes))
                padding = paddingstr * devision + paddingstr[:remaining]
                if paddingdirection in ('l', 'left'):
                    data = (padding + initial)[::-1].encode(self.encoding)[:length].decode(self.encoding, 'ignore')[
                           ::-1]
                elif paddingdirection in ('r', 'right'):
                    data = (initial + padding).encode(self.encoding)[:length].decode(self.encoding, 'ignore')
            else:
                data = initial
        else:
            data = initial

        data_byte = data.encode(self.encoding)
        self.initial_byte_length = len(data_byte)

        if self.initial_byte_length == 1:
            self.size = 2
            if sys.platform in ('darwin', 'linux'):
                self._type = "unsigned short"
                self._array_load = lib.ushort_load
                self._array_store = lib.ushort_store
                self._array_get_and_set = lib.ushort_get_and_set
                self._array_shift = lib.ushort_shift
                self._array_compare_and_set = lib.ushort_compare_and_set
                self._array_compare_and_set_value = lib.ushort_compare_and_set_value
                self._array_add_and_fetch = lib.ushort_add_and_fetch
                self._array_sub_and_fetch = lib.ushort_sub_and_fetch
                self._array_and_and_fetch = lib.ushort_and_and_fetch
                self._array_or_and_fetch = lib.ushort_or_and_fetch
                self._array_xor_and_fetch = lib.ushort_xor_and_fetch
                self._array_nand_and_fetch = lib.ushort_nand_and_fetch
                self._array_fetch_and_add = lib.ushort_fetch_and_add
                self._array_fetch_and_sub = lib.ushort_fetch_and_sub
                self._array_fetch_and_and = lib.ushort_fetch_and_and
                self._array_fetch_and_or = lib.ushort_fetch_and_or
                self._array_fetch_and_xor = lib.ushort_fetch_and_xor
                self._array_fetch_and_nand = lib.ushort_fetch_and_nand
            elif sys.platform == 'win32':
                self._type = "short"
                self._array_load = short_load
                self._array_get_and_set = short_get_and_set
                self._array_compare_and_set_value = short_compare_and_set_value
                self._array_add_and_fetch = short_add_and_fetch
                self._array_sub_and_fetch = short_sub_and_fetch
                self._array_fetch_and_add = short_fetch_and_add
                self._array_fetch_and_sub = short_fetch_and_sub
                self._array_fetch_and_and = short_fetch_and_and
                self._array_fetch_and_or = short_fetch_and_or
                self._array_fetch_and_xor = short_fetch_and_xor

        elif self.initial_byte_length <= 3:
            self.size = 4
            if sys.platform in ('darwin', 'linux'):
                self._type = "unsigned int"
                self._array_load = lib.uint_load
                self._array_store = lib.uint_store
                self._array_get_and_set = lib.uint_get_and_set
                self._array_shift = lib.uint_shift
                self._array_compare_and_set = lib.uint_compare_and_set
                self._array_compare_and_set_value = lib.uint_compare_and_set_value
                self._array_add_and_fetch = lib.uint_add_and_fetch
                self._array_sub_and_fetch = lib.uint_sub_and_fetch
                self._array_and_and_fetch = lib.uint_and_and_fetch
                self._array_or_and_fetch = lib.uint_or_and_fetch
                self._array_xor_and_fetch = lib.uint_xor_and_fetch
                self._array_nand_and_fetch = lib.uint_nand_and_fetch
                self._array_fetch_and_add = lib.uint_fetch_and_add
                self._array_fetch_and_sub = lib.uint_fetch_and_sub
                self._array_fetch_and_and = lib.uint_fetch_and_and
                self._array_fetch_and_or = lib.uint_fetch_and_or
                self._array_fetch_and_xor = lib.uint_fetch_and_xor
                self._array_fetch_and_nand = lib.uint_fetch_and_nand
            elif sys.platform == 'win32':
                self._type = "long"
                self._array_load = long_load
                self._array_get_and_set = long_get_and_set
                self._array_compare_and_set_value = long_compare_and_set_value
                self._array_add_and_fetch = long_add_and_fetch
                self._array_sub_and_fetch = long_sub_and_fetch
                self._array_fetch_and_add = long_fetch_and_add
                self._array_fetch_and_sub = long_fetch_and_sub
                self._array_fetch_and_and = long_fetch_and_and
                self._array_fetch_and_or = long_fetch_and_or
                self._array_fetch_and_xor = long_fetch_and_xor
                self._array_bittest_and_set = long_bittest_and_set
                self._array_bittest_and_reset = long_bittest_and_reset

        elif self.initial_byte_length <= 7:
            self.size = 8
            if sys.platform in ('darwin', 'linux'):
                self._type = "unsigned long long"
                self._array_load = lib.ulonglong_load
                self._array_store = lib.ulonglong_store
                self._array_get_and_set = lib.ulonglong_get_and_set
                self._array_shift = lib.ulonglong_shift
                self._array_compare_and_set = lib.ulonglong_compare_and_set
                self._array_compare_and_set_value = lib.ulonglong_compare_and_set_value
                self._array_add_and_fetch = lib.ulonglong_add_and_fetch
                self._array_sub_and_fetch = lib.ulonglong_sub_and_fetch
                self._array_and_and_fetch = lib.ulonglong_and_and_fetch
                self._array_or_and_fetch = lib.ulonglong_or_and_fetch
                self._array_xor_and_fetch = lib.ulonglong_xor_and_fetch
                self._array_nand_and_fetch = lib.ulonglong_nand_and_fetch
                self._array_fetch_and_add = lib.ulonglong_fetch_and_add
                self._array_fetch_and_sub = lib.ulonglong_fetch_and_sub
                self._array_fetch_and_and = lib.ulonglong_fetch_and_and
                self._array_fetch_and_or = lib.ulonglong_fetch_and_or
                self._array_fetch_and_xor = lib.ulonglong_fetch_and_xor
                self._array_fetch_and_nand = lib.ulonglong_fetch_and_nand
            elif sys.platform == 'win32':
                self._type = "ssize_t"
                self._array_load = int_load
                self._array_get_and_set = int_get_and_set
                self._array_compare_and_set_value = int_compare_and_set_value
                self._array_add_and_fetch = int_add_and_fetch
                self._array_sub_and_fetch = int_sub_and_fetch
                self._array_fetch_and_add = int_fetch_and_add
                self._array_fetch_and_sub = int_fetch_and_sub
                self._array_fetch_and_and = int_fetch_and_and
                self._array_fetch_and_or = int_fetch_and_or
                self._array_fetch_and_xor = int_fetch_and_xor
                self._array_bittest_and_set = int_bittest_and_set
                self._array_bittest_and_reset = int_bittest_and_reset
        else:
            raise ValueError("Input String is longer than 8 bytes!")

        if mode in ('s', 'singleprocessing'):
            self.mode = 's'
            self.windows_unix_compatibility = False
            self.reference = ffi.new(self._type + " *")
            self._set_bytes(data_byte)
        elif mode in ('m', 'multiprocessing'):
            self.mode = 'm'
            if sys.platform in ('darwin', 'linux'):
                void_pointer = lib.mmap(ffi.NULL, self.size, 3, mmap_flags, -1, 0)
                self.reference = ffi.cast(self._type + " *", void_pointer)
                self._set_bytes(data_byte)
            else:
                multiprocessing_win32_init(self, windows_unix_compatibility, self.size)
                self._set_bytes(data_byte)

        else:
            raise NotImplementedError("Unsupported mode!")

    if sys.platform in ("darwin", "linux"):
        def __del__(self):
            if self.mode == 'm':
                lib.munmap(self.reference, self.size)
    else:
        def __del__(self):
            multiprocessing_win32_del(self)

    def resize(self, newlength: cython.char,
               paddingdirection: str = 'right',
               paddingstr: str = ' ',
               trimming_direction: str = 'right'):
        r"""
        trim or pad the original contents in the string
        to a new length, the new length should be no longer than 8 bytes,
        the original string wll be replaced with new string, if the original
        string is shared between threads/processes, other threads/processes
        will wouldn't be aware of the change, still use the old string.

        :param newlength: the expected new length of the original bytes.
        :param paddingdirection: if longer than original, left or right sidethe original bytes should be padded, by default right side,use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
        :param paddingstr: bytes to pad to the original bytes, by default '\\0' can be multiple bytes like b'ab', will be padded to the original bytes in circulation until the expected length is reached.
        :param trimming_direction: if shorted than original, left or right side the original bytes should be padded,use 'right' or 'r' to specify right side,use 'left' or 'l' to specify the left side.
        :return: None
        """

        self.__init__(self.get_string(), mode=self.mode, length=newlength,
                      paddingdirection=paddingdirection,
                      paddingstr=paddingstr,
                      trimming_direction=trimming_direction)

    if sys.platform in ('darwin', 'linux'):
        def _get_int(self) -> cython.size_t:
            """
            Get the integer representation from the string atomically,
            the whole string would be treated as a large integer

            :return: the integer representation
            """
            return self._array_load(self.reference)

        def _set_int(self, input: cython.size_t):
            """
            Set the integer representation of the string atomically,
            the whole array would be treated as a large integer

            :return: None
            """
            newpointer = ffi.new(self._type + " *", input)
            self._array_store(self.reference, newpointer)

        def get_string(self):
            r"""
            Get all the bytes from the string atomically

            :return: all the bytes in the string
            """

            result_bytes = int.to_bytes(self._array_load(self.reference), length=self.size, byteorder='big')
            length = int.from_bytes(result_bytes[0:1], byteorder='big')

            return (result_bytes[1:length + 1]).decode(self.encoding)

        def set_string(self, data: str):
            """
            Set the value in the string,
            if the new data is longer than the original size of the string.
            it will expand the string accordingly which would lose atomicy.
            the size of the string can be check with self.size

            :param data: input string
            :return: None
            """
            data_bytes = data.encode(self.encoding)
            self._set_bytes(data_bytes)

        def _set_bytes(self, data: bytes):
            """
            Set the bytes value in the string,
            if the new data is longer than the original size of the string.
            it will expand the string accordingly which would lose atomicy.
            the size of the string can be check with self.size

            :param data: input string
            :return: None
            """

            desiredlength: cython.char
            integer: cython.size_t

            desiredlength = len(data)

            if 7 >= desiredlength > self.size - 1:
                self.resize(desiredlength)
            elif desiredlength > 7:
                raise ValueError()

            integer = lib.string_byte2int(self.size, data, desiredlength)
            self._set_int(integer)

            # ctype_integer = self.type(integer)
            # self._array_store(self.array_reference, ctypes.byref(ctype_integer))

        def resize(self, newlength: cython.char,
                   paddingdirection: str = 'right',
                   paddingstr: str = ' ',
                   trimming_direction: str = 'right'):
            r"""
            trim or pad the original contents in the string
            to a new length, the new length should be no longer than 8 bytes,
            the original string wll be replaced with new string, if the original
            string is shared between threads/processes, other threads/processes
            will wouldn't be aware of the change, still use the old string.

            :param newlength: the expected new length of the original bytes.
            :param paddingdirection: if longer than original, left or right sidethe original bytes should be padded, by default right side,use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
            :param paddingstr: bytes to pad to the original bytes, by default '\\0' can be multiple bytes like b'ab', will be padded to the original bytes in circulation until the expected length is reached.
            :param trimming_direction: if shorted than original, left or right side the original bytes should be padded,use 'right' or 'r' to specify right side,use 'left' or 'l' to specify the left side.
            :return: None
            """

            self.__init__(self.get_string(), mode=self.mode, length=newlength,
                          paddingdirection=paddingdirection,
                          paddingstr=paddingstr,
                          trimming_direction=trimming_direction)

        def string_store(self, i: self_atomic_string):
            """
            Atomically store contents from another string to the this string,
            if the other string is different with this one in size , the function will fail.

            :param i: another string to store its value to self
            :return:
            """
            if self.size != i.size:
                raise ValueError("Input string has different size!")
            self._array_store(self.reference, i.reference)

        def string_get_and_set(self, data: str) -> str:
            r"""
            Get and set atomically

            :param data: new data
            :return: the original string
            """
            #integer: cython.size_t
            #result_length: cython.char

            #integer = self._str2int(data)
            size: cython.char = self.size
            encoding: str = self.encoding
            data_bytes: bytes = data.encode(encoding)

            integer: cython.size_t = lib.string_byte2int(size, data_bytes, len(data_bytes))
            result = int.to_bytes(self._array_get_and_set(self.reference, integer),
                                  length=size, byteorder='big')
            result_length: cython.char = int.from_bytes(result[:1], byteorder='big')
            result_all = result[1:result_length + 1]
            return result_all.decode(encoding)

        def string_shift(self, i: self_atomic_string, j: self_atomic_string):
            """
            Value exchange between 3 pointers in 2 groups atomically,
            the initial_length field will be updated but not atomically.
            store i in itself after store itself in j

            :param i: one atomic_string
            :param j: another atomic_string
            :return: None
            """
            if self.size != i.size:
                raise ValueError("Comparing string i has different size!")
            if self.size != j.size:
                raise ValueError("Comparing string j has different size!")
            self._array_shift(self.reference,
                              i.reference, j.reference)

        def string_compare_and_set(self, i: self_atomic_string, n: str) -> bool:
            """
            Compare and set atomically, this compares the contents of self
            with the contents of i. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            the operation is a read and the current contents of itself are written into
            i.

            :param i: the string to be compared with
            :param n: another bytes to be ready to self if comparision return True
            :return: if self is equal to i return True, else return False
            """
            integer: cython.size_t
            encoding:str = self.encoding
            data: bytes = n.encode(encoding)
            if self.size != i.size:
                raise ValueError("Comparing string has different size!")
            integer = lib.string_byte2int(self.size, data, len(data))
            return self._array_compare_and_set(self.reference,
                                               i.reference, integer)

        if sys.platform == 'linux':
            def string_compare_and_set_value(self, i: str, n: str) -> str:
                """
                Compare and swap atomically, This compares the contents of self
                with the contents of n. If equal, the operation is a read-modify-write
                operation that writes i into self. If they are not equal,
                No operation is performed.

                :param i: exchange value
                :param n: The value to be compared with
                :return: the initial value of self
                """
                size: cython.char = self.size
                encoding: str = self.encoding
                n_data: bytes = n.encode(encoding)
                i_data: bytes = i.encode(encoding)

                result_bytes = ffi.new("unsigned char []", size)

                return_code: cython.int = lib.string_compare_and_set_value(self.reference, size,
                                                 i_data, len(i_data), n_data, len(n_data),
                                                 result_bytes)
                if return_code == 1:
                    return ffi.string(result_bytes, 8).decode(encoding)
                else:
                    raise ValueError("compare_and_set_value atomic_operation failed with return code " + f'{return_code}')
        else:
            def string_compare_and_set_value(self, i: str, n: str) -> str:
                """
                Compare and swap atomically, This compares the contents of self
                with the contents of n. If equal, the operation is a read-modify-write
                operation that writes i into self. If they are not equal,
                No operation is performed.

                :param i: exchange value
                :param n: The value to be compared with
                :return: the initial value of self
                """
                size: cython.char = self.size
                encoding: str = self.encoding
                n_data: bytes = n.encode(encoding)
                i_data: bytes = i.encode(encoding)

                integer_n: cython.size_t = lib.string_byte2int(size, n_data, len(n_data))
                integer_i: cython.size_t = lib.string_byte2int(size, i_data, len(i_data))
                result_bytes = ffi.new("unsigned char []", size)
                lib.string_int2byte(self._array_compare_and_set_value(self.reference, integer_i, integer_n),
                                    size,
                                    result_bytes)
                return ffi.string(result_bytes).decode(encoding)

        def reencode(self, newencode: str):
            """
            Change the encoding of the string, if the original size is not enough,
            it will elongate the string, if 7 bytes are not enough, it will fail.

            :param newencode: new encoding, such as 'utf-8', 'utf-16-le'
            :return: None
            """
            data = self.get_string()
            new_data = data.encode(newencode)
            self._set_bytes(new_data)
            self.encoding = newencode

        def change_mode(self, newmode='m'):
            """
            Switch between singleprocessing mode and multiprocessing mode,
            the function doesn't exists on windows, since only single processing mode
            is supported on windows platform.
            the contents will be copied , other threads/processes would not be aware of the change.

            :param newmode: the mode to change to, 'm' or 'multiprocessing' for multiproessing, 's' or 'singleprocessing' for singleprocessing. default 'm'
            :return: None
            """
            if newmode in ('m', 'multiprocessing'):
                if self.mode == 's':
                    data = self._get_int()
                    void_pointer = lib.mmap(ffi.NULL, self.size, 3, mmap_flags, -1, 0)
                    self.reference = ffi.cast(self._type + " *", void_pointer)
                    self._set_int(data)

            elif newmode not in ('s', 'singleprocessing'):
                raise ValueError(
                    "newmode has the wrong value, should be 'm','s','multiprocessing' or 'singleprocessing'")

            elif self.mode == 'm':
                data = self._get_int()
                self.reference = ffi.new(self._type + " *")
                self._set_int(data)
            self.mode = newmode

    else:
        def _str2int(self, input: str) -> cython.size_t:
            r"""
            integer of input string padded with leading length and tailing '\\0'

            :param input: input string
            :return: the integer representation and the length
            """
            input_bytes = input.encode(self.encoding)
            integer = self._byte2int(input_bytes)
            return integer

        def _byte2int(self, input_bytes: bytes) -> cython.size_t:
            """
            integer of input bytes padded with leading length and tailing b'\\0'

            :param input_bytes: input bytes
            :return: the integer representation and the length
            """

            integer: cython.size_t
            result_bytes = len(input_bytes).to_bytes(length=1, byteorder='big') + self._rpad_zero(input_bytes)
            integer = int.from_bytes(result_bytes, byteorder='big')
            return integer

        def _rpad_zero(self, input: bytes) -> bytes:
            r"""
            right pad zero to the input string
            :param input: input bytes

            :return: right padded bytes b'\\0' to self.size -1
            """

            input_length: cython.char
            input_length = len(input)
            if input_length < self.size - 1:
                return input + (self.size - 1 - input_length) * b'\0'
            elif input_length > self.size - 1:
                raise ValueError('input length longer than its size!')
            else:
                return input

        def _get_int(self) -> cython.size_t:
            """
            Get the integer representation from the string atomically,
            the whole string would be treated as a large integer

            :return: the integer representation
            """
            return string_get_int(self, self.reference)

        def _set_int(self, input: cython.size_t):
            """
            Set the integer representation of the string atomically,
            the whole array would be treated as a large integer

            :return: None
            """
            string_set_int(self, self.reference, input)

        def get_string(self) -> str:
            r"""
            Get all the bytes from the string atomically

            :return: full string
            """
            return string_get_string(self, self.reference)

        def set_string(self, data: str):
            """
            Set the value in the string,
            if the new data is longer than the original size of the string.
            it will expand the string accordingly which would lose atomicy.
            the size of the string can be check with self.size

            :param data: input string
            :return: None
            """
            string_set_string(self, self.reference, data)

        def _set_bytes(self, data: bytes):
            """
            Set the bytes value in the string,
            if the new data is longer than the original size of the string.
            it will expand the string accordingly which would lose atomicy.
            the size of the string can be check with self.size

            :param data: input string
            :return: None
            """

            string_set_bytes(self, self.reference, data)

        def resize(self, newlength: cython.char,
                   paddingdirection: str = 'right',
                   paddingstr: str = ' ',
                   trimming_direction: str = 'right'):
            r"""
            trim or pad the original contents in the string
            to a new length, the new length should be no longer than 8 bytes,
            the original string wll be replaced with new string, if the original
            string is shared between threads/processes, other threads/processes
            will wouldn't be aware of the change, still use the old string.

            :param newlength: the expected new length of the original bytes.
            :param paddingdirection: if longer than original, left or right sidethe original bytes should be padded, by default right side,use 'right' or 'r' to specify right side, use 'left' or 'l' to specify the left side.
            :param paddingstr: bytes to pad to the original bytes, by default '\\0' can be multiple bytes like b'ab', will be padded to the original bytes in circulation until the expected length is reached.
            :param trimming_direction: if shorted than original, left or right side the original bytes should be padded,use 'right' or 'r' to specify right side,use 'left' or 'l' to specify the left side.
            :return: None
            """

            self.__init__(self.get_string(), mode=self.mode, length=newlength,
                          paddingdirection=paddingdirection,
                          paddingstr=paddingstr,
                          trimming_direction=trimming_direction,
                          windows_unix_compatibility=self.windows_unix_compatibility)

        def string_get_and_set(self, data: str) -> str:
            r"""
            Get and set atomically

            :param data: new data
            :return: the original string
            """
            return string_get_and_set(self, self.reference, data)

        def string_compare_and_set_value(self, i: str, n: str) -> str:
            """
            Compare and swap atomically, This compares the contents of self
            with the contents of n. If equal, the operation is a read-modify-write
            operation that writes n into self. If they are not equal,
            No operation is performed.

            :param i: exchange value
            :param n: The value to be compared with
            :return: the initial value of self
            """
            return string_compare_and_set_value(self, self.reference, i, n)

        def reencode(self, newencode: str):
            """
            Change the encoding of the string, if the original size is not enough,
            it will elongate the string, if 7 bytes are not enough, it will fail.

            :param newencode: new encoding, such as 'utf-8', 'utf-16-le'
            :return: None
            """
            data = self.get_string()
            new_data = data.encode(newencode)
            self._set_bytes(new_data)
            self.encoding = newencode

        def change_mode(self, newmode='m', windows_unix_compatibility: bool=False):
            """
            Switch between singleprocessing mode and multiprocessing mode,
            the function doesn't exists on windows, since only single processing mode
            is supported on windows platform.
            the contents will be copied , other threads/processes would not be aware of the change.

            :param newmode: the mode to change to, 'm' or 'multiprocessing' for multiproessing, 's' or 'singleprocessing' for singleprocessing. default 'm'
            :return: None
            """
            if newmode in ('m', 'multiprocessing'):
                if self.mode == 's':
                    data = self._get_int()
                    multiprocessing_win32_init(self, windows_unix_compatibility, self.size)
                    self._set_int(data)

            elif newmode not in ('s', 'singleprocessing'):
                raise ValueError(
                    "newmode has the wrong value, should be 'm','s','multiprocessing' or 'singleprocessing'")

            elif self.mode == 'm':
                data = self._get_int()
                self.reference = ffi.new(self._type + " *")
                self._set_int(data)
            self.mode = newmode

    value = property(fget=get_string, fset=set_string, doc="same with get_string and set_string")
