import cython
import sys
import os
from typing import TypeVar

from shared_atomic.atomic_decryption_ import ffi
from shared_atomic.atomic_decryption_ import lib as lib_decryption


def sha1(message: bytes) -> str:
    message_len: cython.Py_ssize_t = len(message)
    message_char = ffi.new("unsigned char []", message)
    digest_length = ffi.new("unsigned int *")
    digest_value = ffi.new("unsigned char [20]")
    if lib_decryption.sha1(message_char, message_len, digest_value, digest_length) == 1:
        if digest_length[0] > 20:
            digest_value = ffi.new("unsigned char []", digest_length[0])
            lib_decryption.sha1(message_char, message_len, digest_value, digest_length)
        return b''.join(
            [i.to_bytes(length=1, byteorder='big') for i in ffi.unpack(digest_value, digest_length[0])]).hex()
    else:
        raise ValueError("Checksum failed!")


if sha1(b'===randomstring===') != '===randomstringsha1===':
    raise ValueError("Didn't pass checksum1!")

if sys.platform in ("linux", "darwin"):
    import shared_atomic.shared_atomic_
    from shared_atomic.shared_atomic_ import lib
    from shared_atomic.shared_atomic_ import ffi

    with open(shared_atomic.shared_atomic_.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomiclibsha1===':
            raise ValueError("Didn't pass checksum2!")

    import shared_atomic.atomic_object
    with open(shared_atomic.atomic_object.__file__, 'rb') as f:
        shared_atomic_bytes = f.read()
        if sha1(shared_atomic_bytes) != '===atomic_object_libsha1===':
            raise ValueError("Didn't pass checksum3!")

    from shared_atomic.atomic_object import *
    from shared_atomic import mmap_flags

else:
    raise NotImplementedError("Unsupported platform!")

self_atomic_uint = TypeVar("self_atomic_uint", bound="atomic_uint")

class atomic_uint(atomic_object):
    """Unsigned int provide atomic operations, the unsigned int takes 8 bytes,
        no such class on windows x64, windows_unix_compatibility is useless
        :attributes
            reference: readonly pointer to the integer
            value: read/write value of the integer
            mode: readonly 's' for single process, 'm' for multiprocessing, on windows platform, only singleprocessing is supported
            windows_unix_compatibility: readonly whether the source code should be compatible with windows x64
    """
    mode = cython.declare(str, visibility='readonly')
    windows_unix_compatibility = cython.declare(cython.bint, visibility='readonly')

    def __init__(self, value: cython.size_t, mode: str = 'singleprocessing',
                 windows_unix_compatibility: bool = False):
        super().__init__()
        self.size = 8
        if mode in ('s', 'singleprocessing'):
            self.mode = 's'
            if sys.platform in ('darwin', 'linux'):
                self.reference = ffi.new("size_t *")
                self.set(value)
        elif mode in ('m', 'multiprocessing') and sys.platform in ('darwin', 'linux'):
            self.mode = 'm'
            void_pointer = lib.mmap(ffi.NULL, 8, 3, mmap_flags, -1, 0)
            self.reference = ffi.cast("size_t *", void_pointer)
            self.set(value)
        else:
            raise NotImplementedError("Unsupported mode!")

    def __del__(self):
        if self.mode == 'm':
            lib.munmap(self.reference, 8)

    def get(self) -> cython.size_t:
        return lib.size_t_load(self.reference)

    def set(self, value: cython.size_t):
        lib.size_t_get_and_set(self.reference, value)

    def uint_store(self, n: self_atomic_uint):
        """Store value atomically

         :param v: the pointer to set
         :param n: the pointer from value to set
         :return: None
         """
        lib.size_t_store(self.reference, n.reference)

    def uint_shift(self, n: self_atomic_uint, r: self_atomic_uint):
        """value exchange between 3 pointers in 2 groups atomically, store n in v after store v in r

        :param v: pointer of v
        :param n: pointer of n
        :param r: pointer of r
        :return: None
        """
        return lib.size_t_shift(self.reference, n.reference, r.reference)

    def uint_get_and_set(self, n: cython.size_t) -> cython.size_t:
        """get and set atomically

        :param v: pointer of value to get and set
        :param n: value to set
        :return: original value
        """
        return lib.size_t_get_and_set(self.reference, n)

    def uint_compare_and_set(self, e: self_atomic_uint, n: cython.size_t) -> cython.bint:
        """Compare and set atomically. This compares the contents of v
        with the contents of e. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        the operation is a read and the current contents of v are written into
        e.
        :param v: pointer of v
        :param e: pointer of e
        :param n: value to be set
        :return: whether the contents of v and contents of e is the same
        """
        return lib.size_t_compare_and_set(self.reference, e.reference, n)

    def uint_compare_and_set_value(self, e: cython.size_t, n: cython.size_t) -> cython.size_t:
        """Compare and swap atomically, This compares the contents of self
        with the contents of n. If equal, the operation is a read-modify-write
        operation that writes n into self. If they are not equal,
        No operation is performed.
                :param v: A pointer to the destination value
                :param e: The exchange value
                :param n: The value to compare to v
                :return: initial value of the v
                """
        return lib.size_t_compare_and_set_value(self.reference, e, n)

    def uint_add_and_fetch(self, n: cython.size_t) -> cython.size_t:
        """increment and fetch atomically

        :param v: pointer of value to add to and get
        :param n: data to add
        :return: sum of the 2 values
        """
        return lib.size_t_add_and_fetch(self.reference, n)

    def uint_sub_and_fetch(self, n: cython.size_t) -> cython.size_t:
        """sub and fetch atomically

        :param self of value to subtract and get
        :param n: data to subtract
        :return: difference of the 2 values
        """
        return lib.size_t_sub_and_fetch(self.reference, n)

    def uint_and_and_fetch(self, n: cython.size_t) -> cython.size_t:
        """bitwise AND and fetch the result atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: resulted value
        """
        return lib.size_t_and_and_fetch(self.reference, n)

    def uint_or_and_fetch(self, n: cython.size_t) -> cython.size_t:
        """bitwise XOR and fetch the result atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: resulted value
        """
        return lib.size_t_or_and_fetch(self.reference, n)

    def uint_xor_and_fetch(self, n: cython.size_t) -> cython.size_t:
        """bitwise XOR and fetch the result atomically

        :param self of value to XOR to
        :param n: data to XOR
        :return: resulted value
        """
        return lib.size_t_xor_and_fetch(self.reference, n)

    def uint_nand_and_fetch(self, n: cython.size_t) -> cython.size_t:
        """bitwise NAND and fetch the result atomically

        :param v: pointer of value to NAND to
        :param n: data to NAND
        :return: resulted value
        """
        return lib.size_t_nand_and_fetch(self.reference, n)

    def uint_fetch_and_add(self, n: cython.size_t) -> cython.size_t:
        """fetch and increment atomically

        :param v: pointer of value to add to
        :param n: data to add
        :return: original value in v
        """
        return lib.size_t_fetch_and_add(self.reference, n)

    def uint_fetch_and_sub(self, n: cython.size_t) -> cython.size_t:
        """fetch and subtract  atomically

        :param v: pointer of value to subtract from
        :param n: data to subtract
        :return: original value in v
        """
        return lib.size_t_fetch_and_sub(self.reference, n)

    def uint_fetch_and_and(self, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise AND atomically

        :param v: pointer of value to AND to
        :param n: data to AND
        :return: the result
        """
        return lib.size_t_fetch_and_and(self.reference, n)

    def uint_fetch_and_or(self, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise OR atomically

        :param v: pointer of value to OR to
        :param n: data to OR
        :return: the result
        """
        return lib.size_t_fetch_and_or(self.reference, n)

    def uint_fetch_and_xor(self, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise XOR atomically

        :param v: pointer of value to XOR to
        :param n: data to XOR
        :return: the result
        """
        return lib.size_t_fetch_and_xor(self.reference, n)

    def uint_fetch_and_nand(self, n: cython.size_t) -> cython.size_t:
        """fetch then bitwise NAND atomically

        :param v: pointer of value to NAND to
        :param n: data to NAND
        :return: the result
        """
        return lib.size_t_fetch_and_nand(self.reference, n)

    value = property(fget=get, fset=set, doc="get and set the unsigned integer.")
